@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # Copyright (C) 2008 Noah Kantrowitz <noah@coderanger.net>
 # All rights reserved.
 #
@@ -31,6 +31,12 @@ from datetime import datetime
 
 from trac.util import get_pkginfo
 
+try:
+    from pygments.lexers.templates import DjangoLexer
+    html_jinja_lexer = DjangoLexer()
+except ImportError:
+    html_jinja_lexer = None
+
 pkg_info = get_pkginfo(sys.modules['trac'])
 
 # General substitutions.
@@ -68,6 +74,7 @@ extensions = []
 
 extensions.append('sphinx.ext.autodoc')
 
+autodoc_default_flags = ['members', 'show-inheritance']
 autoclass_content = 'both'
 autodoc_member_order = 'bysource'
 
@@ -368,3 +375,5 @@ def setup(app):
 
     # ifconfig variables
     app.add_config_value('devel', True, '')
+    if html_jinja_lexer:
+        app.add_lexer('html+jinja', html_jinja_lexer)
