@@ -1,22 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # Copyright (C) 2007 Alec Thomas <alec@swapoff.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 from trac.core import *
 from trac.perm import IPermissionPolicy, IPermissionRequestor
 
-revision = "$Rev: 12164 $"
-url = "$URL: http://svn.edgewall.org/repos/trac/tags/trac-1.0.4/sample-plugins/permissions/vulnerability_tickets.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 class SecurityTicketsPolicy(Component):
     """Prevent public access to security sensitive tickets.
