@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2020 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -14,9 +14,8 @@
 
 """Sample Wiki syntax extension plugin."""
 
-from genshi.builder import tag
-
 from trac.core import *
+from trac.util.html import tag
 from trac.util.text import shorten_line
 from trac.versioncontrol.api import NoSuchChangeset, RepositoryManager
 from trac.versioncontrol.web_ui import ChangesetModule
