@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2020 Edgewall Software
+# Copyright (C) 2009-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -119,8 +119,8 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
                                      'disabled')
             self._tester.go_to_admin("Basic Settings")
             tc.find(r'<option value="TimelineModule">TimelineModule</option>')
-            tc.find(r'<span class="hint">TimelineModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*TimelineModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.go(self._tester.url)
             tc.find(r'<h1>Configuration Error</h1>')
             tc.find(r'Cannot find an implementation of the '
@@ -137,14 +137,14 @@ class TestDefaultHandler(FunctionalTwillTestCaseSetup):
             self._tester.go_to_admin("Basic Settings")
             tc.find(r'<option value="BatchModifyModule">BatchModifyModule'
                     r'</option>')
-            tc.find(r'<span class="hint">BatchModifyModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*BatchModifyModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.formvalue('modbasic', 'default_handler', 'BatchModifyModule')
             tc.submit()  # Invalid value should not be replaced on submit
             tc.find(r'<option value="BatchModifyModule">BatchModifyModule'
                     r'</option>')
-            tc.find(r'<span class="hint">BatchModifyModule is not a valid '
-                    r'IRequestHandler or is not enabled.</span>')
+            tc.find(r'<span class="hint">\s*BatchModifyModule is not a valid '
+                    r'IRequestHandler or is not enabled.\s*</span>')
             tc.go(self._tester.url)
             tc.find(r'<h1>Configuration Error</h1>')
             tc.find(r'<code>BatchModifyModule</code> is not a valid default '
@@ -161,7 +161,7 @@ class TestLoggingNone(FunctionalTwillTestCaseSetup):
         tc.find('trac.log')
         tc.formvalue('modlog', 'log_type', 'none')
         tc.submit()
-        tc.find('selected="selected">None</option')
+        tc.find('selected="selected" value="none">None</option')
 
 
 class TestLoggingAuthorization(AuthorizationTestCaseSetup):
@@ -181,7 +181,7 @@ class TestLoggingToFile(FunctionalTwillTestCaseSetup):
         tc.formvalue('modlog', 'log_file', 'trac.log2')
         tc.formvalue('modlog', 'log_level', 'INFO')
         tc.submit()
-        tc.find('selected="selected">File</option')
+        tc.find('selected="selected" value="file">File</option')
         tc.find('id="log_file".*value="trac.log2"')
         tc.find('selected="selected">INFO</option>')
 
@@ -195,7 +195,7 @@ class TestLoggingToFileNormal(FunctionalTwillTestCaseSetup):
         tc.formvalue('modlog', 'log_file', 'trac.log')
         tc.formvalue('modlog', 'log_level', 'DEBUG')
         tc.submit()
-        tc.find('selected="selected">File</option')
+        tc.find('selected="selected" value="file">File</option')
         tc.find('id="log_file".*value="trac.log"')
         tc.find('selected="selected">DEBUG</option>')
 
@@ -221,6 +221,20 @@ class TestCreatePermissionGroup(FunctionalTwillTestCaseSetup):
         tc.find('%s:%s' % (somegroup, REPORT_CREATE))
 
 
+class TestRemovePermissionGroup(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Remove a permissions group"""
+        self._tester.go_to_admin("Permissions")
+        tc.find('Manage Permissions')
+        somegroup = unicode_to_base64('somegroup')
+        REPORT_CREATE = unicode_to_base64('REPORT_CREATE')
+        tc.find('%s:%s' % (somegroup, REPORT_CREATE))
+        tc.formvalue('revokeform', 'sel', '%s:%s' % (somegroup, REPORT_CREATE))
+        tc.submit()
+        tc.notfind('%s:%s' % (somegroup, REPORT_CREATE))
+        tc.notfind(somegroup)
+
+
 class TestAddUserToGroup(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Add a user to a permissions group"""
@@ -247,9 +261,9 @@ class TestAddUserToGroup(FunctionalTwillTestCaseSetup):
             tc.formvalue('addsubj', 'sg_group', 'authenticated')
             tc.submit()
             tc.find("The subject someuser was not added to the "
-                    "group authenticated. The group has TICKET_CHGPROP "
-                    "permission and you cannot grant permissions you "
-                    "don't possess.")
+                    "group authenticated because the group has "
+                    "TICKET_CREATE permission and users cannot "
+                    "grant permissions they don&#39;t possess.")
         finally:
             self._tester.login('admin')
             self._tester.go_to_admin("Permissions")
@@ -271,20 +285,6 @@ class TestRemoveUserFromGroup(FunctionalTwillTestCaseSetup):
         tc.notfind('%s:%s' % (authenticated, somegroup))
 
 
-class TestRemovePermissionGroup(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Remove a permissions group"""
-        self._tester.go_to_admin("Permissions")
-        tc.find('Manage Permissions')
-        somegroup = unicode_to_base64('somegroup')
-        REPORT_CREATE = unicode_to_base64('REPORT_CREATE')
-        tc.find('%s:%s' % (somegroup, REPORT_CREATE))
-        tc.formvalue('revokeform', 'sel', '%s:%s' % (somegroup, REPORT_CREATE))
-        tc.submit()
-        tc.notfind('%s:%s' % (somegroup, REPORT_CREATE))
-        tc.notfind(somegroup)
-
-
 class TestCopyPermissions(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Tests for the Copy Permissions functionality
@@ -523,7 +523,7 @@ class RegressionTestTicket11257(FunctionalTwillTestCaseSetup):
         catalog_hint = "Message catalogs have not been compiled."
         language_select = '<select name="default_language">'
         disabled_language_select = \
-            '<select name="default_language" disabled="disabled" ' \
+            '<select disabled="disabled" name="default_language" ' \
             'title="Translations are currently unavailable">'
 
         self._tester.go_to_admin("Basic Settings")
@@ -556,9 +556,9 @@ def functionalSuite(suite=None):
     suite.addTest(TestLoggingToFileNormal())
     suite.addTest(TestPermissionsAuthorization())
     suite.addTest(TestCreatePermissionGroup())
+    suite.addTest(TestRemovePermissionGroup())
     suite.addTest(TestAddUserToGroup())
     suite.addTest(TestRemoveUserFromGroup())
-    suite.addTest(TestRemovePermissionGroup())
     suite.addTest(TestCopyPermissions())
     suite.addTest(TestPluginSettings())
     suite.addTest(TestPluginsAuthorization())
