@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,22 +11,40 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os
-import tempfile
+import sys
 import unittest
 
 from trac.config import ConfigurationError
 from trac.db.api import DatabaseManager
 from trac.db.schema import Column, Index, Table
 from trac.env import Environment
-from trac.test import EnvironmentStub, MockRequest, get_dburi, rmtree
+from trac.test import EnvironmentStub, MockRequest, get_dburi, mkdtemp, rmtree
 from trac.util import translation
 
 
 class DatabaseFileTestCase(unittest.TestCase):
 
+    stdout = None
+    stderr = None
+    devnull = None
+
+    @classmethod
+    def setUpClass(cls):
+        cls.stdout = sys.stdout
+        cls.stderr = sys.stderr
+        cls.devnull = io.open(os.devnull, 'wb')
+        sys.stdout = sys.stderr = cls.devnull
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.devnull.close()
+        sys.stdout = cls.stdout
+        sys.stderr = cls.stderr
+
     def setUp(self):
-        self.env_path = tempfile.mkdtemp(prefix='trac-tempenv-')
+        self.env_path = mkdtemp()
         self.db_path = os.path.join(self.env_path, 'db', 'trac.db')
         self.env = None
 
@@ -56,7 +74,7 @@ class DatabaseFileTestCase(unittest.TestCase):
 
     def test_no_permissions(self):
         self._create_env()
-        os.chmod(self.db_path, 0444)
+        os.chmod(self.db_path, 0o444)
         self.env = Environment(self.env_path)
         try:
             self._db_query(self.env)
@@ -137,7 +155,7 @@ class SQLiteConnectionTestCase(unittest.TestCase):
         with self.env.db_query as db:
             cursor = db.cursor()
             cursor.execute("PRAGMA index_list(%s)" % db.quote(table))
-            results = dict((row[1], {'unique': row[2]}) for row in cursor)
+            results = {row[1]: {'unique': row[2]} for row in cursor}
             for index, info in results.iteritems():
                 cursor.execute("PRAGMA index_info(%s)" % db.quote(index))
                 info['columns'] = [row[2] for row in cursor]
