@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,9 +14,6 @@
 import os
 import unittest
 
-from genshi.core import Stream
-from genshi.input import HTMLParser, XML
-
 from trac.mimeview.api import Mimeview
 from trac.mimeview.patch import PatchRenderer
 from trac.test import EnvironmentStub, MockRequest
@@ -30,17 +27,23 @@ class PatchRendererTestCase(unittest.TestCase):
         req = MockRequest(env)
         self.context = web_context(req)
         self.patch = Mimeview(env).renderers[0]
-        patch_html = open(os.path.join(os.path.split(__file__)[0],
-                                       'patch.html'))
-        self.patch_html = Stream(list(HTMLParser(patch_html, encoding='utf-8')))
+        html_file = os.path.join(os.path.dirname(__file__), 'patch.data')
+        self.patch_html = {}
+        testcase = []
+        with open(html_file, 'rb') as f:
+            for line in f.readlines():
+                if line.startswith('#'):
+                    self.patch_html[line[1:].strip()] = testcase = []
+                else:
+                    testcase.append(line.rstrip())
 
     def _expected(self, expected_id):
-        return self.patch_html.select('//div[@id="%s"]/div' % expected_id)
+        return self.patch_html[expected_id]
 
     def _test(self, expected_id, result):
-        expected = self._expected(expected_id).render(encoding='utf-8')
-        result = XML(result.render(encoding='utf-8')).render(encoding='utf-8')
-        expected, result = expected.splitlines(), result.splitlines()
+        expected = self._expected(expected_id)
+        result = result.splitlines()
+        self.maxDiff = None
         for exp, res in zip(expected, result):
             self.assertEqual(exp, res)
         self.assertEqual(len(expected), len(result))
