@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2017-2020 Edgewall Software
+# Copyright (C) 2017-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,6 +16,7 @@ from xml.etree import ElementTree
 
 from trac.test import EnvironmentStub, MockRequest
 from trac.mimeview.txtl import TextileRenderer, has_textile
+from trac.util.html import Markup
 from trac.web.chrome import web_context
 
 
@@ -33,7 +34,7 @@ class TextileRendererTestCase(unittest.TestCase):
 
     def _render(self, text):
         result = self.renderer.render(self.context, 'textile', text)
-        self.assertEqual(unicode, type(result))
+        self.assertEqual(Markup, type(result))
         return result
 
     def _parse_xml(self, source):
@@ -87,9 +88,8 @@ class TextileRendererTestCase(unittest.TestCase):
             <span style="background-image:url(http://example.org/foo.png)">unsafe</span>
             """))
         self.assertNotIn('href="ftp://', result)
-        self.assertNotIn('<img src="//example.org/foo.png" />', result)
-        self.assertIn('<img src="//example.org/foo.png" '
-                      'crossorigin="anonymous" />', result)
+        self.assertIn(
+            '<img src="//example.org/foo.png" crossorigin="anonymous"', result)
         self.assertNotIn('url(http://example.org/foo.png)', result)
         self.assertIn('<span>unsafe</span>', result)
 
