@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
-from . import api, mail, model
+from . import api, mail, model, prefs
 
 
 def test_suite():
@@ -21,6 +21,7 @@ def test_suite():
     suite.addTest(api.test_suite())
     suite.addTest(mail.test_suite())
     suite.addTest(model.test_suite())
+    suite.addTest(prefs.test_suite())
     return suite
 
 
