@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,7 +14,6 @@
 import unittest
 from email import message_from_string
 
-import trac.tests.compat
 from trac.core import Component, implements
 from trac.notification.api import (
     IEmailAddressResolver, IEmailSender, INotificationFormatter,
@@ -196,9 +195,9 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'baz@example.net', 'qux@example.net',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'baz@example.net', 'qux@example.net',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self._assert_mail(message, 'text/plain', 'blah')
 
@@ -217,7 +216,7 @@ class EmailDistributorTestCase(unittest.TestCase):
                 self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
             if 'cc@example.org' in recipients:
                 self.assertEqual('trac@example.org', from_addr)
-                self.assertEqual(set(('cc@example.org', 'bcc@example.org')),
+                self.assertEqual({'cc@example.org', 'bcc@example.org'},
                                  set(recipients))
                 self._assert_mail(message, 'text/plain', 'blah')
 
@@ -232,9 +231,9 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(2, len(history))
         for from_addr, recipients, message in history:
             if 'foo@example.org' in recipients:
-                self.assertEqual(set(('foo@example.org', 'cc@example.org',
-                                      'bcc@example.org')),
-                                 set(recipients))
+                self.assertEqual(
+                    {'foo@example.org', 'cc@example.org', 'bcc@example.org'},
+                    set(recipients))
                 self._assert_mail(message, 'text/plain', 'blah')
             if 'bar@example.org' in recipients:
                 self.assertEqual(['bar@example.org'], recipients)
@@ -301,8 +300,8 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self._assert_mail(message, 'text/plain', 'raise-text-html')
 
@@ -327,11 +326,11 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual(1, len(history))
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
-        self.assertEqual(set(('foo@example.org', 'bar@example.org',
-                              'cc@example.org', 'bcc@example.org')),
+        self.assertEqual({'foo@example.org', 'bar@example.org',
+                          'cc@example.org', 'bcc@example.org'},
                          set(recipients))
         self.assertEqual('cc@example.org, foo@example.org', message['Cc'])
-        self.assertEqual(None, message['Bcc'])
+        self.assertIsNone(message['Bcc'])
         self._assert_mail(message, 'text/plain', 'blah')
 
     def test_from_author_disabled(self):
@@ -500,9 +499,9 @@ class EmailDistributorTestCase(unittest.TestCase):
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
         self.assertEqual('"My Project" <trac@example.org>', message['From'])
-        self.assertEqual(set(['foo@example.com', 'foo@bar.example.org',
-                              'baz@example.com', 'cc@example.org',
-                              'bcc@example.org']), set(recipients))
+        self.assertEqual({'foo@example.com', 'foo@bar.example.org',
+                          'baz@example.com', 'cc@example.org',
+                          'bcc@example.org'}, set(recipients))
         self._assert_equal_sets(['cc@example.org'],
                                 self._cclist(message['Cc']))
 
