@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,13 +12,12 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import os
-import tempfile
 import unittest
 
 from trac.core import Component, TracValueError, implements
 from trac.prefs.api import IPreferencePanelProvider
 from trac.prefs.web_ui import PreferencesModule
-from trac.test import EnvironmentStub, MockRequest
+from trac.test import EnvironmentStub, MockRequest, mkdtemp
 from trac.util import create_file
 from trac.util.html import Markup
 from trac.web.api import RequestDone
@@ -50,7 +49,8 @@ class AdvancedPreferencePanelTestCase(unittest.TestCase):
         self.assertEqual(name, req.session.get('name'))
         self.assertEqual(email, req.session.get('email'))
         self.assertTrue(module.match_request(req))
-        self.assertRaises(RequestDone, module.process_request, req)
+        with self.assertRaises(RequestDone):
+            module.process_request(req)
         self.assertIn("Your preferences have been saved.",
                       req.chrome['notices'])
         self.assertEqual(new_sid, req.session.sid)
@@ -67,12 +67,10 @@ class AdvancedPreferencePanelTestCase(unittest.TestCase):
         module = PreferencesModule(self.env)
 
         self.assertTrue(module.match_request(req))
-        try:
+        with self.assertRaises(TracValueError) as cm:
             module.process_request(req)
-        except TracValueError as e:
-            self.assertEqual("Session ID must be alphanumeric.", unicode(e))
-        else:
-            self.fail("TracValueError not raised.")
+        self.assertEqual("Session ID must be alphanumeric.",
+                         unicode(cm.exception))
 
     def test_load_session_key(self):
         """Load session key."""
@@ -104,27 +102,64 @@ class AdvancedPreferencePanelTestCase(unittest.TestCase):
         module = PreferencesModule(self.env)
 
         self.assertTrue(module.match_request(req))
-        try:
+        with self.assertRaises(TracValueError) as cm:
             module.process_request(req)
-        except TracValueError as e:
-            self.assertEqual("Session ID must be alphanumeric.", unicode(e))
-        else:
-            self.fail("TracValueError not raised.")
+        self.assertEqual("Session ID must be alphanumeric.",
+                         unicode(cm.exception))
+
+
+class UserInterfacePreferencePanelTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+
+    def tearDown(self):
+        self.env.reset_db()
+
+    def _test_auto_preview_timeout(self, timeout):
+        req = MockRequest(self.env, method='POST',
+                          path_info='/prefs/userinterface',
+                          cookie='trac_session=1234567890abcdef;',
+                          args={'ui.auto_preview_timeout': timeout})
+        module = PreferencesModule(self.env)
+
+        self.assertTrue(module.match_request(req))
+        with self.assertRaises(RequestDone) as cm:
+            module.process_request(req)
+
+        self.assertEqual("Your preferences have been saved.",
+                         req.session['chrome.notices.0'])
+        return req
+
+    def test_save_auto_preview_timeout(self):
+        timeout = '3'
+        req = self._test_auto_preview_timeout(timeout)
+
+        self.assertEqual(timeout, req.session['ui.auto_preview_timeout'])
+
+    def test_auto_preview_timeout_value_invalid(self):
+        timeout = '3'
+        req = self._test_auto_preview_timeout(timeout)
+        req.session.save()
+        req = self._test_auto_preview_timeout('A')
+
+        self.assertEqual('Discarded invalid value "A" for auto preview '
+                         'timeout.', req.session['chrome.warnings.0'])
+        self.assertEqual(timeout, req.session['ui.auto_preview_timeout'])
+
+    def test_delete_auto_preview_timeout(self):
+        self._test_auto_preview_timeout('1')
+        req = self._test_auto_preview_timeout('')
+
+        self.assertNotIn('ui.auto_preview_timeout', req.session)
 
 
 class PreferencesModuleTestCase(unittest.TestCase):
 
-    def setUp(self):
-        self.env = EnvironmentStub(
-            path=tempfile.mkdtemp(prefix='trac-tempenv-'))
-        self.templates_dir = os.path.join(self.env.path, 'templates')
-        os.mkdir(self.templates_dir)
-
-    def tearDown(self):
-        self.env.reset_db_and_disk()
-
-    def test_render_child_preferences_panels(self):
+    panel_providers = []
 
+    @classmethod
+    def setUpClass(cls):
         class ParentPanel(Component):
             implements(IPreferencePanelProvider)
 
@@ -134,8 +169,8 @@ class PreferencesModuleTestCase(unittest.TestCase):
             def render_preference_panel(self, req, panel):
                 return 'prefs_panel_1.html', {}, None
 
-        child_template = '<h2>${title}</h2>'
-        child1_template_name = 'prefs_child_1.html'
+        cls.child_template = '<h2>${title}</h2>'
+        cls.child1_template_name = 'prefs_child_1.html'
 
         class Child1Panel(Component):
             implements(IPreferencePanelProvider)
@@ -144,9 +179,9 @@ class PreferencesModuleTestCase(unittest.TestCase):
                 yield 'child1', 'Child 1', 'panel1'
 
             def render_preference_panel(self, req, panel):
-                return child1_template_name, {'title': 'Child 1'}
+                return cls.child1_template_name, {'title': 'Child 1'}
 
-        child2_template_name = 'prefs_child_2.html'
+        cls.child2_template_name = 'prefs_child_2.html'
 
         class Child2Panel(Component):
             implements(IPreferencePanelProvider)
@@ -155,12 +190,29 @@ class PreferencesModuleTestCase(unittest.TestCase):
                 yield 'child2', 'Child 2', 'panel1'
 
             def render_preference_panel(self, req, panel):
-                return child2_template_name, {'title': 'Child 2'}
+                return cls.child2_template_name, {'title': 'Child 2'}
 
-        create_file(os.path.join(self.templates_dir, child1_template_name),
-                    child_template)
-        create_file(os.path.join(self.templates_dir, child2_template_name),
-                    child_template)
+        cls.panel_providers = [ParentPanel, Child1Panel, Child2Panel]
+
+    @classmethod
+    def tearDownClass(cls):
+        from trac.core import ComponentMeta
+        for component in cls.panel_providers:
+            ComponentMeta.deregister(component)
+
+    def setUp(self):
+        self.env = EnvironmentStub(path=mkdtemp())
+        self.templates_dir = os.path.join(self.env.path, 'templates')
+        os.mkdir(self.templates_dir)
+
+    def tearDown(self):
+        self.env.reset_db_and_disk()
+
+    def test_render_child_preferences_panels(self):
+        create_file(os.path.join(self.templates_dir, self.child1_template_name),
+                    self.child_template)
+        create_file(os.path.join(self.templates_dir, self.child2_template_name),
+                    self.child_template)
 
         req = MockRequest(self.env, path_info='/prefs/panel1')
         mod = PreferencesModule(self.env)
@@ -179,6 +231,7 @@ class PreferencesModuleTestCase(unittest.TestCase):
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(AdvancedPreferencePanelTestCase))
+    suite.addTest(unittest.makeSuite(UserInterfacePreferencePanelTestCase))
     suite.addTest(unittest.makeSuite(PreferencesModuleTestCase))
     return suite
 
