@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -18,6 +18,7 @@ import unittest
 from trac.search.web_ui import SearchModule
 from trac.test import EnvironmentStub, MockRequest
 from trac.ticket.model import Ticket
+from trac.ticket.test import insert_ticket
 from trac.ticket.web_ui import TicketModule
 from trac.wiki.admin import WikiAdmin
 from trac.wiki.web_ui import WikiModule
@@ -40,13 +41,10 @@ class SearchModuleTestCase(unittest.TestCase):
 
     def _insert_ticket(self, **kw):
         """Helper for inserting a ticket into the database"""
-        ticket = Ticket(self.env)
-        for k, v in kw.items():
-            ticket[k] = v
-        return ticket.insert()
+        return insert_ticket(self.env, **kw)
 
     def test_process_request_page_in_range(self):
-        for _ in range(0, 21):
+        for _ in xrange(21):
             self._insert_ticket(summary="Trac")
         req = MockRequest(self.env,
                           args={'page': '3', 'q': 'Trac', 'ticket': 'on'})
@@ -58,7 +56,7 @@ class SearchModuleTestCase(unittest.TestCase):
 
     def test_process_request_page_out_of_range(self):
         """Out of range value for page defaults to page 1."""
-        for _ in range(0, 20):
+        for _ in xrange(20):
             self._insert_ticket(summary="Trac")
         req = MockRequest(self.env,
                           args={'page': '3', 'q': 'Trac', 'ticket': 'on'})
