@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -13,16 +13,17 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import contextlib
+import copy
 import os
-import tempfile
 import time
 import unittest
 
-import trac.tests.compat
+from trac.admin.console import TracAdmin
+from trac.admin.test import TracAdminTestCaseBase
 from trac.config import *
 from trac.config import UnicodeConfigParser
 from trac.core import Component, ComponentMeta, Interface, implements
-from trac.test import Configuration, EnvironmentStub, rmtree
+from trac.test import Configuration, EnvironmentStub, mkdtemp, rmtree
 from trac.util import create_file, read_file
 from trac.util.compat import wait_for_file_mtime_change
 from trac.util.datefmt import time_now
@@ -45,7 +46,7 @@ def readlines(filename):
 class UnicodeParserTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.tempdir = tempfile.mkdtemp()
+        self.tempdir = mkdtemp()
         self.filename = os.path.join(self.tempdir, 'config.ini')
         _write(self.filename, [
             u'[ä]', u'öption = ÿ',
@@ -161,9 +162,9 @@ class UnicodeParserTestCase(unittest.TestCase):
 class BaseTestCase(unittest.TestCase):
 
     def setUp(self):
-        tmpdir = os.path.realpath(tempfile.gettempdir())
-        self.filename = os.path.join(tmpdir, 'trac-test.ini')
-        self.sitename = os.path.join(tmpdir, 'trac-site.ini')
+        self.tmpdir = mkdtemp()
+        self.filename = os.path.join(self.tmpdir, 'trac-test.ini')
+        self.sitename = os.path.join(self.tmpdir, 'trac-site.ini')
         self.env = EnvironmentStub()
         self._write([])
         self._orig = {
@@ -173,9 +174,9 @@ class BaseTestCase(unittest.TestCase):
             'Option.registry': Option.registry,
         }
         ComponentMeta._components = list(ComponentMeta._components)
-        ComponentMeta._registry = dict((interface, list(classes))
-                                       for interface, classes
-                                       in ComponentMeta._registry.iteritems())
+        ComponentMeta._registry = {interface: list(classes)
+                                   for interface, classes
+                                   in ComponentMeta._registry.iteritems()}
         ConfigSection.registry = {}
         Option.registry = {}
 
@@ -184,9 +185,7 @@ class BaseTestCase(unittest.TestCase):
         ComponentMeta._registry = self._orig['ComponentMeta._registry']
         ConfigSection.registry = self._orig['ConfigSection.registry']
         Option.registry = self._orig['Option.registry']
-        os.remove(self.filename)
-        if os.path.exists(self.sitename):
-            os.remove(self.sitename)
+        rmtree(self.tmpdir)
 
     def _read(self):
         return Configuration(self.filename)
@@ -253,15 +252,15 @@ class ConfigurationTestCase(unittest.TestCase):
 
     def test_contains_from_config(self):
         """Contains returns `True` for section defined in config."""
-        self.assertTrue(u'séction1' in self.config)
+        self.assertIn(u'séction1', self.config)
 
     def test_contains_from_inherited(self):
         """Contains returns `True` for section defined in inherited config."""
-        self.assertTrue(u'séction2' in self.config)
+        self.assertIn(u'séction2', self.config)
 
     def test_contains_from_default(self):
         """Contains returns `True` for section defined in an option."""
-        self.assertTrue(u'séction3' in self.config)
+        self.assertIn(u'séction3', self.config)
 
     def test_sections_with_default(self):
         """Sections including defaults."""
@@ -372,14 +371,14 @@ class IntegrationTestCase(BaseTestCase):
             option_a = PathOption('a', 'opt1', 'file.ini')
             option_b = PathOption('a', 'opt2', '/somewhere/file.ini')
         self.assertEqual('file.ini', config.get('a', 'opt1'))
-        self.assertNotEquals('file.ini', config.getpath('a', 'opt1'))
+        self.assertNotEqual('file.ini', config.getpath('a', 'opt1'))
         self.assertTrue(os.path.isabs(config.getpath('a', 'opt1')))
         self.assertEqual('/somewhere/file.ini', os.path.splitdrive(
                          config.getpath('a', 'opt2'))[1].replace('\\', '/'))
         self.assertEqual('/none.ini', os.path.splitdrive(
                          config.getpath('a', 'opt3',
                                         '/none.ini'))[1].replace('\\', '/'))
-        self.assertNotEquals('none.ini', config.getpath('a', 'opt3', 'none.ini'))
+        self.assertNotEqual('none.ini', config.getpath('a', 'opt3', 'none.ini'))
 
     def test_read_and_get(self):
         self._write(['[a]', 'option = x'])
@@ -463,9 +462,8 @@ class IntegrationTestCase(BaseTestCase):
 
     def test_read_and_getlist_false_values(self):
         config = self._read()
-        values = [None, False, '', 'foo', u'', u'bar',
-                  0, 0L, 0.0, 0j, 42, 43.0]
-        self.assertEqual([False, 'foo', u'bar', 0, 0L, 0.0, 0j, 42, 43.0],
+        values = [None, False, '', 'foo', u'', u'bar', 0, 0.0, 0j, 42, 43.0]
+        self.assertEqual([False, 'foo', u'bar', 0, 0.0, 0j, 42, 43.0],
                          config.getlist('a', 'false', values))
         self.assertEqual(values, config.getlist('a', 'false', values,
                                                 keep_empty=True))
@@ -624,7 +622,7 @@ class IntegrationTestCase(BaseTestCase):
         config.set('b', u'öption0', 'y')
         config.set(u'aä', 'öption0', 'x')
         config.set('aä', 'option2', "Voilà l'été")  # UTF-8
-        config.set(u'aä', 'option1', u"Voilà l'été") # unicode
+        config.set(u'aä', 'option1', u"Voilà l'été")  # unicode
         section = config['b']
         section.set('option1', None)
         section = config[u'aä']
@@ -665,7 +663,7 @@ class IntegrationTestCase(BaseTestCase):
             self._write(['[a]', 'option = x'], site=True)
             config = self._read()
             config.set('a', 'option2', "Voilà l'été")  # UTF-8
-            config.set('a', 'option1', u"Voilà l'été") # unicode
+            config.set('a', 'option1', u"Voilà l'été")  # unicode
             self.assertEqual('x', config.get('a', 'option'))
             self.assertEqual(u"Voilà l'été", config.get('a', 'option1'))
             self.assertEqual(u"Voilà l'été", config.get('a', 'option2'))
@@ -727,14 +725,14 @@ class IntegrationTestCase(BaseTestCase):
     def test_simple_remove(self):
         self._write(['[a]', 'option = x'])
         config = self._read()
-        config.get('a', 'option') # populates the cache
+        config.get('a', 'option')  # populates the cache
         config.set(u'aä', u'öption', u'öne')
         config.remove('a', 'option')
         self.assertEqual('', config.get('a', 'option'))
         config.remove(u'aä', u'öption')
         self.assertEqual('', config.get('aä', 'öption'))
-        config.remove('a', 'option2') # shouldn't fail
-        config.remove('b', 'option2') # shouldn't fail
+        config.remove('a', 'option2')  # shouldn't fail
+        config.remove('b', 'option2')  # shouldn't fail
 
     def test_sections(self):
         self._write(['[a]', 'option = x', '[b]', 'option = y'])
@@ -764,12 +762,12 @@ class IntegrationTestCase(BaseTestCase):
     def test_options(self):
         self._write(['[a]', 'option = x', '[b]', 'option = y'])
         config = self._read()
-        self.assertEqual(('option', 'x'), iter(config.options('a')).next())
-        self.assertEqual(('option', 'y'), iter(config.options('b')).next())
-        self.assertRaises(StopIteration, iter(config.options('c')).next)
-        self.assertEqual('option', iter(config['a']).next())
-        self.assertEqual('option', iter(config['b']).next())
-        self.assertRaises(StopIteration, iter(config['c']).next)
+        self.assertEqual(('option', 'x'), next(iter(config.options('a'))))
+        self.assertEqual(('option', 'y'), next(iter(config.options('b'))))
+        self.assertRaises(StopIteration, next, iter(config.options('c')))
+        self.assertEqual('option', next(iter(config['a'])))
+        self.assertEqual('option', next(iter(config['b'])))
+        self.assertRaises(StopIteration, next, iter(config['c']))
 
         class Foo(object):
             option_a = Option('a', 'b', 'c')
@@ -780,10 +778,10 @@ class IntegrationTestCase(BaseTestCase):
     def test_options_unicode(self):
         self._write([u'[ä]', u'öption = x', '[b]', 'option = y'])
         config = self._read()
-        self.assertEqual((u'öption', 'x'), iter(config.options(u'ä')).next())
-        self.assertEqual(('option', 'y'), iter(config.options('b')).next())
-        self.assertRaises(StopIteration, iter(config.options('c')).next)
-        self.assertEqual(u'öption', iter(config['ä']).next())
+        self.assertEqual((u'öption', 'x'), next(iter(config.options(u'ä'))))
+        self.assertEqual(('option', 'y'), next(iter(config.options('b'))))
+        self.assertRaises(StopIteration, next, iter(config.options('c')))
+        self.assertEqual(u'öption', next(iter(config['ä'])))
 
         class Foo(object):
             option_a = Option(u'ä', u'öption2', 'c')
@@ -794,11 +792,11 @@ class IntegrationTestCase(BaseTestCase):
     def test_has_option(self):
         config = self._read()
         self.assertFalse(config.has_option('a', 'option'))
-        self.assertFalse('option' in config['a'])
+        self.assertNotIn('option', config['a'])
         self._write(['[a]', 'option = x'])
         config = self._read()
         self.assertTrue(config.has_option('a', 'option'))
-        self.assertTrue('option' in config['a'])
+        self.assertIn('option', config['a'])
 
         class Foo(object):
             option_a = Option('a', 'option2', 'x2')
@@ -808,11 +806,11 @@ class IntegrationTestCase(BaseTestCase):
     def test_has_option_unicode(self):
         config = self._read()
         self.assertFalse(config.has_option(u'ä', u'öption'))
-        self.assertFalse(u'öption' in config[u'ä'])
+        self.assertNotIn(u'öption', config[u'ä'])
         self._write([u'[ä]', u'öption = x'])
         config = self._read()
         self.assertTrue(config.has_option(u'ä', u'öption'))
-        self.assertTrue(u'öption' in config[u'ä'])
+        self.assertIn(u'öption', config[u'ä'])
 
         class Foo(object):
             option_a = Option(u'ä', u'öption2', 'x2')
@@ -844,10 +842,10 @@ class IntegrationTestCase(BaseTestCase):
             config = self._read()
             self.assertEqual('x', config.get('a', 'option'))
             self.assertEqual(['a', 'inherit'], config.sections())
-            config.remove('a', 'option') # Should *not* remove option in parent
+            config.remove('a', 'option')  # Should *not* remove option in parent
             self.assertEqual('x', config.get('a', 'option'))
             self.assertEqual([('option', 'x')], list(config.options('a')))
-            self.assertTrue('a' in config)
+            self.assertIn('a', config)
 
     def test_inherit_multiple(self):
         class Foo(object):
@@ -892,11 +890,11 @@ class IntegrationTestCase(BaseTestCase):
             option_blah = Option('a', 'blah', u'Blàh!')
             option_true = BoolOption('a', 'true', True)
             option_false = BoolOption('a', 'false', False)
-            option_list1 = ListOption('a', 'list', ['#cc0', 4.2, 42L, 0, None,
+            option_list1 = ListOption('a', 'list', ['#cc0', 4.2, 42, 0, None,
                                                     True, False, None],
                                       sep='|', keep_empty=True)
             option_list2 = ListOption('a', 'list-seps',
-                                      ['#cc0', 4.2, 42L, 0, None, True, False,
+                                      ['#cc0', 4.2, 42, 0, None, True, False,
                                        None],
                                       sep=(',', '|'), keep_empty=True)
             option_choice = ChoiceOption('a', 'choice', [-42, 42])
@@ -904,21 +902,20 @@ class IntegrationTestCase(BaseTestCase):
         config = self._read()
         config.set_defaults()
         config.save()
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n',            f.next())
-            self.assertEqual('\n',                                   f.next())
-            self.assertEqual('[a]\n',                                f.next())
-            self.assertEqual('blah = Blàh!\n',                       f.next())
-            self.assertEqual('choice = -42\n',                       f.next())
-            self.assertEqual('false = disabled\n',                   f.next())
-            self.assertEqual('list = #cc0|4.2|42|0||enabled|disabled|\n',
-                             f.next())
-            self.assertEqual('list-seps = #cc0,4.2,42,0,,enabled,disabled,\n',
-                             f.next())
-            self.assertEqual('none = \n',                            f.next())
-            self.assertEqual('true = enabled\n',                     f.next())
-            self.assertEqual('\n',                                   f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[a]\n',
+            'blah = Blàh!\n',
+            'choice = -42\n',
+            'false = disabled\n',
+            'list = #cc0|4.2|42|0||enabled|disabled|\n',
+            'list-seps = #cc0,4.2,42,0,,enabled,disabled,\n',
+            'none = \n',
+            'true = enabled\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_unicode_option_with_raw_default(self):
         class Foo(object):
@@ -927,7 +924,7 @@ class IntegrationTestCase(BaseTestCase):
             option_true = BoolOption(u'résumé', u'trüé', True)
             option_false = BoolOption(u'résumé', u'fálsé', False)
             option_list = ListOption(u'résumé', u'liśt',
-                                     [u'#ccö', 4.2, 42L, 0, None, True,
+                                     [u'#ccö', 4.2, 42, 0, None, True,
                                         False, None],
                                      sep='|', keep_empty=True)
             option_choice = ChoiceOption(u'résumé', u'chöicé', [-42, 42])
@@ -935,19 +932,19 @@ class IntegrationTestCase(BaseTestCase):
         config = self._read()
         config.set_defaults()
         config.save()
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n',            f.next())
-            self.assertEqual('\n',                                   f.next())
-            self.assertEqual('[résumé]\n',                           f.next())
-            self.assertEqual('bláh = Blàh!\n',                       f.next())
-            self.assertEqual('chöicé = -42\n',                       f.next())
-            self.assertEqual('fálsé = disabled\n',                   f.next())
-            self.assertEqual('liśt = #ccö|4.2|42|0||enabled|disabled|\n',
-                             f.next())
-            self.assertEqual('nöné = \n',                            f.next())
-            self.assertEqual('trüé = enabled\n',                     f.next())
-            self.assertEqual('\n',                                   f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[résumé]\n',
+            'bláh = Blàh!\n',
+            'chöicé = -42\n',
+            'fálsé = disabled\n',
+            'liśt = #ccö|4.2|42|0||enabled|disabled|\n',
+            'nöné = \n',
+            'trüé = enabled\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_option_with_non_normal_default(self):
         class Foo(object):
@@ -1008,9 +1005,10 @@ class IntegrationTestCase(BaseTestCase):
 class ConfigurationSetDefaultsTestCase(BaseTestCase):
     """Tests for the `set_defaults` method of the `Configuration` class."""
 
-    def setUp(self):
-        super(ConfigurationSetDefaultsTestCase, self).setUp()
+    components = []
 
+    @classmethod
+    def setUpClass(cls):
         class CompA(Component):
             opt1 = Option('compa', 'opt1', 1)
             opt2 = Option('compa', 'opt2', 'a')
@@ -1019,46 +1017,47 @@ class ConfigurationSetDefaultsTestCase(BaseTestCase):
             opt3 = Option('compb', 'opt3', 2)
             opt4 = Option('compb', 'opt4', 'b')
 
+        cls.components = [CompA, CompB]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.components:
+            ComponentMeta.deregister(component)
+
     def test_component_module_no_match(self):
         """No defaults written if component doesn't match."""
         config = self._read()
         config.set_defaults(component='trac.tests.conf')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        self.assertEqual(['# -*- coding: utf-8 -*-\n', '\n'],
+                         readlines(self.filename))
 
     def test_component_class_no_match(self):
         """No defaults written if module doesn't match."""
         config = self._read()
         config.set_defaults(component='trac.tests.conf.CompC')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        self.assertEqual(['# -*- coding: utf-8 -*-\n', '\n'],
+                         readlines(self.filename))
 
     def test_component_module_match(self):
         """Defaults of components in matching module are written."""
         config = self._read()
         config.set_defaults(component='trac.tests.config')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt1 = 1\n',                f.next())
-            self.assertEqual('opt2 = a\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compb]\n',                 f.next())
-            self.assertEqual('opt3 = 2\n',                f.next())
-            self.assertEqual('opt4 = b\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt1 = 1\n',
+            'opt2 = a\n',
+            '\n',
+            '[compb]\n',
+            'opt3 = 2\n',
+            'opt4 = b\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_component_module_wildcard_match(self):
         """Defaults of components in matching module are written.
@@ -1067,34 +1066,34 @@ class ConfigurationSetDefaultsTestCase(BaseTestCase):
         config = self._read()
         config.set_defaults(component='trac.tests.config.*')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt1 = 1\n',                f.next())
-            self.assertEqual('opt2 = a\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compb]\n',                 f.next())
-            self.assertEqual('opt3 = 2\n',                f.next())
-            self.assertEqual('opt4 = b\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt1 = 1\n',
+            'opt2 = a\n',
+            '\n',
+            '[compb]\n',
+            'opt3 = 2\n',
+            'opt4 = b\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_component_class_match(self):
         """Defaults of matching component are written."""
         config = self._read()
         config.set_defaults(component='trac.tests.config.CompA')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt1 = 1\n',                f.next())
-            self.assertEqual('opt2 = a\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt1 = 1\n',
+            'opt2 = a\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_component_no_overwrite(self):
         """Values in configuration are not overwritten."""
@@ -1103,15 +1102,15 @@ class ConfigurationSetDefaultsTestCase(BaseTestCase):
         config.save()
         config.set_defaults(component='trac.tests.config.CompA')
         config.save()
-
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt1 = 3\n',                f.next())
-            self.assertEqual('opt2 = a\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt1 = 3\n',
+            'opt2 = a\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
     def test_component_no_overwrite_parent(self):
         """Values in parent configuration are not overwritten."""
@@ -1125,24 +1124,26 @@ class ConfigurationSetDefaultsTestCase(BaseTestCase):
         config.set_defaults(component='trac.tests.config.CompA')
         config.save()
 
-        with open(self.sitename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt1 = 3\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt1 = 3\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.sitename))
 
-        with open(self.filename, 'r') as f:
-            self.assertEqual('# -*- coding: utf-8 -*-\n', f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[compa]\n',                 f.next())
-            self.assertEqual('opt2 = a\n',                f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertEqual('[inherit]\n',               f.next())
-            self.assertEqual('file = trac-site.ini\n',    f.next())
-            self.assertEqual('\n',                        f.next())
-            self.assertRaises(StopIteration, f.next)
+        expected = [
+            '# -*- coding: utf-8 -*-\n',
+            '\n',
+            '[compa]\n',
+            'opt2 = a\n',
+            '\n',
+            '[inherit]\n',
+            'file = trac-site.ini\n',
+            '\n',
+        ]
+        self.assertEqual(expected, readlines(self.filename))
 
 
 class OptionDocTestCase(BaseTestCase):
@@ -1156,7 +1157,7 @@ class OptionDocTestCase(BaseTestCase):
             section_c = ConfigSection('c', '')
 
         self.assertEqual('Doc for a', Dummy.section_a.__doc__)
-        self.assertEqual(None, Dummy.section_a.doc_args)
+        self.assertIsNone(Dummy.section_a.doc_args)
         self.assertEqual('Doc for a', Dummy.section_a.doc)
         self.assertEqual('Doc for [%(page)s@%(version)d b]',
                          Dummy.section_b.__doc__)
@@ -1164,7 +1165,7 @@ class OptionDocTestCase(BaseTestCase):
                          Dummy.section_b.doc_args)
         self.assertEqual('Doc for [WikiStart@42 b]', Dummy.section_b.doc)
         self.assertEqual('', Dummy.section_c.__doc__)
-        self.assertEqual(None, Dummy.section_c.doc_args)
+        self.assertIsNone(Dummy.section_c.doc_args)
         self.assertEqual('', Dummy.section_c.doc)
 
     def test_options(self):
@@ -1199,7 +1200,7 @@ class OptionDocTestCase(BaseTestCase):
                 doc='Doc for %(name)s', doc_args={'name': 'ordered_ext_opt'})
 
         self.assertEqual('', Dummy.opt_nodoc.__doc__)
-        self.assertEqual(None, Dummy.opt_nodoc.doc_args)
+        self.assertIsNone(Dummy.opt_nodoc.doc_args)
         self.assertEqual('', Dummy.opt_nodoc.doc)
         self.assertEqual('Doc for %(name)s', Dummy.opt.__doc__)
         self.assertEqual({'name': 'opt'}, Dummy.opt.doc_args)
@@ -1228,6 +1229,113 @@ class OptionDocTestCase(BaseTestCase):
         self.assertEqual('Doc for ordered_ext_opt', Dummy.ordered_ext_opt.doc)
 
 
+class TracAdminTestCase(TracAdminTestCaseBase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.admin = TracAdmin()
+        self.admin.env_set('', self.env)
+
+    def tearDown(self):
+        self.env = None
+
+    def test_config_get(self):
+        """
+        Tests the 'config get' command in trac-admin.  This particular
+        test gets the project name from the config.
+        """
+        self.env.config.set('project', 'name', 'Test project')
+        rv, output = self.execute('config get project name')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_config_set(self):
+        """
+        Tests the 'config set' command in trac-admin.  This particular
+        test sets the project name using an option value containing a space.
+        """
+        rv, output = self.execute('config set project name "Test project"')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+        self.assertEqual('Test project',
+                         self.env.config.get('project', 'name'))
+
+    def test_config_remove(self):
+        """
+        Tests the 'config remove' command in trac-admin.  This particular
+        test removes the project name from the config, therefore reverting
+        the option to the default value.
+        """
+        self.env.config.set('project', 'name', 'Test project')
+        rv, output = self.execute('config remove project name')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+        self.assertEqual('My Project', self.env.config.get('project', 'name'))
+
+    def test_config_set_complete_section(self):
+        """Tab complete on a configuration section."""
+        # Empty sections are included.
+        output = self.complete_command('config', 'set', '')
+        self.assertIn('components', output)
+        self.assertIn('project', output)
+        self.assertNotIn('foo-section', output)
+
+        # Sections not defined in registry are included.
+        self.env.config.set('foo-section', 'bar-option', '1')
+        output = self.complete_command('config', 'set', '')
+        self.assertIn('foo-section', output)
+
+    def test_config_set_complete_option(self):
+        """Tab complete on a configuration option."""
+        output = self.complete_command('config', 'set', 'project', '')
+        self.assertEqual(['admin', 'admin_trac_url', 'descr', 'footer', 'icon',
+                          'name', 'url'], sorted(output))
+
+        # Options not defined in registry are included.
+        self.env.config.set('project', 'bar-option', '1')
+        output = self.complete_command('config', 'set', 'project', '')
+        self.assertIn('bar-option', output)
+
+
+class TracAdminComponentTestCase(TracAdminTestCaseBase):
+
+    components = []
+
+    @classmethod
+    def setUpClass(cls):
+        class CompA(Component):
+            opt1 = Option('compa', 'opt1', 1)
+            opt2 = Option('compa', 'opt2', 2)
+
+        cls.components = [CompA]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.components:
+            ComponentMeta.deregister(component)
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.admin = TracAdmin()
+        self.admin.env_set('', self.env)
+
+    def tearDown(self):
+        self.env = None
+
+    def test_config_component_enable(self):
+        self.env.config.save()
+        initial_file = copy.deepcopy(self.env.config.parser)
+
+        rv, output = self.execute('config set components '
+                                   'trac.tests.config.* enabled')
+
+        self.assertEqual(0, rv, output)
+        self.assertFalse(initial_file.has_section('compa'))
+        self.assertIn('compa', self.env.config)
+        self.assertIn('1', self.env.config.parser.get('compa', 'opt1'))
+        self.assertIn('2', self.env.config.parser.get('compa', 'opt2'))
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(UnicodeParserTestCase))
@@ -1239,6 +1347,12 @@ def test_suite():
         print("SKIP: trac.tests.config.ConfigurationSetDefaultsTestCase "
               "(__name__ is not trac.tests.config)")
     suite.addTest(unittest.makeSuite(OptionDocTestCase))
+    suite.addTest(unittest.makeSuite(TracAdminTestCase))
+    if __name__ == 'trac.tests.config':
+        suite.addTest(unittest.makeSuite(TracAdminComponentTestCase))
+    else:
+        print("SKIP: trac.tests.config.TracAdminComponentTestCase "
+              "(__name__ is not trac.tests.config)")
     return suite
 
 
