@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -14,7 +14,6 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-import trac.tests.compat
 from trac.core import *
 from trac.core import ComponentManager
 
@@ -27,7 +26,7 @@ class TracErrorTestCase(unittest.TestCase):
         e = TracError("the message", "the title", True)
         self.assertEqual("the message", e.message)
         self.assertEqual("the title", e.title)
-        self.assertEqual(True, e.show_traceback)
+        self.assertTrue(e.show_traceback)
 
     def test_unicode(self):
         e = TracError("the message")
@@ -199,7 +198,7 @@ class ComponentTestCase(unittest.TestCase):
             implements(ITest)
         tests = iter(ComponentA(self.compmgr).tests)
         try:
-            tests.next().test()
+            next(tests).test()
             self.fail('Expected AttributeError')
         except AttributeError:
             pass
@@ -212,7 +211,7 @@ class ComponentTestCase(unittest.TestCase):
         class ComponentA(Component):
             tests = ExtensionPoint(ITest)
         tests = iter(ComponentA(self.compmgr).tests)
-        self.assertRaises(StopIteration, tests.next)
+        self.assertRaises(StopIteration, next, tests)
 
     def test_extension_point_with_one_extension(self):
         """
@@ -227,8 +226,8 @@ class ComponentTestCase(unittest.TestCase):
             def test(self):
                 return 'x'
         tests = iter(ComponentA(self.compmgr).tests)
-        self.assertEqual('x', tests.next().test())
-        self.assertRaises(StopIteration, tests.next)
+        self.assertEqual('x', next(tests).test())
+        self.assertRaises(StopIteration, next, tests)
 
     def test_extension_point_with_two_extensions(self):
         """
@@ -261,8 +260,8 @@ class ComponentTestCase(unittest.TestCase):
             def test(self):
                 return 'x'
         tests = iter(ConcreteComponent(self.compmgr).tests)
-        self.assertEqual('x', tests.next().test())
-        self.assertRaises(StopIteration, tests.next)
+        self.assertEqual('x', next(tests).test())
+        self.assertRaises(StopIteration, next, tests)
 
     def test_inherited_implements(self):
         """
@@ -311,8 +310,8 @@ class ComponentTestCase(unittest.TestCase):
         mgr = ManagerComponent('Test', 42)
         self.assertEqual(id(mgr), id(mgr[ManagerComponent]))
         tests = iter(mgr.tests)
-        self.assertEqual('x', tests.next().test())
-        self.assertRaises(StopIteration, tests.next)
+        self.assertEqual('x', next(tests).test())
+        self.assertRaises(StopIteration, next, tests)
 
     def test_component_manager_component_isolation(self):
         """
