@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -55,40 +55,27 @@ Requirements:
  - lxml for XHTML validation (optional)
 """
 
-import exceptions
 import os
-import shutil
-import signal
-import stat
-import sys
-import time
+import subprocess
 import unittest
-from datetime import datetime, timedelta
 from pkg_resources import parse_version
 
-import trac
-from trac.test import rmtree
-from trac.util.compat import close_fds
+try:
+    from svn import core
+except ImportError:
+    has_svn = False
+else:
+    has_svn = True
 
 # Handle missing twill so we can print a useful 'SKIP'
 # message.  We import subprocess first to allow customizing it on Windows
 # to select pywin32 in favor of _subprocess for low-level calls.  If Twill
 # is allowed to load first, its (unmodified) copy will always be loaded.
-import subprocess
-
-from trac.tests.functional.better_twill import twill, b, tc, ConnectError
-
-try:
-    # This is the first indicator of whether the subversion bindings are
-    # correctly installed.
-    from svn import core
-    has_svn = True
-except ImportError:
-    has_svn = False
 
+import trac
 from trac.test import TestSetup, TestCaseSetup
-from trac.tests.contentgen import random_sentence, random_page, random_word, \
-    random_unique_camel
+from trac.tests.functional.better_twill import b, tc, twill
+
 
 internal_error = 'Trac detected an internal error:'
 
@@ -173,10 +160,10 @@ else:
         def __init__(self):
             raise ImportError("Twill not installed")
 
-    class FunctionalTwillTestCaseSetup:
+    class FunctionalTwillTestCaseSetup(object):
         pass
 
-    class FunctionalTestCaseSetup:
+    class FunctionalTestCaseSetup(object):
         pass
 
 
@@ -210,6 +197,8 @@ def test_suite():
         trac.timeline.tests.functionalSuite(suite)
         import trac.admin.tests
         trac.admin.tests.functionalSuite(suite)
+        import trac.search.tests
+        trac.search.tests.functionalSuite(suite)
         # The db tests should be last since the backup test occurs there.
         import trac.db.tests
         trac.db.tests.functionalSuite(suite)
