@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,7 +14,6 @@
 
 import os
 import re
-import time
 import unittest
 
 from trac.tests.functional import FunctionalTwillTestCaseSetup, \
@@ -27,13 +26,13 @@ class TestAttachmentNonexistentParent(FunctionalTwillTestCaseSetup):
         """TracError should be raised when navigating to the attachment
         page for a nonexistent resource."""
         self._tester.go_to_wiki('NonexistentPage')
-        tc.find("The page <strong>NonexistentPage</strong> does not exist. "
-                "You can create it here.")
+        tc.find("The page[ \n]+<strong>NonexistentPage</strong>[ \n]+"
+                "does not exist. You can create it here.")
         tc.find(r"\bCreate this page\b")
 
         tc.go(self._tester.url + '/attachment/wiki/NonexistentPage')
         tc.find('<h1>Trac Error</h1>\s+<p class="message">'
-                'Parent resource NonexistentPage doesn\'t exist</p>')
+                'Parent resource NonexistentPage doesn&#39;t exist</p>')
 
 
 class TestAboutPage(FunctionalTwillTestCaseSetup):
@@ -52,8 +51,6 @@ class TestErrorPage(FunctionalTwillTestCaseSetup):
     """
     def runTest(self):
         env = self._testenv.get_trac_environment()
-        env.config.set('components', 'RaiseExceptionPlugin.*', 'enabled')
-        env.config.save()
         create_file(os.path.join(env.plugins_dir, 'RaiseExceptionPlugin.py'),
 """\
 from trac.core import Component, TracError, implements
@@ -128,109 +125,52 @@ class RaiseExceptionPlugin(Component):
             env.config.set('components', 'RaiseExceptionPlugin.*', 'disabled')
 
 
-class RegressionTestRev6017(FunctionalTwillTestCaseSetup):
+class RegressionTestTicket3833(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of the plugin reload fix in r6017"""
-        # Setup the DeleteTicket plugin
+        """Test for regression of https://trac.edgewall.org/ticket/3833"""
         env = self._testenv.get_trac_environment()
-        plugin = open(os.path.join(self._testenv.trac_src,
-                                   'sample-plugins', 'workflow',
-                                   'DeleteTicket.py')).read()
-        plugin_path = os.path.join(env.plugins_dir, 'DeleteTicket.py')
-        open(plugin_path, 'w').write(plugin)
-        prevconfig = env.config.get('ticket', 'workflow')
-        env.config.set('ticket', 'workflow',
-                       prevconfig + ',DeleteTicketActionController')
+        trac_log = os.path.join(env.log_dir, 'trac.log')
+
+        def read_log_file(offset):
+            with open(trac_log) as fd:
+                fd.seek(offset)
+                return fd.read()
+
+        # Verify that file logging is enabled as info level.
+        log_size = os.path.getsize(trac_log)
+        env.log.info("RegressionTestTicket3833 info1")
+        info1 = "RegressionTestTicket3833 info1"
+        log_content = read_log_file(log_size)
+        self.assertEqual('INFO', env.config.get('logging', 'log_level'))
+        self.assertIn(info1, log_content),
+
+        # Verify that info level is not logged at warning level.
+        env.config.set('logging', 'log_level', 'WARNING')
         env.config.save()
-        env = self._testenv.get_trac_environment() # reloads the environment
-
-        loaded_components = env.compmgr.__metaclass__._components
-        delete_plugins = [c for c in loaded_components
-                          if 'DeleteTicketActionController' in c.__name__]
-        try:
-            self.assertEqual(len(delete_plugins), 1,
-                             "Plugin loaded more than once.")
-
-        finally:
-            # Remove the DeleteTicket plugin
-            env.config.set('ticket', 'workflow', prevconfig)
-            env.config.save()
-            for ext in ('py', 'pyc', 'pyo'):
-                filename = os.path.join(env.plugins_dir,
-                                        'DeleteTicket.%s' % ext)
-                if os.path.exists(filename):
-                    os.unlink(filename)
-
-
-class RegressionTestTicket3833a(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of https://trac.edgewall.org/ticket/3833 a"""
         env = self._testenv.get_trac_environment()
-        # Assume the logging is already set to debug.
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
-
-        # Verify that logging is on initially
-        env.log.debug("RegressionTestTicket3833 debug1")
-        debug1 = traclogfile.read()
-        self.assertNotEqual(debug1.find("RegressionTestTicket3833 debug1"), -1,
-                            'Logging off when it should have been on.\n%r'
-                            % debug1)
-
-
-class RegressionTestTicket3833b(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of https://trac.edgewall.org/ticket/3833 b"""
-        # Turn logging off, try to log something, and verify that it does
-        # not show up.
-        env = self._testenv.get_trac_environment()
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
+        log_size = os.path.getsize(trac_log)
+        info2 = "RegressionTestTicket3833 info2"
+        warn2 = "RegressionTestTicket3833 warn2"
+        env.log.info(info2)
+        env.log.warning(warn2)
+        log_content = read_log_file(log_size)
+        self.assertEqual('WARNING', env.config.get('logging', 'log_level'))
+        self.assertNotIn(info2, log_content)
+        self.assertIn(warn2, log_content)
 
+        # Revert to info level logging.
         env.config.set('logging', 'log_level', 'INFO')
         env.config.save()
         env = self._testenv.get_trac_environment()
-        env.log.debug("RegressionTestTicket3833 debug2")
-        env.log.info("RegressionTestTicket3833 info2")
-        debug2 = traclogfile.read()
-        self.assertNotEqual(debug2.find("RegressionTestTicket3833 info2"), -1,
-                            'Logging at info failed.\n%r' % debug2)
-        self.assertEqual(debug2.find("RegressionTestTicket3833 debug2"), -1,
-                         'Logging still on when it should have been off.\n%r'
-                         % debug2)
-
-
-class RegressionTestTicket3833c(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of https://trac.edgewall.org/ticket/3833 c"""
-        # Turn logging back on, try to log something, and verify that it
-        # does show up.
-        env = self._testenv.get_trac_environment()
-        traclogfile = open(os.path.join(env.log_dir, 'trac.log'))
-        # Seek to the end of file so we only look at new log output
-        traclogfile.seek(0, 2)
-
-        env.config.set('logging', 'log_level', 'DEBUG')
-        time.sleep(2)
-        env.config.save()
-        #time.sleep(2)
-        env = self._testenv.get_trac_environment()
-        #time.sleep(2)
-        env.log.debug("RegressionTestTicket3833 debug3")
-        env.log.info("RegressionTestTicket3833 info3")
-        #time.sleep(2)
-        debug3 = traclogfile.read()
-        message = ''
-        success = debug3.find("RegressionTestTicket3833 debug3") != -1
-        if not success:
-            # Ok, the testcase failed, but we really need logging enabled.
-            env.log.debug("RegressionTestTicket3833 fixup3")
-            fixup3 = traclogfile.read()
-            message = 'Logging still off when it should have been on.\n' \
-                      '%r\n%r' % (debug3, fixup3)
-        self.assertTrue(success, message)
+        log_size = os.path.getsize(trac_log)
+        info3 = "RegressionTestTicket3833 info3"
+        warn3 = "RegressionTestTicket3833 warn3"
+        env.log.info(info3)
+        env.log.warning(warn3)
+        log_content = read_log_file(log_size)
+        self.assertEqual('INFO', env.config.get('logging', 'log_level'))
+        self.assertIn(info3, log_content)
+        self.assertIn(warn3, log_content)
 
 
 class RegressionTestTicket5572(FunctionalTwillTestCaseSetup):
@@ -297,26 +237,14 @@ class RegressionTestTicket3663(FunctionalTwillTestCaseSetup):
 
         Verify that URLs not encoded with UTF-8 are reported as invalid.
         """
-        import httplib
-        # Work around for InvalidURL since Python 2.7.17 (#13233)
-        saved_re = httplib._contains_disallowed_url_pchar_re \
-                   if hasattr(httplib, '_contains_disallowed_url_pchar_re') \
-                   else None
-        try:
-            if saved_re:
-                httplib._contains_disallowed_url_pchar_re = \
-                    re.compile(r'[\x00-\x20]')
-            # invalid PATH_INFO
-            self._tester.go_to_wiki(u'été'.encode('latin1'))
-            tc.code(404)
-            tc.find('Invalid URL encoding')
-            # invalid SCRIPT_NAME
-            tc.go(u'été'.encode('latin1'))
-            tc.code(404)
-            tc.find('Invalid URL encoding')
-        finally:
-            if saved_re:
-                httplib._contains_disallowed_url_pchar_re = saved_re
+        # invalid PATH_INFO
+        self._tester.go_to_wiki(u'été'.encode('latin1'))
+        tc.code(404)
+        tc.find('Invalid URL encoding')
+        # invalid SCRIPT_NAME
+        tc.go(u'été'.encode('latin1'))
+        tc.code(404)
+        tc.find('Invalid URL encoding')
 
 
 class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
@@ -332,6 +260,7 @@ class RegressionTestTicket6318(FunctionalTwillTestCaseSetup):
         self._tester.logout()
         try:
             # also test a regular ascii user name
+            self._testenv.adduser(u'user')
             self._tester.login(u'user')
             self._tester.go_to_front()
             self._tester.logout()
@@ -434,10 +363,7 @@ def functionalSuite(suite=None):
     suite.addTest(TestAttachmentNonexistentParent())
     suite.addTest(TestAboutPage())
     suite.addTest(TestErrorPage())
-    suite.addTest(RegressionTestRev6017())
-    suite.addTest(RegressionTestTicket3833a())
-    suite.addTest(RegressionTestTicket3833b())
-    suite.addTest(RegressionTestTicket3833c())
+    suite.addTest(RegressionTestTicket3833())
     suite.addTest(RegressionTestTicket5572())
     suite.addTest(RegressionTestTicket7209())
     suite.addTest(RegressionTestTicket9880())
