@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -20,11 +20,10 @@ import locale
 import os
 import re
 import sys
-import textwrap
 import time
 from subprocess import call, Popen, PIPE, STDOUT
 
-from trac.config import UnicodeConfigParser
+from trac.config import Configuration, UnicodeConfigParser
 from trac.db.api import DatabaseManager
 from trac.env import open_environment
 from trac.test import EnvironmentStub, get_dburi, rmtree
@@ -43,7 +42,7 @@ from trac.util.text import to_utf8
 #
 #     Backends::
 #       sqlite3+pysqlite2, postgres+psycopg2 python bindings,
-#       mysql+mysqldb with server v4, mysql+mysqldb with server v5
+#       mysql+pymysql with server v5
 #       (those need to test search escaping, among many other things like long
 #       paths in browser and unicode chars being allowed/translating...)
 
@@ -137,20 +136,13 @@ class FunctionalTestEnvironment(object):
         self.logfile = open(os.path.join(self.dirname, 'testing.log'), 'w')
         self.create_repo()
 
-        repo_path = self.repo_path_for_initenv()
-        args = ['initenv', self.tracdir, self.dburi]
-        if repo_path:
-            config_file = os.path.join(self.dirname, 'init.ini')
-            create_file(config_file, textwrap.dedent("""\
-                [components]
-                tracopt.versioncontrol.%(repo_type)s.* = enabled
-                [repositories]
-                .type = %(repo_type)s
-                .dir = %(repo_path)s
-                """ % {'repo_type': self.repotype,
-                       'repo_path': repo_path}))
-            args += ['--config=%s' % config_file]
-        self._tracadmin(*args)
+        config_file = os.path.join(self.dirname, 'config.ini')
+        config = Configuration(config_file)
+        config.set('repositories', '.dir', self.repo_path_for_initenv())
+        config.set('repositories', '.type', self.repotype)
+        config.save()
+        self._tracadmin('initenv', self.tracdir, self.dburi,
+                        '--config=%s' % config_file)
         if call([sys.executable,
                  os.path.join(self.trac_src, 'contrib', 'htpasswd.py'), "-c",
                  "-b", self.htpasswd, "admin", "admin"], close_fds=close_fds,
@@ -159,9 +151,7 @@ class FunctionalTestEnvironment(object):
         self.adduser('user')
         self.adduser('joe')
         self.grant_perm('admin', 'TRAC_ADMIN')
-        # Setup Trac logging
         env = self.get_trac_environment()
-        env.config.set('logging', 'log_type', 'file')
         for component in self.get_enabled_components():
             env.config.set('components', component, 'enabled')
         env.config.save()
@@ -225,13 +215,13 @@ class FunctionalTestEnvironment(object):
     def remove_config(self, *args):
         """Calls trac-admin to remove the value for the given option
         in `trac.ini`."""
-        self._tracadmin('config', 'remove', *args)
+        return self._tracadmin('config', 'remove', *args)
 
     def _tracadmin(self, *args):
         """Internal utility method for calling trac-admin"""
         proc = Popen([sys.executable, os.path.join(self.trac_src, 'trac',
                       'admin', 'console.py'), self.tracdir],
-                     stdin=PIPE, stdout=PIPE, stderr=PIPE,
+                     stdin=PIPE, stdout=PIPE, stderr=STDOUT,
                      close_fds=close_fds, cwd=self.command_cwd)
         if args:
             if any('\n' in arg for arg in args):
@@ -243,12 +233,13 @@ class FunctionalTestEnvironment(object):
                              for idx, arg in enumerate(args))
         else:
             input = None
-        out, err = proc.communicate(input=input)
-        if proc.returncode or err:
-            self.logfile.write(err)
+        out = proc.communicate(input=input)[0]
+        if proc.returncode:
+            print(out)
+            self.logfile.write(out)
             raise Exception("Failed while running trac-admin with arguments %r.\n"
                             "Exitcode: %s \n%s"
-                            % (args, proc.returncode, err))
+                            % (args, proc.returncode, out))
         else:
             # trac-admin is started in interactive mode, so we strip away
             # everything up to the to the interactive prompt
@@ -256,14 +247,20 @@ class FunctionalTestEnvironment(object):
 
     def start(self):
         """Starts the webserver, and waits for it to come up."""
-        args = [
-            sys.executable,
-            os.path.join(self.trac_src, 'trac', 'web', 'standalone.py')
-        ]
+        if 'FIGLEAF' in os.environ:
+            exe = os.environ['FIGLEAF']
+            if ' ' in exe: # e.g. 'coverage run'
+                args = exe.split()
+            else:
+                args = [exe]
+        else:
+            args = [sys.executable]
         options = ["--port=%s" % self.port, "-s", "--hostname=127.0.0.1",
                    "--basic-auth=trac,%s," % self.htpasswd]
         if 'TRAC_TEST_TRACD_OPTIONS' in os.environ:
             options += os.environ['TRAC_TEST_TRACD_OPTIONS'].split()
+        args.append(os.path.join(self.trac_src, 'trac', 'web',
+                                 'standalone.py'))
         server = Popen(args + options + [self.tracdir],
                        stdout=self.logfile, stderr=self.logfile,
                        close_fds=close_fds,
@@ -301,7 +298,7 @@ class FunctionalTestEnvironment(object):
 
     def repo_path_for_initenv(self):
         """Default to no repository"""
-        return None
+        return ''
 
     def call_in_dir(self, dir, args, environ=None):
         proc = Popen(args, stdout=PIPE, stderr=self.logfile,
