@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018-2020 Edgewall Software
+# Copyright (C) 2017-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -10,8 +10,6 @@
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
-#
-# Author: Christopher Lenz <cmlenz@gmx.de>
 
 import glob
 import os
@@ -19,12 +17,11 @@ import pkg_resources
 import shutil
 import subprocess
 import sys
-import tempfile
 import unittest
 
 from trac import loader
 from trac.core import ComponentMeta
-from trac.test import EnvironmentStub
+from trac.test import EnvironmentStub, mkdtemp
 from trac.util import create_file
 from trac.util.compat import close_fds
 
@@ -94,8 +91,8 @@ class ComponentB(Component):
 class LoadComponentsTestCase(unittest.TestCase):
 
     def setUp(self):
-        tempdir = tempfile.mkdtemp(prefix='trac-tempenv-')
-        self.env = EnvironmentStub(path=tempdir)
+        self.env = EnvironmentStub(path=mkdtemp())
+        os.makedirs(os.path.join(self.env.path, 'plugins'))
         self.components = []
 
     def tearDown(self):
@@ -107,8 +104,12 @@ class LoadComponentsTestCase(unittest.TestCase):
     def _cleanup_working_set(self):
         for plugin_name in os.listdir(self.env.plugins_dir):
             plugin_path = os.path.join(self.env.plugins_dir, plugin_name)
-            pkg_resources.working_set.entries.remove(plugin_path)
-            del pkg_resources.working_set.entry_keys[plugin_path]
+            try:
+                pkg_resources.working_set.entries.remove(plugin_path)
+            except ValueError:
+                pass
+            else:
+                del pkg_resources.working_set.entry_keys[plugin_path]
 
     def _build_egg_file(self, module_name):
         plugin_src = os.path.join(self.env.path, 'plugin_src')
@@ -137,7 +138,6 @@ class LoadComponentsTestCase(unittest.TestCase):
         egg_file_src = self._build_egg_file('plugin1')
         egg_file_dst = os.path.join(self.env.plugins_dir,
                                     os.path.basename(egg_file_src))
-        os.mkdir(self.env.plugins_dir)
         shutil.copyfile(egg_file_src, egg_file_dst)
 
         loader.load_components(self.env)
@@ -152,6 +152,26 @@ class LoadComponentsTestCase(unittest.TestCase):
         self.assertNotIn(ComponentB, ComponentMeta._components)
         self.assertNotIn(ComponentB, registry.get(IEnvironmentSetupParticipant))
 
+    def test_component_loaded_once(self):
+        create_file(os.path.join(self.env.plugins_dir,
+                                 'RegressionTestRev6017.py'), """\
+from trac.wiki.macros import WikiMacroBase
+
+class RegressionTestRev6017Macro(WikiMacroBase):
+    def expand_macro(self, formatter, name, content, args):
+        return "Hello World"
+
+""")
+
+        loader.load_components(self.env)
+        loader.load_components(self.env)
+
+        loaded_components = [c for c in ComponentMeta._components
+                               if 'RegressionTestRev6017' in c.__name__]
+
+        self.assertEqual(1, len(loaded_components),
+                         "Plugin loaded more than once.")
+
 
 def test_suite():
     suite = unittest.TestSuite()
