@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
@@ -27,6 +27,7 @@ import socket
 import string
 import threading
 import unittest
+from contextlib import closing
 
 from trac.config import ConfigurationError
 from trac.notification import SendmailEmailSender, SmtpEmailSender
@@ -35,7 +36,6 @@ from trac.test import EnvironmentStub
 LF = '\n'
 CR = '\r'
 SMTP_TEST_PORT = 7000 + os.getpid() % 1000
-email_re = re.compile(r'([\w\d_\.\-])+\@(([\w\d\-])+\.)+([\w\d]{2,4})+')
 header_re = re.compile(r'^=\?(?P<charset>[\w\d\-]+)\?(?P<code>[qb])\?(?P<value>.*)\?=$')
 
 
@@ -324,13 +324,12 @@ class SMTPThreadedServer(threading.Thread):
         # run from the main thread
         self.server.stop()
         # send a message to make the SMTP server quit gracefully
-        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            s.connect(('127.0.0.1', self.port))
-            r = s.send("QUIT\r\n")
-        except socket.error:
-            pass
-        s.close()
+        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
+            try:
+                s.connect(('127.0.0.1', self.port))
+                s.send("QUIT\r\n")
+            except socket.error:
+                pass
         # wait for the SMTP server to complete (for up to 2 secs)
         self.join(2.0)
         # clean up the SMTP server (and force quit if needed)
@@ -349,15 +348,6 @@ class SMTPThreadedServer(threading.Thread):
         self.store.reset(None)
 
 
-def smtp_address(fulladdr):
-    mo = email_re.search(fulladdr)
-    if mo:
-        return mo.group(0)
-    if start >= 0:
-        return fulladdr[start+1:-1]
-    return fulladdr
-
-
 def decode_header(header):
     """ Decode a MIME-encoded header value """
     mo = header_re.match(header)
