@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,10 +14,29 @@
 import unittest
 
 from trac import perm
-from trac.core import *
+from trac.admin.console import TracAdmin
+from trac.admin.test import TracAdminTestCaseBase
+from trac.core import Component, ComponentMeta, TracError, implements
 from trac.resource import Resource
 from trac.test import EnvironmentStub
 
+# IPermissionRequestor implementations
+import trac.about
+import trac.admin.web_ui
+import trac.perm
+import trac.search.web_ui
+import trac.ticket.api
+import trac.ticket.batch
+import trac.ticket.report
+import trac.ticket.roadmap
+import trac.timeline.web_ui
+import trac.versioncontrol.admin
+import trac.versioncontrol.web_ui.browser
+import trac.versioncontrol.web_ui.changeset
+import trac.versioncontrol.web_ui.log
+import trac.web.chrome
+import trac.wiki.web_ui
+
 
 class DefaultPermissionStoreTestCase(unittest.TestCase):
 
@@ -37,7 +56,7 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
              ('john', 'REPORT_ADMIN'),
              ('kate', 'TICKET_CREATE')])
         self.assertEqual(['REPORT_ADMIN', 'WIKI_MODIFY'],
-                         sorted(self.store.get_user_permissions('john')))
+                         self.store.get_user_permissions('john'))
         self.assertEqual(['TICKET_CREATE'],
                          self.store.get_user_permissions('kate'))
 
@@ -48,7 +67,7 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
              ('dev', 'REPORT_ADMIN'),
              ('john', 'dev')])
         self.assertEqual(['REPORT_ADMIN', 'WIKI_MODIFY'],
-                         sorted(self.store.get_user_permissions('john')))
+                         self.store.get_user_permissions('john'))
 
     def test_nested_groups(self):
         self.env.db_transaction.executemany(
@@ -58,7 +77,7 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
              ('admin', 'dev'),
              ('john', 'admin')])
         self.assertEqual(['REPORT_ADMIN', 'WIKI_MODIFY'],
-                         sorted(self.store.get_user_permissions('john')))
+                         self.store.get_user_permissions('john'))
 
     def test_mixed_case_group(self):
         self.env.db_transaction.executemany(
@@ -68,7 +87,7 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
              ('Admin', 'Dev'),
              ('john', 'Admin')])
         self.assertEqual(['REPORT_ADMIN', 'WIKI_MODIFY'],
-                         sorted(self.store.get_user_permissions('john')))
+                         self.store.get_user_permissions('john'))
 
     def test_builtin_groups(self):
         self.env.db_transaction.executemany(
@@ -77,7 +96,7 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
              ('authenticated', 'REPORT_ADMIN'),
              ('anonymous', 'TICKET_CREATE')])
         self.assertEqual(['REPORT_ADMIN', 'TICKET_CREATE', 'WIKI_MODIFY'],
-                         sorted(self.store.get_user_permissions('john')))
+                         self.store.get_user_permissions('john'))
         self.assertEqual(['TICKET_CREATE'],
                          self.store.get_user_permissions('anonymous'))
 
@@ -91,17 +110,53 @@ class DefaultPermissionStoreTestCase(unittest.TestCase):
                     ('dev', 'REPORT_ADMIN'),
                     ('john', 'dev')]
         for res in self.store.get_all_permissions():
-            self.assertFalse(res not in expected)
+            self.assertIn(res, expected)
+
+    def test_get_permission_groups(self):
+        self.env.db_transaction.executemany(
+            "INSERT INTO permission VALUES (%s,%s)",
+            [('user1', 'group1'),
+             ('group1', 'group2'),
+             ('group2', 'group3'),
+             ('user2', 'group4'),
+             ('user1', 'group5'),
+             ('group6', 'group7'),
+             ('user3', 'group8'),  # test recursion
+             ('group8', 'group9'),
+             ('group9', 'group8'),
+             ('user3', 'group11'),
+             ('group11', 'group10'),  # test recursion
+             ('group10', 'group11'),
+             ('group10', 'group10')])
+        self.assertEqual(['group1', 'group2', 'group3', 'group5'],
+                         self.store.get_permission_groups('user1'))
+        self.assertEqual(['group4'],
+                         self.store.get_permission_groups('user2'))
+        self.assertEqual(['group10', 'group11', 'group8', 'group9'],
+                         self.store.get_permission_groups('user3'))
 
 
-class TestPermissionRequestor(Component):
-    implements(perm.IPermissionRequestor)
+class BaseTestCase(unittest.TestCase):
 
-    def get_permission_actions(self):
-        return ['TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY',
-                ('TEST_CREATE', []),
-                ('TEST_ADMIN', ['TEST_CREATE', 'TEST_DELETE']),
-                ('TEST_ADMIN', ['TEST_MODIFY'])]
+    permission_requestors = []
+
+    @classmethod
+    def setUpClass(cls):
+        class TestPermissionRequestor(Component):
+            implements(perm.IPermissionRequestor)
+
+            def get_permission_actions(self):
+                return ['TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY',
+                        ('TEST_CREATE', []),
+                        ('TEST_ADMIN', ['TEST_CREATE', 'TEST_DELETE']),
+                        ('TEST_ADMIN', ['TEST_MODIFY'])]
+
+        cls.permission_requestors = [TestPermissionRequestor]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.permission_requestors:
+            ComponentMeta.deregister(component)
 
 
 class PermissionErrorTestCase(unittest.TestCase):
@@ -111,13 +166,12 @@ class PermissionErrorTestCase(unittest.TestCase):
 
     def test_default_message(self):
         permission_error = perm.PermissionError()
-        self.assertEqual(None, permission_error.action)
-        self.assertEqual(None, permission_error.resource)
-        self.assertEqual(None, permission_error.env)
+        self.assertIsNone(permission_error.action)
+        self.assertIsNone(permission_error.resource)
+        self.assertIsNone(permission_error.env)
         self.assertEqual("Insufficient privileges to perform this operation.",
                          unicode(permission_error))
         self.assertEqual("Forbidden", permission_error.title)
-        self.assertEqual(unicode(permission_error), permission_error.msg)
         self.assertEqual(unicode(permission_error), permission_error.message)
 
     def test_message_specified(self):
@@ -129,8 +183,8 @@ class PermissionErrorTestCase(unittest.TestCase):
         action = 'WIKI_VIEW'
         permission_error = perm.PermissionError(action)
         self.assertEqual(action, permission_error.action)
-        self.assertEqual(None, permission_error.resource)
-        self.assertEqual(None, permission_error.env)
+        self.assertIsNone(permission_error.resource)
+        self.assertIsNone(permission_error.env)
         self.assertEqual("WIKI_VIEW privileges are required to perform this "
                          "operation. You don't have the required "
                          "permissions.", unicode(permission_error))
@@ -158,12 +212,13 @@ class PermissionErrorTestCase(unittest.TestCase):
                          "permissions.", unicode(permission_error))
 
 
-class PermissionSystemTestCase(unittest.TestCase):
+class PermissionSystemTestCase(BaseTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(enable=[perm.PermissionSystem,
-                                           perm.DefaultPermissionStore,
-                                           TestPermissionRequestor])
+                                           perm.DefaultPermissionGroupProvider,
+                                           perm.DefaultPermissionStore] +
+                                          self.permission_requestors)
         self.perm = perm.PermissionSystem(self.env)
 
     def tearDown(self):
@@ -172,9 +227,9 @@ class PermissionSystemTestCase(unittest.TestCase):
     def test_get_actions(self):
         tpr_perms = ['TEST_ADMIN', 'TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY']
         all_perms = tpr_perms + ['TRAC_ADMIN']
-        self.assertEqual(all_perms, sorted(self.perm.get_actions()))
+        self.assertEqual(all_perms, self.perm.get_actions())
         self.assertEqual(tpr_perms,
-                         sorted(self.perm.get_actions(skip=self.perm)))
+                         self.perm.get_actions(skip=self.perm))
 
     def test_get_actions_dict(self):
         self.assertEqual({
@@ -182,7 +237,7 @@ class PermissionSystemTestCase(unittest.TestCase):
             'TEST_CREATE': [],
             'TEST_DELETE': [],
             'TEST_MODIFY': [],
-            'TRAC_ADMIN': ['TEST_CREATE', 'TEST_DELETE', 'TEST_ADMIN',
+            'TRAC_ADMIN': ['TEST_ADMIN', 'TEST_CREATE', 'TEST_DELETE',
                            'TEST_MODIFY'],
         }, self.perm.get_actions_dict())
         self.assertEqual({
@@ -216,6 +271,17 @@ class PermissionSystemTestCase(unittest.TestCase):
                           'TEST_MODIFY': True,  'TEST_ADMIN': True},
                          self.perm.get_user_permissions('jane'))
 
+    def test_undefined_permissions(self):
+        """Only defined actions are returned in the dictionary."""
+        self.perm.grant_permission('bob', 'TEST_CREATE')
+        self.perm.grant_permission('jane', 'TEST_DELETE')
+        self.perm.grant_permission('jane', 'TEST_MODIFY')
+
+        self.env.disable_component(self.permission_requestors[0])
+
+        self.assertEqual({}, self.perm.get_user_permissions('bob'))
+        self.assertEqual({}, self.perm.get_user_permissions('jane'))
+
     def test_grant_permission_differs_from_action_by_casing(self):
         """`TracError` is raised when granting a permission that differs
         from an action by casing.
@@ -223,13 +289,29 @@ class PermissionSystemTestCase(unittest.TestCase):
         self.assertRaises(TracError, self.perm.grant_permission, 'user1',
                           'Test_Create')
 
+    def test_grant_permission_already_granted(self):
+        """`PermissionExistsError` is raised when granting a permission
+        that has already been granted.
+        """
+        self.perm.grant_permission('user1', 'TEST_CREATE')
+        self.assertRaises(perm.PermissionExistsError,
+                          self.perm.grant_permission, 'user1', 'TEST_CREATE')
+
+    def test_grant_permission_already_in_group(self):
+        """`PermissionExistsError` is raised when adding a user to
+        a group of which they are already a member.
+        """
+        self.perm.grant_permission('user1', 'group1')
+        self.assertRaises(perm.PermissionExistsError,
+                          self.perm.grant_permission, 'user1', 'group1')
+
     def test_get_all_permissions(self):
         self.perm.grant_permission('bob', 'TEST_CREATE')
         self.perm.grant_permission('jane', 'TEST_ADMIN')
         expected = [('bob', 'TEST_CREATE'),
                     ('jane', 'TEST_ADMIN')]
         for res in self.perm.get_all_permissions():
-            self.assertFalse(res not in expected)
+            self.assertIn(res, expected)
 
     def test_get_groups_dict(self):
         permissions = [
@@ -262,20 +344,48 @@ class PermissionSystemTestCase(unittest.TestCase):
                          users['user1'])
         self.assertEqual(['TEST_CREATE'], users['user2'])
 
+    def test_get_permission_groups(self):
+        permissions = [
+            ('user1', 'group1'),
+            ('group1', 'group2'),
+            ('group2', 'group3'),
+            ('user2', 'group4'),
+            ('user1', 'group5'),
+            ('group6', 'group7'),
+            ('user3', 'group8'), # test recursion
+            ('group8', 'group9'),
+            ('group9', 'group8'),
+            ('user3', 'group11'),
+            ('group11', 'group10'),  # test recursion
+            ('group10', 'group11'),
+            ('group10', 'group10'),
+        ]
+        for perm_ in permissions:
+            self.perm.grant_permission(*perm_)
+
+        self.assertEqual(['anonymous', 'authenticated', 'group1', 'group2',
+                          'group3', 'group5'],
+                         self.perm.get_permission_groups('user1'))
+        self.assertEqual(['anonymous', 'authenticated', 'group4'],
+                         self.perm.get_permission_groups('user2'))
+        self.assertEqual(['anonymous', 'authenticated', 'group10', 'group11',
+                          'group8', 'group9'],
+                         self.perm.get_permission_groups('user3'))
+
     def test_expand_actions_iter_7467(self):
         # Check that expand_actions works with iterators (#7467)
-        perms = set(['TRAC_ADMIN', 'TEST_DELETE', 'TEST_MODIFY',
-                     'TEST_CREATE', 'TEST_ADMIN'])
+        perms = ['TEST_ADMIN', 'TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY',
+                 'TRAC_ADMIN']
         self.assertEqual(perms, self.perm.expand_actions(['TRAC_ADMIN']))
         self.assertEqual(perms, self.perm.expand_actions(iter(['TRAC_ADMIN'])))
 
 
-class PermissionCacheTestCase(unittest.TestCase):
+class PermissionCacheTestCase(BaseTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(enable=[perm.DefaultPermissionStore,
-                                           perm.DefaultPermissionPolicy,
-                                           TestPermissionRequestor])
+                                           perm.DefaultPermissionPolicy] +
+                                          self.permission_requestors)
         self.env.config.set('trac', 'permission_policies',
                             'DefaultPermissionPolicy')
         self.perm_system = perm.PermissionSystem(self.env)
@@ -289,9 +399,9 @@ class PermissionCacheTestCase(unittest.TestCase):
         self.env.reset_db()
 
     def test_contains(self):
-        self.assertTrue('TEST_MODIFY' in self.perm)
-        self.assertTrue('TEST_ADMIN' in self.perm)
-        self.assertFalse('TRAC_ADMIN' in self.perm)
+        self.assertIn('TEST_MODIFY', self.perm)
+        self.assertIn('TEST_ADMIN', self.perm)
+        self.assertNotIn('TRAC_ADMIN', self.perm)
 
     def test_has_permission(self):
         self.assertTrue(self.perm.has_permission('TEST_MODIFY'))
@@ -357,13 +467,13 @@ class TestPermissionPolicy(Component):
         return result
 
 
-class PermissionPolicyTestCase(unittest.TestCase):
+class PermissionPolicyTestCase(BaseTestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(enable=[perm.DefaultPermissionStore,
                                            perm.DefaultPermissionPolicy,
-                                           TestPermissionPolicy,
-                                           TestPermissionRequestor])
+                                           TestPermissionPolicy] +
+                                          self.permission_requestors)
         self.env.config.set('trac', 'permission_policies',
                             'TestPermissionPolicy')
         self.policy = TestPermissionPolicy(self.env)
@@ -383,8 +493,8 @@ class PermissionPolicyTestCase(unittest.TestCase):
 
     def test_grant_revoke_permissions(self):
         self.policy.grant('testuser', ['TEST_MODIFY', 'TEST_ADMIN'])
-        self.assertEqual('TEST_MODIFY' in self.perm, True)
-        self.assertEqual('TEST_ADMIN' in self.perm, True)
+        self.assertIn('TEST_MODIFY', self.perm)
+        self.assertIn('TEST_ADMIN', self.perm)
         self.assertEqual(self.policy.results,
                          {('testuser', 'TEST_MODIFY'): True,
                           ('testuser', 'TEST_ADMIN'): True})
@@ -399,8 +509,8 @@ class PermissionPolicyTestCase(unittest.TestCase):
         self.assertEqual(list(system.policies),
                          [self.policy,
                           perm.DefaultPermissionPolicy(self.env)])
-        self.assertEqual('TEST_MODIFY' in self.perm, True)
-        self.assertEqual('TEST_ADMIN' in self.perm, True)
+        self.assertIn('TEST_MODIFY', self.perm)
+        self.assertIn('TEST_ADMIN', self.perm)
         self.assertEqual(self.policy.results,
                          {('testuser', 'TEST_MODIFY'): True,
                           ('testuser', 'TEST_ADMIN'): None})
@@ -409,11 +519,11 @@ class PermissionPolicyTestCase(unittest.TestCase):
 class RecursivePolicyTestCase(unittest.TestCase):
     """Test case for policies that perform recursive permission checks."""
 
-    def setUp(self):
-        self.env = EnvironmentStub()
-        self.env.clear_component_registry()
-        decisions = []
-        self.decisions = decisions
+    permission_policies = []
+    decisions = []
+
+    @classmethod
+    def setUpClass(cls):
 
         class PermissionPolicy1(Component):
 
@@ -421,6 +531,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
 
             def __init__(self):
                 self.call_count = 0
+                self.decisions = cls.decisions
 
             def check_permission(self, action, username, resource, perm):
                 self.call_count += 1
@@ -429,7 +540,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
                     decision = None
                 elif action == 'ACTION_1':
                     decision = username == 'user1'
-                decisions.append(('policy1', action, decision))
+                self.decisions.append(('policy1', action, decision))
                 return decision
 
         class PermissionPolicy2(Component):
@@ -438,23 +549,32 @@ class RecursivePolicyTestCase(unittest.TestCase):
 
             def __init__(self):
                 self.call_count = 0
+                self.decisions = cls.decisions
 
             def check_permission(self, action, username, resource, perm):
                 self.call_count += 1
                 decision = None
                 if action == 'ACTION_2':
                     decision = username == 'user2'
-                decisions.append(('policy2', action, decision))
+                self.decisions.append(('policy2', action, decision))
                 return decision
 
-        self.env.enable_component(PermissionPolicy1)
-        self.env.enable_component(PermissionPolicy2)
+        cls.permission_policies = [PermissionPolicy1, PermissionPolicy2]
+
+    @classmethod
+    def tearDownClass(cls):
+        from trac.core import ComponentMeta
+        for component in cls.permission_policies:
+            ComponentMeta.deregister(component)
+
+    def setUp(self):
+        self.__class__.decisions = []
+        self.env = EnvironmentStub(enable=self.permission_policies)
         self.env.config.set('trac', 'permission_policies',
                             'PermissionPolicy1, PermissionPolicy2')
         self.ps = perm.PermissionSystem(self.env)
 
     def tearDown(self):
-        self.env.restore_component_registry()
         self.env.reset_db()
 
     def test_user1_allowed_by_policy1(self):
@@ -462,7 +582,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
         for ACTION_2.
         """
         perm_cache = perm.PermissionCache(self.env, 'user1')
-        self.assertTrue('ACTION_1' in perm_cache)
+        self.assertIn('ACTION_1', perm_cache)
         self.assertEqual(2, self.ps.policies[0].call_count)
         self.assertEqual(1, self.ps.policies[1].call_count)
         self.assertEqual([
@@ -476,7 +596,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
         policy2 consulted for ACTION_2.
         """
         perm_cache = perm.PermissionCache(self.env, 'user2')
-        self.assertFalse('ACTION_1' in perm_cache)
+        self.assertNotIn('ACTION_1', perm_cache)
         self.assertEqual(2, self.ps.policies[0].call_count)
         self.assertEqual(2, self.ps.policies[1].call_count)
         self.assertEqual([
@@ -490,7 +610,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
         """policy1 consulted for ACTION_2. policy2 consulted for ACTION_2.
         """
         perm_cache = perm.PermissionCache(self.env, 'user1')
-        self.assertFalse('ACTION_2' in perm_cache)
+        self.assertNotIn('ACTION_2', perm_cache)
         self.assertEqual(1, self.ps.policies[0].call_count)
         self.assertEqual(1, self.ps.policies[1].call_count)
         self.assertEqual([
@@ -502,7 +622,7 @@ class RecursivePolicyTestCase(unittest.TestCase):
         """policy1 consulted for ACTION_2. policy2 consulted for ACTION_2.
         """
         perm_cache = perm.PermissionCache(self.env, 'user2')
-        self.assertTrue('ACTION_2' in perm_cache)
+        self.assertIn('ACTION_2', perm_cache)
         self.assertEqual(1, self.ps.policies[0].call_count)
         self.assertEqual(1, self.ps.policies[1].call_count)
         self.assertEqual([
@@ -511,6 +631,231 @@ class RecursivePolicyTestCase(unittest.TestCase):
         ], self.decisions)
 
 
+class TracAdminTestCase(TracAdminTestCaseBase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True)
+        self.admin = TracAdmin()
+        self.admin.env_set('', self.env)
+
+    def tearDown(self):
+        self.env.reset_db()
+        self.env = None
+
+    def test_permission_list_ok(self):
+        """Tests the 'permission list' command in trac-admin."""
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_list_includes_undefined_actions(self):
+        """Undefined actions are included in the User Action table,
+        but not in the Available Actions list.
+        """
+        self.env.disable_component(trac.search.web_ui.SearchModule)
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_add_one_action_ok(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes valid arguments to add one permission and checks for
+        success.
+        """
+        self.execute('permission add test_user WIKI_VIEW')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_add_multiple_actions_ok(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes valid arguments to add multiple permissions and checks for
+        success.
+        """
+        self.execute('permission add test_user LOG_VIEW FILE_VIEW')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_add_already_exists(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes a permission that already exists and checks for the
+        message. Other permissions passed are added.
+        """
+        rv, output = self.execute('permission add anonymous WIKI_CREATE '
+                                   'WIKI_VIEW WIKI_MODIFY')
+        self.assertEqual(0, rv, output)
+        rv, output2 = self.execute('permission list')
+        self.assertEqual(0, rv, output2)
+        self.assertExpectedResult(output + output2)
+
+    def test_permission_add_subject_already_in_group(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes a group that the subject is already a member of and
+        checks for the message. Other permissions passed are added.
+        """
+        rv, output1 = self.execute('permission add user1 group2')
+        self.assertEqual(0, rv, output1)
+        rv, output2 = self.execute('permission add user1 group1 group2 '
+                                    'group3')
+        self.assertEqual(0, rv, output2)
+        rv, output3 = self.execute('permission list')
+        self.assertEqual(0, rv, output3)
+        self.assertExpectedResult(output2 + output3)
+
+    def test_permission_add_differs_from_action_by_casing(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes a permission that differs from an action by casing and
+        checks for the message. None of the permissions in the list are
+        granted.
+        """
+        rv, output = self.execute('permission add joe WIKI_CREATE '
+                                   'Trac_Admin WIKI_MODIFY')
+        self.assertEqual(2, rv, output)
+        rv, output2 = self.execute('permission list')
+        self.assertEqual(0, rv, output2)
+        self.assertExpectedResult(output + output2)
+
+    def test_permission_add_unknown_action(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test tries granting NOT_A_PERM to a user. NOT_A_PERM does not exist
+        in the system. None of the permissions in the list are granted.
+        """
+        rv, output = self.execute('permission add joe WIKI_CREATE '
+                                   'NOT_A_PERM WIKI_MODIFY')
+        self.assertEqual(2, rv, output)
+        rv, output2 = self.execute('permission list')
+        self.assertEqual(0, rv, output2)
+        self.assertExpectedResult(output + output2)
+
+    def test_permission_remove_one_action_ok(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test passes valid arguments to remove one permission and checks for
+        success.
+        """
+        self.execute('permission remove anonymous TICKET_MODIFY')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_multiple_actions_ok(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test passes valid arguments to remove multiple permission and checks
+        for success.
+        """
+        self.execute('permission remove anonymous WIKI_CREATE WIKI_MODIFY')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_all_actions_for_user(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test removes all permissions for anonymous.
+        """
+        self.execute('permission remove anonymous *')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_action_for_all_users(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test removes the TICKET_CREATE permission from all users.
+        """
+        self.execute('permission add anonymous TICKET_CREATE')
+        self.execute('permission remove * TICKET_CREATE')
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_unknown_user(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing a permission from an unknown user.
+        """
+        rv, output = self.execute('permission remove joe TICKET_VIEW')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_action_not_granted(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing TICKET_CREATE from user anonymous, who doesn't
+        have that permission.
+        """
+        rv, output = self.execute('permission remove anonymous TICKET_CREATE')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_action_granted_through_meta_permission(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing WIKI_VIEW from a user. WIKI_VIEW has been granted
+        through user anonymous."""
+        self.execute('permission add joe TICKET_VIEW')
+        rv, output = self.execute('permission remove joe WIKI_VIEW')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_unknown_action(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing NOT_A_PERM from a user. NOT_A_PERM does not exist
+        in the system."""
+        rv, output = self.execute('permission remove joe NOT_A_PERM')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_unknown_action_granted(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing NOT_A_PERM from a user. NOT_A_PERM does not exist
+        in the system, but the user possesses the permission."""
+        self.env.db_transaction("""
+            INSERT INTO permission VALUES (%s, %s)
+        """, ('joe', 'NOT_A_PERM'))
+        rv, output = self.execute('permission remove joe NOT_A_PERM')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_export_ok(self):
+        """
+        Tests the 'permission export' command in trac-admin.  This particular
+        test exports the default permissions to stdout.
+        """
+        rv, output = self.execute('permission export')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_import_ok(self):
+        """
+        Tests the 'permission import' command in trac-admin.  This particular
+        test exports additional permissions, removes them and imports them back.
+        """
+        user = u'test_user\u0250'
+        self.execute('permission add ' + user + ' WIKI_VIEW')
+        self.execute('permission add ' + user + ' TICKET_VIEW')
+        rv, output = self.execute('permission export')
+        self.execute('permission remove ' + user + ' *')
+        rv, output = self.execute('permission import', input=output)
+        self.assertEqual(0, rv, output)
+        self.assertEqual('', output)
+        rv, output = self.execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(DefaultPermissionStoreTestCase))
@@ -519,6 +864,7 @@ def test_suite():
     suite.addTest(unittest.makeSuite(PermissionCacheTestCase))
     suite.addTest(unittest.makeSuite(PermissionPolicyTestCase))
     suite.addTest(unittest.makeSuite(RecursivePolicyTestCase))
+    suite.addTest(unittest.makeSuite(TracAdminTestCase))
     return suite
 
 
