@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2020 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,10 +14,10 @@
 import doctest
 import unittest
 
-from genshi.builder import tag
 from trac import resource
 from trac.core import Component, implements
 from trac.test import EnvironmentStub, MockRequest
+from trac.util.html import tag
 from trac.web.chrome import web_context
 
 
