@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,8 +13,8 @@
 
 import unittest
 
-from trac.core import TracError
-from trac.resource import ResourceNotFound
+from trac.perm import PermissionError, PermissionSystem
+from trac.resource import ResourceExistsError, ResourceNotFound
 from trac.test import EnvironmentStub, MockRequest
 from trac.ticket.admin import ComponentAdminPanel, MilestoneAdminPanel, \
                               PriorityAdminPanel, ResolutionAdminPanel, \
@@ -52,11 +52,10 @@ class ComponentAdminPanelTestCase(BaseTestCase):
         req = MockRequest(self.env, method='POST',
                           args={'name': ' component3 ', 'owner': owner,
                                 'add': True})
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             cap.render_admin_panel(req, 'ticket', 'component', None)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Component "component3" already exists', unicode(e))
+        self.assertIn('Component "component3" already exists',
+                      unicode(cm.exception))
 
     def test_add_component_with_spaces(self):
         cap = ComponentAdminPanel(self.env)
@@ -71,12 +70,10 @@ class ComponentAdminPanelTestCase(BaseTestCase):
                       req.chrome['notices'])
         component = Component(self.env, name)
         self.assertEqual(name, component.name)
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             cap.render_admin_panel(req, 'ticket', 'component', None)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Component "comp on ent 4" already exists',
-                          unicode(e))
+        self.assertIn('Component "comp on ent 4" already exists',
+                      unicode(cm.exception))
 
     def test_save_component(self):
         cap = ComponentAdminPanel(self.env)
@@ -91,11 +88,10 @@ class ComponentAdminPanelTestCase(BaseTestCase):
         req = MockRequest(self.env, method='POST',
                           args={'name': ' component1 ', 'owner': 'somebody',
                                 'save': True})
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             cap.render_admin_panel(req, 'ticket', 'component', old_name)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Component "component1" already exists', unicode(e))
+        self.assertIn('Component "component1" already exists',
+                      unicode(cm.exception))
 
         req = MockRequest(self.env, method='POST',
                           args={'name': ' comp \t on \t ent \t 2 ',
@@ -195,12 +191,10 @@ class MilestoneAdminPanelTestCase(BaseTestCase):
         milestone = Milestone(self.env, name)
         self.assertEqual(name, milestone.name)
 
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             map.render_admin_panel(req, 'ticket', 'milestone', None)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Milestone "mile stone 5" already exists',
-                          unicode(e))
+        self.assertIn('Milestone "mile stone 5" already exists',
+                      unicode(cm.exception))
 
     def test_save_milestone(self):
         map = MilestoneAdminPanel(self.env)
@@ -211,11 +205,10 @@ class MilestoneAdminPanelTestCase(BaseTestCase):
 
         req = MockRequest(self.env, method='POST',
                           args={'name': ' milestone1 ', 'save': True})
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             map.render_admin_panel(req, 'ticket', 'milestone', old_name)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Milestone "milestone1" already exists', unicode(e))
+        self.assertIn('Milestone "milestone1" already exists',
+                unicode(cm.exception))
 
         req = MockRequest(self.env, method='POST',
                           args={'name': ' mile \t stone \t 6 ', 'save': True})
@@ -230,31 +223,59 @@ class MilestoneAdminPanelTestCase(BaseTestCase):
         self.assertRaises(ResourceNotFound, Milestone, self.env, old_name)
 
     def test_set_default_milestone(self):
+        """Set default milestone."""
         name = 'milestone2'
         config_key = 'default_milestone'
-        map = MilestoneAdminPanel(self.env)
-        req = MockRequest(self.env, method='POST',
+        PermissionSystem(self.env).grant_permission('user1', 'TICKET_ADMIN')
+        req = MockRequest(self.env, authname='user1', method='POST',
                           args={'ticket_default': name, 'apply': True})
 
-        self.assertRaises(RequestDone, map.render_admin_panel, req,
-                          'ticket', 'milestone', None)
+        self.assertEqual('', self.env.config.get('ticket', config_key))
+        with self.assertRaises(RequestDone):
+            MilestoneAdminPanel(self.env).render_admin_panel(
+                req, 'ticket', 'milestone', None)
         self.assertIn('Your changes have been saved.', req.chrome['notices'])
         self.assertEqual([], req.chrome['warnings'])
         self.assertEqual(name, self.env.config.get('ticket', config_key))
 
+    def test_set_default_milestone_requires_ticket_admin(self):
+        """Setting default milestone requires TICKET_ADMIN."""
+        PermissionSystem(self.env).grant_permission('user1', 'MILESTONE_ADMIN')
+        req = MockRequest(self.env, authname='user1', method='POST',
+                          args={'ticket_default': 'milestone1', 'apply': True})
+
+        self.assertNotIn('TICKET_ADMIN', req.perm)
+        with self.assertRaises(PermissionError):
+            MilestoneAdminPanel(self.env).render_admin_panel(
+                req, 'ticket', 'milestone', None)
+
     def test_set_default_retarget_to(self):
+        """Set default retarget milestone."""
         name = 'milestone2'
         config_key = 'default_retarget_to'
-        map = MilestoneAdminPanel(self.env)
-        req = MockRequest(self.env, method='POST',
+        PermissionSystem(self.env).grant_permission('user1', 'TICKET_ADMIN')
+        req = MockRequest(self.env, authname='user1', method='POST',
                           args={'retarget_default': name, 'apply': True})
 
-        self.assertRaises(RequestDone, map.render_admin_panel, req,
-                          'ticket', 'milestone', None)
+        self.assertEqual('', self.env.config.get('ticket', config_key))
+        with self.assertRaises(RequestDone):
+            MilestoneAdminPanel(self.env).render_admin_panel(
+                req, 'ticket', 'milestone', None)
         self.assertIn('Your changes have been saved.', req.chrome['notices'])
         self.assertEqual([], req.chrome['warnings'])
         self.assertEqual(name, self.env.config.get('milestone', config_key))
 
+    def test_set_default_retarget_to_requires_ticket_admin(self):
+        """Setting default retarget milestone requires TICKET_ADMIN."""
+        PermissionSystem(self.env).grant_permission('user1', 'MILESTONE_ADMIN')
+        req = MockRequest(self.env, authname='user1', method='POST',
+                          args={'retarget_to': 'milestone1', 'apply': True})
+
+        self.assertNotIn('TICKET_ADMIN', req.perm)
+        with self.assertRaises(PermissionError):
+            MilestoneAdminPanel(self.env).render_admin_panel(
+                req, 'ticket', 'milestone', None)
+
     def test_remove_default_milestone(self):
         name = 'milestone2'
         map = MilestoneAdminPanel(self.env)
@@ -283,89 +304,124 @@ class MilestoneAdminPanelTestCase(BaseTestCase):
         self.assertEqual('/trac.cgi/query?group=status&milestone=blah',
                          data['query_href']('blah'))
 
+    def test_user_with_milestone_admin_can_view(self):
+        """User with MILESTONE_ADMIN can view."""
+        PermissionSystem(self.env).grant_permission('user1', 'MILESTONE_ADMIN')
+        req = MockRequest(self.env, authname='user1')
+        rv = MilestoneAdminPanel(self.env).get_admin_panels(req)
+        self.assertEqual([('ticket', "Ticket System", 'milestones',
+                           "Milestones")], list(rv))
+
+    def test_user_with_ticket_admin_can_view(self):
+        """User with MILESTONE_VIEW and TICKET_ADMIN can view."""
+        PermissionSystem(self.env).grant_permission('user1', 'TICKET_ADMIN')
+        req = MockRequest(self.env, authname='user1')
+        rv = MilestoneAdminPanel(self.env).get_admin_panels(req)
+        self.assertEqual([('ticket', "Ticket System", 'milestones',
+                           "Milestones")], list(rv))
+
+    def test_user_without_milestone_or_ticket_admin_cannot_view(self):
+        """User without MILESTONE_ADMIN or TICKET_ADMIN cannot view."""
+        req = MockRequest(self.env, authname='user1')
+        rv = MilestoneAdminPanel(self.env).get_admin_panels(req)
+        self.assertEqual([], list(rv))
+
 
 class AbstractEnumTestCase(BaseTestCase):
 
+    admin = None
     type = None
     type_name = None
     cls = None
 
-    def _test_add(self, panel, name, norm_name=None):
+    def _test_add(self, name, norm_name=None):
         if norm_name is None:
             norm_name = name
         req = MockRequest(self.env, method='POST',
                           args={'name': name, 'add': True})
 
         self.assertRaises(ResourceNotFound, self.cls, self.env, norm_name)
-        self.assertRaises(RequestDone, panel.render_admin_panel, req,
+        self.assertRaises(RequestDone, self.admin.render_admin_panel, req,
                           'ticket', self.type, None)
         self.assertIn('The %s value "%s" has been added.' %
                       (self.type_name, norm_name), req.chrome['notices'])
         item = self.cls(self.env, norm_name)
         self.assertEqual(norm_name, item.name)
 
-    def _test_add_non_unique(self, panel, name, norm_name=None):
+    def _test_add_non_unique(self, name, norm_name=None):
         if norm_name is None:
             norm_name = name
         req = MockRequest(self.env, method='POST',
                           args={'name': name, 'add': True})
 
-        try:
-            panel.render_admin_panel(req, 'ticket', self.type, None)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('%s value "%s" already exists' %
-                          (self.type_name, norm_name), unicode(e))
+        with self.assertRaises(ResourceExistsError) as cm:
+            self.admin.render_admin_panel(req, 'ticket', self.type, None)
+        self.assertIn('%s value "%s" already exists' %
+                      (self.type_name, norm_name), unicode(cm.exception))
 
-    def _test_save(self, panel, old_name, name, norm_name=None):
+    def _test_save(self, old_name, name, norm_name=None):
         if norm_name is None:
             norm_name = name
         req = MockRequest(self.env, method='POST',
                           args={'name': name, 'save': True})
 
-        self.assertRaises(RequestDone, panel.render_admin_panel, req,
+        self.assertRaises(RequestDone, self.admin.render_admin_panel, req,
                           'ticket', self.type, old_name)
         item = self.cls(self.env, norm_name)
         self.assertEqual(norm_name, item.name)
 
-    def _test_save_non_unique(self, panel, old_name, name, norm_name=None):
+    def _test_save_non_unique(self, old_name, name, norm_name=None):
         if norm_name is None:
             norm_name = name
         req = MockRequest(self.env, method='POST',
                           args={'name': name, 'save': True})
-        try:
-            panel.render_admin_panel(req, 'ticket', self.type, old_name)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('value "%s" already exists' % norm_name, unicode(e))
+        with self.assertRaises(ResourceExistsError) as cm:
+            self.admin.render_admin_panel(req, 'ticket', self.type, old_name)
+        self.assertIn('value "%s" already exists' % norm_name,
+                      unicode(cm.exception))
 
-    def _test_set_default(self, panel, name):
+    def _test_set_default(self, name):
         config_key = 'default_' + self.type
         req = MockRequest(self.env, method='POST',
                           args={'default': name, 'apply': True})
         for item in self.cls.select(self.env):
             req.args.update({'value_' + str(item.value): str(item.value)})
 
-        self.assertRaises(RequestDone, panel.render_admin_panel, req,
+        self.assertRaises(RequestDone, self.admin.render_admin_panel, req,
                           'ticket', self.type, None)
         self.assertIn('Your changes have been saved.', req.chrome['notices'])
         self.assertEqual([], req.chrome['warnings'])
         self.assertEqual(name, self.env.config.get('ticket', config_key))
 
-    def _test_remove_default(self, panel, name):
+    def _test_remove_default(self, name):
         config_key = 'default_' + self.type
         self.env.config.set('ticket', config_key, name)
 
         req = MockRequest(self.env, method='POST',
                           args={'sel': name, 'remove': True})
 
-        self.assertRaises(RequestDone, panel.render_admin_panel, req,
+        self.assertRaises(RequestDone, self.admin.render_admin_panel, req,
                           'ticket', self.type, None)
         self.assertIn('The selected %s values have been removed.' %
                       self.type_name, req.chrome['notices'])
         self.assertEqual([], req.chrome['warnings'])
         self.assertEqual('', self.env.config.get('ticket', config_key))
 
+    def _test_edit_description(self, name):
+        description = 'the edit'
+        req = MockRequest(self.env, method='POST',
+                          path_info='/admin/ticket/%s/%s' % (self.type, name),
+                          args={'name': name, 'description': description,
+                                'save': True})
+
+        self.assertEqual(1, len(list(self.admin.get_admin_panels(req))))
+        with self.assertRaises(RequestDone):
+            self.admin.render_admin_panel(req, 'ticket', self.type, name)
+
+        self.assertIn("Your changes have been saved",
+                      unicode(req.chrome['notices']))
+        self.assertEqual(description, self.cls(self.env, name).description)
+
 
 class PriorityAdminPanelTestCase(AbstractEnumTestCase):
 
@@ -373,28 +429,30 @@ class PriorityAdminPanelTestCase(AbstractEnumTestCase):
     type_name = 'Priority'
     cls = Priority
 
+    def setUp(self):
+        super(PriorityAdminPanelTestCase, self).setUp()
+        self.admin = PriorityAdminPanel(self.env)
+
     def test_add_priority(self):
-        ap = PriorityAdminPanel(self.env)
-        self._test_add(ap, 'priority 1')
-        self._test_add(ap, ' prio \t rity \t 2 ', 'prio rity 2')
-        self._test_add_non_unique(ap, 'critical')
-        self._test_add_non_unique(ap, ' priority \t 1 ', 'priority 1')
+        self._test_add('priority 1')
+        self._test_add(' prio \t rity \t 2 ', 'prio rity 2')
+        self._test_add_non_unique('critical')
+        self._test_add_non_unique(' priority \t 1 ', 'priority 1')
 
     def test_save_priority(self):
-        ap = PriorityAdminPanel(self.env)
-        self._test_save(ap, 'critical', 'critical!!')
-        self._test_save(ap, 'critical!!', ' crit \t ical ', 'crit ical')
-        self._test_save_non_unique(ap, 'crit ical', 'blocker')
-        self._test_save_non_unique(ap, 'blocker',
-                                   ' crit \t ical ', 'crit ical')
+        self._test_save('critical', 'critical!!')
+        self._test_save('critical!!', ' crit \t ical ', 'crit ical')
+        self._test_save_non_unique('crit ical', 'blocker')
+        self._test_save_non_unique('blocker', ' crit \t ical ', 'crit ical')
 
     def test_set_default_priority(self):
-        ap = PriorityAdminPanel(self.env)
-        self._test_set_default(ap, 'critical')
+        self._test_set_default('critical')
 
     def test_remove_default_priority(self):
-        ap = PriorityAdminPanel(self.env)
-        self._test_remove_default(ap, 'critical')
+        self._test_remove_default('critical')
+
+    def test_edit_description(self):
+        self._test_edit_description('critical')
 
 
 class ResolutionAdminPanelTestCase(AbstractEnumTestCase):
@@ -403,27 +461,31 @@ class ResolutionAdminPanelTestCase(AbstractEnumTestCase):
     type_name = 'Resolution'
     cls = Resolution
 
+    def setUp(self):
+        super(ResolutionAdminPanelTestCase, self).setUp()
+        self.admin = ResolutionAdminPanel(self.env)
+
     def test_add_resolution(self):
-        ap = ResolutionAdminPanel(self.env)
-        self._test_add(ap, 'resolution 1')
-        self._test_add(ap, ' resol \t ution 2 ', 'resol ution 2')
-        self._test_add_non_unique(ap, 'fixed')
-        self._test_add_non_unique(ap, ' resolution \t 1 ', 'resolution 1')
+        self._test_add('resolution 1')
+        self._test_add(' resol \t ution 2 ', 'resol ution 2')
+        self._test_add_non_unique('fixed')
+        self._test_add_non_unique(' resolution \t 1 ', 'resolution 1')
 
     def test_save_resolution(self):
         ap = ResolutionAdminPanel(self.env)
-        self._test_save(ap, 'invalid', 'invalid!!')
-        self._test_save(ap, 'invalid!!', ' in \t valid ', 'in valid')
-        self._test_save_non_unique(ap, 'wontfix', 'fixed')
-        self._test_save_non_unique(ap, 'wontfix', ' in \t valid ', 'in valid')
+        self._test_save('invalid', 'invalid!!')
+        self._test_save('invalid!!', ' in \t valid ', 'in valid')
+        self._test_save_non_unique('wontfix', 'fixed')
+        self._test_save_non_unique('wontfix', ' in \t valid ', 'in valid')
 
     def test_set_default_resolution(self):
-        ap = ResolutionAdminPanel(self.env)
-        self._test_set_default(ap, 'invalid')
+        self._test_set_default('invalid')
 
     def test_remove_default_resolution(self):
-        ap = ResolutionAdminPanel(self.env)
-        self._test_remove_default(ap, 'invalid')
+        self._test_remove_default('invalid')
+
+    def test_edit_description(self):
+        self._test_edit_description('invalid')
 
 
 class SeverityAdminPanelTestCase(AbstractEnumTestCase):
@@ -432,38 +494,47 @@ class SeverityAdminPanelTestCase(AbstractEnumTestCase):
     type_name = 'Severity'
     cls = Severity
 
+    def setUp(self):
+        super(SeverityAdminPanelTestCase, self).setUp()
+        self.admin = SeverityAdminPanel(self.env)
+
     def test_add_severity(self):
-        ap = SeverityAdminPanel(self.env)
-        self._test_add(ap, 'severity 1')
-        self._test_add(ap, ' seve  rity  2 ', 'seve rity 2')
-        self._test_add_non_unique(ap, 'severity 1')
-        self._test_add_non_unique(ap, ' seve \t rity \t 2 ', 'seve rity 2')
+        self._test_add('severity 1')
+        self._test_add(' seve  rity  2 ', 'seve rity 2')
+        self._test_add_non_unique('severity 1')
+        self._test_add_non_unique(' seve \t rity \t 2 ', 'seve rity 2')
 
     def test_save_severity(self):
-        self.env.db_transaction.executemany("""
-            INSERT INTO enum (type, name, value) VALUES ('severity',%s,%s)
-            """, [('severity 1', 1), ('severity 2', 2),
-                  ('severity 3', 3), ('severity 4', 4)])
-        ap = SeverityAdminPanel(self.env)
-        self._test_save(ap, 'severity 1', 'severity 42')
-        self._test_save(ap, 'severity 2', ' severity \t z ', 'severity z')
-        self._test_save_non_unique(ap, 'severity 3', 'severity 4')
-        self._test_save_non_unique(ap, 'severity 4', ' severity \t 3 ',
+        with self.env.db_transaction:
+            self._insert_severity('severity 1')
+            self._insert_severity('severity 2')
+            self._insert_severity('severity 3')
+            self._insert_severity('severity 4')
+        self._test_save('severity 1', 'severity 42')
+        self._test_save('severity 2', ' severity \t z ', 'severity z')
+        self._test_save_non_unique('severity 3', 'severity 4')
+        self._test_save_non_unique('severity 4', ' severity \t 3 ',
                                    'severity 3')
 
-    def test_set_default_severity(self):
+    def _insert_severity(self, name):
         s = Severity(self.env)
-        s.name = 'severity 1'
+        s.name = name
         s.insert()
-        ap = SeverityAdminPanel(self.env)
-        self._test_set_default(ap, 'severity 1')
+
+    def test_add_severity(self):
+        self._test_add('severity 1')
+
+    def test_set_default_severity(self):
+        self._insert_severity('severity 1')
+        self._test_set_default('severity 1')
 
     def test_remove_default_severity(self):
-        s = Severity(self.env)
-        s.name = 'severity 1'
-        s.insert()
-        ap = SeverityAdminPanel(self.env)
-        self._test_remove_default(ap, 'severity 1')
+        self._insert_severity('severity 1')
+        self._test_remove_default('severity 1')
+
+    def test_edit_description(self):
+        self._insert_severity('severity 1')
+        self._test_edit_description('severity 1')
 
 
 class TicketTypeAdminPanelTestCase(AbstractEnumTestCase):
@@ -472,27 +543,30 @@ class TicketTypeAdminPanelTestCase(AbstractEnumTestCase):
     type_name = 'Ticket Type'
     cls = Type
 
+    def setUp(self):
+        super(TicketTypeAdminPanelTestCase, self).setUp()
+        self.admin = TicketTypeAdminPanel(self.env)
+
     def test_add_type(self):
-        ap = TicketTypeAdminPanel(self.env)
-        self._test_add(ap, 'improvement')
-        self._test_add(ap, ' new \t feature ', 'new feature')
-        self._test_add_non_unique(ap, 'task')
-        self._test_add_non_unique(ap, ' new \t feature ', 'new feature')
+        self._test_add('improvement')
+        self._test_add(' new \t feature ', 'new feature')
+        self._test_add_non_unique('task')
+        self._test_add_non_unique(' new \t feature ', 'new feature')
 
     def test_save_severity(self):
-        ap = TicketTypeAdminPanel(self.env)
-        self._test_save(ap, 'defect', 'bug')
-        self._test_save(ap, 'task', ' new \t task ', 'new task')
-        self._test_save_non_unique(ap, 'bug', 'enhancement')
-        self._test_save_non_unique(ap, 'bug', ' new \t task ', 'new task')
+        self._test_save('defect', 'bug')
+        self._test_save('task', ' new \t task ', 'new task')
+        self._test_save_non_unique('bug', 'enhancement')
+        self._test_save_non_unique('bug', ' new \t task ', 'new task')
 
     def test_set_default_type(self):
-        ap = TicketTypeAdminPanel(self.env)
-        self._test_set_default(ap, 'task')
+        self._test_set_default('task')
 
     def test_remove_default_type(self):
-        ap = TicketTypeAdminPanel(self.env)
-        self._test_remove_default(ap, 'task')
+        self._test_remove_default('task')
+
+    def test_edit_description(self):
+        self._test_edit_description('task')
 
 
 class VersionAdminPanelTestCase(BaseTestCase):
@@ -523,11 +597,10 @@ class VersionAdminPanelTestCase(BaseTestCase):
         version = Version(self.env, name)
         self.assertEqual(name, version.name)
 
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             ap.render_admin_panel(req, 'ticket', 'version', None)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Version "4.0 dev" already exists', unicode(e))
+        self.assertIn('Version "4.0 dev" already exists',
+                      unicode(cm.exception))
 
     def test_save_version(self):
         ap = VersionAdminPanel(self.env)
@@ -538,11 +611,9 @@ class VersionAdminPanelTestCase(BaseTestCase):
 
         req = MockRequest(self.env, method='POST',
                           args={'name': ' 1.0 ', 'save': True})
-        try:
+        with self.assertRaises(ResourceExistsError) as cm:
             ap.render_admin_panel(req, 'ticket', 'version', old_name)
-            self.fail('TracError not raised')
-        except TracError as e:
-            self.assertIn('Version "1.0" already exists', unicode(e))
+        self.assertIn('Version "1.0" already exists', unicode(cm.exception))
 
         req = MockRequest(self.env, method='POST',
                           args={'name': ' 4.0 \t dev ', 'save': True})
