@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,6 +19,7 @@ from trac.test import EnvironmentStub, MockRequest
 from trac.ticket import model
 from trac.ticket.api import TicketSystem
 from trac.ticket.model import Milestone, Ticket, Version
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import datetime_now, utc
 
 import unittest
@@ -37,14 +38,12 @@ class TicketSystemTestCase(unittest.TestCase):
 
     def _get_actions(self, ticket_dict):
         ts = TicketSystem(self.env)
-        ticket = Ticket(self.env)
-        ticket.populate(ticket_dict)
-        id = ticket.insert()
-        return ts.get_available_actions(self.req, Ticket(self.env, id))
+        ticket = insert_ticket(self.env, **ticket_dict)
+        return ts.get_available_actions(self.req, Ticket(self.env, ticket.id))
 
     def _get_ticket_field(self, field_name):
         fields = TicketSystem(self.env).get_ticket_fields()
-        return (i for i in fields if i['name'] == field_name).next()
+        return next((i for i in fields if i['name'] == field_name))
 
     def test_custom_field_text(self):
         self.env.config.set('ticket-custom', 'test', 'text')
@@ -53,8 +52,8 @@ class TicketSystemTestCase(unittest.TestCase):
         self.env.config.set('ticket-custom', 'test.format', 'wiki')
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'test', 'type': 'text', 'label': 'Test',
-                          'value': 'Foo bar', 'order': 0, 'format': 'wiki',
-                          'custom': True},
+                          'value': 'Foo bar', 'max_size': 0, 'order': 0,
+                          'format': 'wiki', 'custom': True},
                          fields[0])
 
     def test_custom_field_select(self):
@@ -95,7 +94,7 @@ class TicketSystemTestCase(unittest.TestCase):
         fields = TicketSystem(self.env).get_custom_fields()
         self.assertEqual({'name': 'test', 'type': 'textarea', 'label': 'Test',
                           'value': 'Foo bar', 'height': 4, 'order': 0,
-                          'format': 'wiki', 'custom': True},
+                          'max_size': 0, 'format': 'wiki', 'custom': True},
                          fields[0])
 
     def test_description_field(self):
@@ -146,27 +145,48 @@ class TicketSystemTestCase(unittest.TestCase):
         ticket_custom.set('FiEld5', 'text')
 
         ts = TicketSystem(self.env)
+        custom_fields = ts.custom_fields
+        fields = ts.fields
 
-        self.assertEqual(2, len(ts.custom_fields))
-        self.assertIsNotNone(ts.custom_fields.by_name('field4'))
-        self.assertIsNotNone(ts.custom_fields.by_name('field5'))
-        self.assertIsNotNone(ts.fields.by_name('field4'))
-        self.assertIsNotNone(ts.fields.by_name('field5'))
+        self.assertEqual(2, len(custom_fields))
+        self.assertIsNotNone(custom_fields.by_name('field4'))
+        self.assertIsNotNone(custom_fields.by_name('field5'))
+        self.assertIsNotNone(fields.by_name('field4'))
+        self.assertIsNotNone(fields.by_name('field5'))
+        self.assertIn(
+            ('WARNING', u'Invalid name for custom field: "_field1" (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING', u'Invalid name for custom field: "2field" (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING', u'Invalid name for custom field: "f3%^&*" (ignoring)'),
+            self.env.log_messages)
 
     def test_custom_field_with_reserved_name(self):
         ticket_custom = self.env.config['ticket-custom']
         ticket_custom.set('owner', 'select')
+        ticket_custom.set('owner.options', 'u1|u2|u3')
         ticket_custom.set('description', 'text')
 
         ts = TicketSystem(self.env)
+        custom_fields = ts.custom_fields
 
+        self.assertIn(
+            ('WARNING',
+             u'Field name "owner" is a reserved name (ignoring)'),
+            self.env.log_messages)
+        self.assertIn(
+            ('WARNING',
+             u'Field name "description" is a reserved name (ignoring)'),
+            self.env.log_messages)
         self.assertEqual({'name': 'owner', 'label': 'Owner', 'type': 'text'},
                          ts.fields.by_name('owner'))
         self.assertEqual({'name': 'description', 'label': 'Description',
                           'type': 'textarea', 'format': 'wiki'},
                          ts.fields.by_name('description'))
-        self.assertIsNone(ts.custom_fields.by_name('owner'))
-        self.assertIsNone(ts.custom_fields.by_name('description'))
+        self.assertIsNone(custom_fields.by_name('owner'))
+        self.assertIsNone(custom_fields.by_name('description'))
 
     def test_custom_field_order(self):
         self.env.config.set('ticket-custom', 'test1', 'text')
@@ -379,7 +399,7 @@ class TicketSystemTestCase(unittest.TestCase):
                          updated_milestone_field['options'])
 
     def test_resource_exists_valid_resource_id(self):
-        Ticket(self.env).insert()
+        insert_ticket(self.env)
         r1 = Resource('ticket', 1)
         r2 = Resource('ticket', 2)
 
