@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -24,6 +24,7 @@ from trac.tests.functional import FunctionalTwillTestCaseSetup, b, \
                                   internal_error, tc
 from trac.util.datefmt import datetime_now, format_date, format_datetime, \
                               localtz, utc
+from trac.util.html import escape
 
 
 class AdminEnumDefaultTestCaseSetup(FunctionalTwillTestCaseSetup):
@@ -35,14 +36,14 @@ class AdminEnumDefaultTestCaseSetup(FunctionalTwillTestCaseSetup):
         tc.formvalue('enumtable', 'default', name)
         tc.submit('apply')
         tc.url(url + '$')
-        tc.find('radio.*"%s"\\schecked="checked"' % name)
+        tc.find('radio.*checked="checked" value="%s"' % name)
         # Test the "Clear default" button
         tc.go(url)
         tc.submit('clear', formname='enumtable')
         tc.url(url + '$')
         tc.notfind(internal_error)
-        tc.find('<input type="radio" name="default" value="[^>]+" />')
-        tc.notfind('type="radio" name="default" value=".+" checked="checked"')
+        tc.find('<input type="radio" name="default" value="[^>]+"/>')
+        tc.notfind('type="radio" name="default" checked="checked" value=".+"')
 
 
 class TestAdminComponent(FunctionalTwillTestCaseSetup):
@@ -102,7 +103,7 @@ class TestAdminComponentDefault(FunctionalTwillTestCaseSetup):
         tc.go(component_url)
         tc.formvalue('component_table', 'default', name)
         tc.submit('apply')
-        tc.find('type="radio" name="default" value="%s" checked="checked"' % \
+        tc.find('type="radio" name="default" checked="checked" value="%s"' % \
                 name)
         tc.go(self._tester.url + '/newticket')
         tc.find('<option selected="selected" value="%s">%s</option>'
@@ -111,9 +112,9 @@ class TestAdminComponentDefault(FunctionalTwillTestCaseSetup):
         self._testenv.set_config('ticket', 'allowed_empty_fields', 'component')
         tc.go(component_url)
         tc.submit('clear', formname='component_table')
-        tc.notfind('type="radio" name="default" value=".+" checked="checked"')
+        tc.notfind('type="radio" name="default" checked="checked" value=".+"')
         self._tester.create_ticket()
-        tc.find('<th id="h_component" class="missing">\s*Component:\s*</th>'
+        tc.find('<th class="missing" id="h_component">\s*Component:\s*</th>'
                 '\s*<td headers="h_component">\s*</td>')
         self._testenv.remove_config('ticket', 'allowed_empty_fields')
 
@@ -206,13 +207,49 @@ class TestAdminMilestoneListing(FunctionalTwillTestCaseSetup):
         tc.url(milestone_url)
         tc.find(r'<a href="/admin/ticket/milestones/%(name)s">%(name)s</a>'
                 % {'name': name1})
-        tc.find(r'<a href="/query\?group=status&amp;milestone=%(name)s">'
-                r'1</a>' % {'name': name1})
+        m1_query_link = r'<a href="/query\?group=status&amp;' \
+                        r'milestone=%(name)s">1</a>' % {'name': name1}
+        tc.find(m1_query_link)
         tc.find(r'<a href="/admin/ticket/milestones/%(name)s">%(name)s</a>'
                 % {'name': name2})
         tc.notfind(r'<a href="/query\?group=status&amp;milestone=%(name)s">'
                    r'0</a>' % {'name': name2})
 
+        apply_submit = '<input type="submit" name="apply" ' \
+                       'value="Apply changes" />'
+        clear_submit = '<input type="submit"[ \t\n]+title="Clear default ' \
+                       'ticket milestone and default retargeting milestone"' \
+                       '[ \t\n]+name="clear" value="Clear defaults" />'
+        tc.find(apply_submit)
+        tc.find(clear_submit)
+        tc.find('<input type="radio" name="ticket_default" value="%(name)s"/>'
+                % {'name': name1})
+        tc.find('<input type="radio" name="retarget_default" value="%(name)s"/>'
+                % {'name': name1})
+
+        # TICKET_ADMIN is required to change the ticket default and retarget
+        # default configuration options. TICKET_VIEW is required for the
+        # milestone tickets query link to be present.
+        try:
+            self._testenv.grant_perm('user', 'MILESTONE_ADMIN')
+            self._testenv.revoke_perm('anonymous', 'TICKET_VIEW')
+            self._tester.go_to_front()
+            self._tester.logout()
+            self._tester.login('user')
+            tc.go(milestone_url)
+            tc.notfind(apply_submit)
+            tc.notfind(clear_submit)
+            tc.find('<input type="radio" name="ticket_default" '
+                    'disabled="disabled" value="%(name)s"/>' % {'name': name1})
+            tc.find('<input type="radio" name="retarget_default" '
+                    'disabled="disabled" value="%(name)s"/>' % {'name': name1})
+            tc.notfind(m1_query_link)
+        finally:
+            self._testenv.revoke_perm('user', 'MILESTONE_ADMIN')
+            self._testenv.grant_perm('anonymous', 'TICKET_VIEW')
+            self._tester.logout()
+            self._tester.login('admin')
+
 
 class TestAdminMilestoneDetail(FunctionalTwillTestCaseSetup):
     def runTest(self):
@@ -381,7 +418,7 @@ class TestAdminMilestoneRemove(FunctionalTwillTestCaseSetup):
         tc.url(milestone_url + '$')
         tc.notfind(name)
         self._tester.go_to_ticket(tid)
-        tc.find('<th id="h_milestone" class="missing">'
+        tc.find('<th class="missing" id="h_milestone">'
                 '[ \t\n]*Milestone:[ \t\n]*</th>')
         tc.find('<th class="trac-field-milestone">Milestone:</th>[ \t\n]+'
                 '<td>[ \t\n]+'
@@ -395,18 +432,18 @@ class TestAdminMilestoneRemoveMulti(FunctionalTwillTestCaseSetup):
         """Admin remove multiple milestones"""
         name = "MultiRemoveMilestone"
         count = 3
-        for i in range(count):
+        for i in xrange(count):
             self._tester.create_milestone("%s%s" % (name, i))
         milestone_url = self._tester.url + '/admin/ticket/milestones'
         tc.go(milestone_url)
         tc.url(milestone_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.find("%s%s" % (name, i))
-        for i in range(count):
+        for i in xrange(count):
             tc.formvalue('milestone_table', 'sel', "%s%s" % (name, i))
         tc.submit('remove')
         tc.url(milestone_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.notfind("%s%s" % (name, i))
 
 
@@ -428,11 +465,11 @@ class TestAdminMilestoneDefaults(FunctionalTwillTestCaseSetup):
             tc.go(milestone_url)
             tc.submit('clear', formname='milestone_table')
             tc.notfind('type="radio" name="ticket_default" '
-                       'value=".+" checked="checked"')
+                       'checked="checked" value=".+"')
             tc.notfind('type="radio" name="retarget_default" '
-                       'value=".+" checked="checked"')
+                       'checked="checked value=".+""')
             self._tester.go_to_ticket(tid)
-            tc.find('<th id="h_milestone" class="missing">[ \t\n]+'
+            tc.find('<th class="missing" id="h_milestone">[ \t\n]+'
                     'Milestone:[ \t\n]+</th>[ \t\n]+'
                     '(?!<td headers="h_milestone">)')
             self._tester.go_to_milestone(mid2)
@@ -450,10 +487,10 @@ class TestAdminMilestoneDefaults(FunctionalTwillTestCaseSetup):
         tc.go(milestone_url)
         tc.formvalue('milestone_table', 'ticket_default', mid1)
         tc.submit('apply')
-        tc.find('type="radio" name="ticket_default" value="%s" '
-                'checked="checked"' % mid1)
-        tc.notfind('type="radio" name="retarget_default" value=".+" '
-                   'checked="checked"')
+        tc.find('type="radio" name="ticket_default"'
+                ' checked="checked" value="%s"' % mid1)
+        tc.notfind('type="radio" name="retarget_default"'
+                   ' checked="checked" value=".+"')
         # verify it is the default on the newticket page.
         tc.go(self._tester.url + '/newticket')
         tc.find('<option selected="selected" value="%s">%s</option>'
@@ -464,10 +501,10 @@ class TestAdminMilestoneDefaults(FunctionalTwillTestCaseSetup):
         tc.go(milestone_url)
         tc.formvalue('milestone_table', 'retarget_default', mid1)
         tc.submit('apply')
-        tc.find('type="radio" name="retarget_default" value="%s" '
-                'checked="checked"' % mid1)
-        tc.notfind('type="radio" name="ticket_default" value=".+" '
-                   'checked="checked"')
+        tc.find('type="radio" name="retarget_default"'
+                ' checked="checked" value="%s"' % mid1)
+        tc.notfind('type="radio" name="ticket_default"'
+                   ' checked="checked" value=".+"')
         # verify it is the default on the confirm delete page.
         self._tester.go_to_milestone(mid2)
         tc.submit(formname='deletemilestone')
@@ -480,10 +517,10 @@ class TestAdminMilestoneDefaults(FunctionalTwillTestCaseSetup):
         tc.formvalue('milestone_table', 'ticket_default', mid1)
         tc.formvalue('milestone_table', 'retarget_default', mid1)
         tc.submit('apply')
-        tc.find('type="radio" name="ticket_default" value="%s" '
-                'checked="checked"' % mid1)
-        tc.find('type="radio" name="retarget_default" value="%s" '
-                'checked="checked"' % mid1)
+        tc.find('type="radio" name="ticket_default"'
+                ' checked="checked" value="%s"' % mid1)
+        tc.find('type="radio" name="retarget_default"'
+                ' checked="checked" value="%s"' % mid1)
         # verify it is the default on the newticket page.
         tc.go(self._tester.url + '/newticket')
         tc.find('<option selected="selected" value="%s">%s</option>'
@@ -498,13 +535,13 @@ class TestAdminMilestoneDefaults(FunctionalTwillTestCaseSetup):
         #Set neither
         tc.go(milestone_url)
         tc.submit('apply', formname='milestone_table')
-        tc.notfind('type="radio" name="retarget_default" value=".+" '
-                   'checked="checked"')
-        tc.notfind('type="radio" name="ticket_default" value=".+" '
-                   'checked="checked"')
+        tc.notfind('type="radio" name="retarget_default"'
+                   ' checked="checked" value=".+"')
+        tc.notfind('type="radio" name="ticket_default"'
+                   ' checked="checked" value=".+"')
         # verify no default on the newticket page.
         tc.go(self._tester.url + '/newticket')
-        tc.find('<th id="h_milestone" class="missing">[ \t\n]+'
+        tc.find('<th class="missing" id="h_milestone">[ \t\n]+'
                 'Milestone:[ \t\n]+</th>[ \t\n]+'
                 '(?!<td headers="h_milestone">)')
         # verify none selected on the confirm delete page.
@@ -573,18 +610,18 @@ class TestAdminPriorityRemoveMulti(FunctionalTwillTestCaseSetup):
         """Admin remove multiple priorities"""
         name = "MultiRemovePriority"
         count = 3
-        for i in range(count):
+        for i in xrange(count):
             self._tester.create_priority("%s%s" % (name, i))
         priority_url = self._tester.url + '/admin/ticket/priority'
         tc.go(priority_url)
         tc.url(priority_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.find("%s%s" % (name, i))
-        for i in range(count):
+        for i in xrange(count):
             tc.formvalue('enumtable', 'sel', "%s%s" % (name, i))
         tc.submit('remove')
         tc.url(priority_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.notfind("%s%s" % (name, i))
 
 
@@ -693,7 +730,7 @@ class TestAdminResolutionDuplicates(FunctionalTwillTestCaseSetup):
         name = "DuplicateResolution"
         self._tester.create_resolution(name)
         self._tester.create_resolution(name)
-        tc.find('Resolution value "%s" already exists' % name)
+        tc.find(escape('Resolution value "%s" already exists' % name))
 
 
 class TestAdminResolutionDefault(AdminEnumDefaultTestCaseSetup):
@@ -723,7 +760,7 @@ class TestAdminSeverityDuplicates(FunctionalTwillTestCaseSetup):
         name = "DuplicateSeverity"
         self._tester.create_severity(name)
         self._tester.create_severity(name)
-        tc.find('Severity value "%s" already exists' % name)
+        tc.find(escape('Severity value "%s" already exists' % name))
 
 
 class TestAdminSeverityDefault(AdminEnumDefaultTestCaseSetup):
@@ -753,7 +790,7 @@ class TestAdminTypeDuplicates(FunctionalTwillTestCaseSetup):
         name = "DuplicateType"
         self._tester.create_type(name)
         self._tester.create_type(name)
-        tc.find('Type value "%s" already exists' % name)
+        tc.find(escape('Type value "%s" already exists' % name))
 
 
 class TestAdminTypeDefault(AdminEnumDefaultTestCaseSetup):
@@ -787,7 +824,7 @@ class TestAdminVersionDuplicates(FunctionalTwillTestCaseSetup):
         tc.formvalue('addversion', 'name', name)
         tc.submit()
         tc.notfind(internal_error)
-        tc.find('Version "%s" already exists.' % name)
+        tc.find(escape('Version "%s" already exists.' % name))
 
 
 class TestAdminVersionDetail(FunctionalTwillTestCaseSetup):
@@ -828,8 +865,7 @@ class TestAdminVersionDetailTime(FunctionalTwillTestCaseSetup):
 
         # Empty time value is not automatically populated.
         tc.follow(name)
-        tc.find('<input type="text" id="releaseddate"')
-        tc.notfind('<input type="text" id="releaseddate"[^>]*value="')
+        tc.find('<input type="text" id="releaseddate"[^>]*value=""')
         tc.submit('save', formname="edit")
         tc.url(version_admin + '$')
         tc.find(name + '(<[^>]*>|\\s)*<[^>]* name="default" value="%s"'
@@ -872,18 +908,18 @@ class TestAdminVersionRemoveMulti(FunctionalTwillTestCaseSetup):
         """Admin remove multiple versions"""
         name = "MultiRemoveVersion"
         count = 3
-        for i in range(count):
+        for i in xrange(count):
             self._tester.create_version("%s%s" % (name, i))
         version_url = self._tester.url + '/admin/ticket/versions'
         tc.go(version_url)
         tc.url(version_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.find("%s%s" % (name, i))
-        for i in range(count):
+        for i in xrange(count):
             tc.formvalue('version_table', 'sel', "%s%s" % (name, i))
         tc.submit('remove')
         tc.url(version_url + '$')
-        for i in range(count):
+        for i in xrange(count):
             tc.notfind("%s%s" % (name, i))
 
 
@@ -905,7 +941,7 @@ class TestAdminVersionDefault(FunctionalTwillTestCaseSetup):
         tc.go(version_url)
         tc.formvalue('version_table', 'default', name)
         tc.submit('apply')
-        tc.find('type="radio" name="default" value="%s" checked="checked"' % \
+        tc.find('type="radio" name="default" checked="checked" value="%s"' % \
                 name)
         # verify it is the default on the newticket page.
         tc.go(self._tester.url + '/newticket')
@@ -914,9 +950,9 @@ class TestAdminVersionDefault(FunctionalTwillTestCaseSetup):
         # Test the "Clear default" button
         tc.go(version_url)
         tc.submit('clear', formname='version_table')
-        tc.notfind('type="radio" name="default" value=".+" checked="checked"')
+        tc.notfind('type="radio" name="default" checked="checked" value=".+"')
         self._tester.create_ticket()
-        tc.find('<th id="h_version" class="missing">[ \t\n]+'
+        tc.find('<th class="missing" id="h_version">[ \t\n]+'
                 'Version:[ \t\n]+</th>[ \t\n]+'
                 '(?!<td headers="h_version">)')
 
