@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,12 +16,12 @@ import difflib
 import re
 import unittest
 
-import trac.tests.compat
 from trac.mimeview.api import Mimeview
 from trac.test import Mock, EnvironmentStub, MockPerm, MockRequest
 from trac.ticket.api import TicketSystem
 from trac.ticket.model import Milestone, Severity, Ticket, Version
 from trac.ticket.query import Query, QueryModule, TicketQueryMacro
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import utc
 from trac.web.api import arg_list_to_args, parse_arg_list
 from trac.web.chrome import Chrome, web_context
@@ -97,16 +97,16 @@ class QueryTestCase(unittest.TestCase):
         with self.env.db_transaction:
             ids = []
             for idx in xrange(self.n_tickets):
-                t = Ticket(self.env)
-                t['summary'] = 'Summary %d' % idx
-                t['owner'] = owner[idx % len(owner)]
-                t['type'] = type[idx % len(type)]
-                t['status'] = status[idx % len(status)]
-                t['priority'] = priority[idx % len(priority)]
-                t['milestone'] = milestone[idx % len(milestone)]
-                t['version'] = version[idx % len(version)]
-                t['keywords'] = keywords[idx % len(keywords)]
-                ids.append(t.insert(when=when + timedelta(days=idx * 10)))
+                t = insert_ticket(self.env, summary='Summary %d' % idx,
+                                  owner=owner[idx % len(owner)],
+                                  type=type[idx % len(type)],
+                                  status=status[idx % len(status)],
+                                  priority=priority[idx % len(priority)],
+                                  milestone=milestone[idx % len(milestone)],
+                                  version=version[idx % len(version)],
+                                  keywords=keywords[idx % len(keywords)],
+                                  when=when + timedelta(days=idx * 10))
+                ids.append(t.id)
                 t.save_changes(comment='...',
                                when=when + timedelta(days=idx * 10 + 1))
         return ids
@@ -473,11 +473,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 
     def test_constrained_by_id_and_custom_field(self):
         self.env.config.set('ticket-custom', 'foo', 'text')
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'joe'
-        ticket['summary'] = 'Foo'
-        ticket['foo'] = 'blah'
-        ticket.insert()
+        ticket = insert_ticket(self.env, reporter='joe', summary='Foo',
+                               foo='blah')
 
         query = Query.from_string(self.env, 'id=%d-42&foo=blah' % ticket.id)
         tickets = self._execute_query(query)
@@ -500,12 +497,9 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         for f in fields:
             self.env.config.set('ticket-custom', f, 'text')
 
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'joe'
-        ticket['summary'] = 'Foo'
-        for idx, f in enumerate(fields):
-            ticket[f] = '%d.%s' % (idx, f)
-        ticket.insert()
+        ticket = insert_ticket(self.env, reporter='joe', summary='Foo',
+                               **{f: '%d.%s' % (idx, f)
+                                  for idx, f in enumerate(fields)})
 
         string = 'col_00=0.col_00&order=id&col=id&col=reporter&col=summary' + \
                  ''.join('&col=' + f for f in fields)
@@ -676,7 +670,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
 WHERE (((t.time>=%s AND t.time<%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        self.assertEqual([1217548800000000, 1220227200000000], args)
         tickets = self._execute_query(query)
         self.assertEqual(['2008-08-10T12:34:56.987654+00:00',
                           '2008-08-20T12:34:56.987654+00:00',
@@ -692,7 +686,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
 WHERE ((NOT (t.time>=%s AND t.time<%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        self.assertEqual([1217548800000000, 1220227200000000], args)
         tickets = self._execute_query(query)
         self.assertEqual(['2008-07-01T12:34:56.987654+00:00',
                           '2008-07-11T12:34:56.987654+00:00',
@@ -712,7 +706,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
 WHERE ((t.time>=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        self.assertEqual([1217548800000000L], args)
+        self.assertEqual([1217548800000000], args)
         tickets = self._execute_query(query)
         self.assertEqual(['2008-08-10T12:34:56.987654+00:00',
                           '2008-08-20T12:34:56.987654+00:00',
@@ -731,7 +725,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
 WHERE ((t.time<%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        self.assertEqual([1220227200000000L], args)
+        self.assertEqual([1220227200000000], args)
         tickets = self._execute_query(query)
         self.assertEqual(['2008-07-01T12:34:56.987654+00:00',
                           '2008-07-11T12:34:56.987654+00:00',
@@ -751,7 +745,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
 WHERE (((t.changetime>=%s AND t.changetime<%s)))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        self.assertEqual([1217548800000000, 1220227200000000], args)
         tickets = self._execute_query(query)
         self.assertEqual(['2008-08-01T12:34:56.987654+00:00',
                           '2008-08-11T12:34:56.987654+00:00',
@@ -869,8 +863,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         query = Query.from_string(self.env,
                                   'priority_value=baz&priority_value=foo')
         tickets = query.execute()
-        self.assertEqual(set(['foo', 'baz']),
-                         set(t['priority_value'] for t in tickets))
+        self.assertEqual({'foo', 'baz'},
+                         {t['priority_value'] for t in tickets})
         self.assertIn(tickets[0]['_priority_value'],
                       (None, '1', '2', '3', '4', '5'))
 
@@ -900,11 +894,9 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         del tktsys.custom_fields
         with self.env.db_transaction as db:
             for value in ('foo', 'bar', 'baz', 'blah'):
-                t = Ticket(self.env)
-                t['reporter'] = 'joe'
-                t['summary'] = 'Summary "%s"' % value
-                t[name] = value
-                t.insert()
+                insert_ticket(self.env, reporter='joe',
+                              summary='Summary "%s"' % value,
+                              **{name: value})
             for name in [name]:
                 quoted[name] = db.quote(name)
         return quoted
@@ -1086,15 +1078,15 @@ WHERE ((COALESCE(c.%(ticket)s,'') %(like)s))
 ORDER BY COALESCE(c.%(ticket)s,'')='',c.%(ticket)s,t.id""" % quoted)
         tickets = self._execute_query(query)
         self.assertEqual(tktids, [t['id'] for t in tickets])
-        self.assertEqual(set(['new']), set(t['status'] for t in tickets))
+        self.assertEqual({'new'}, {t['status'] for t in tickets})
         self.assertEqual(['ticket-0', 'ticket-1', 'ticket-2'],
                          sorted(t['ticket'] for t in tickets))
         self.assertEqual(['', 'name-0', 'name-2'],
                          sorted(t['name'] for t in tickets))
         self.assertEqual(['', 'value-0', 'value-1'],
                          sorted(t['value'] for t in tickets))
-        self.assertEqual(set(['test_ticket_custom_field']),
-                         set(t['summary'] for t in tickets))
+        self.assertEqual({'test_ticket_custom_field'},
+                         {t['summary'] for t in tickets})
         self.assertEqual(3, len(tickets))
 
     def test_csv_escape(self):
@@ -1148,7 +1140,7 @@ ORDER BY COALESCE(c.%(ticket)s,'')='',c.%(ticket)s,t.id""" % quoted)
 
     def test_properties_script_data(self):
         req = MockRequest(self.env, path_info='/query')
-        template, data, content_type = self._process_request(req)
+        template, data = self._process_request(req)
         prop = req.chrome['script_data']['properties']['milestone']
         self.assertEqual('select', prop['type'])
         self.assertEqual('Milestone', prop['label'])
@@ -1164,7 +1156,7 @@ ORDER BY COALESCE(c.%(ticket)s,'')='',c.%(ticket)s,t.id""" % quoted)
         self.env.db_transaction("DELETE FROM milestone")
         self.env.config.set('ticket-custom', 'milestone', 'text')
         req = MockRequest(self.env, path_info='/query')
-        template, data, content_type = self._process_request(req)
+        template, data = self._process_request(req)
         prop = req.chrome['script_data']['properties']['milestone']
         self.assertEqual({'label': 'Milestone', 'type': 'text',
                           'format': 'plain'}, prop)
@@ -1207,14 +1199,14 @@ ORDER BY COALESCE(c.%(ticket)s,'')='',c.%(ticket)s,t.id""" % quoted)
                          t2['time'].isoformat())
         self.assertEqual('2017-11-09T12:56:35.654321+00:00',
                          t2['changetime'].isoformat())
-        self.assertEqual(None, t2['due'])
+        self.assertIsNone(t2['due'])
 
         t3 = tickets[2]
         self.assertEqual('2008-07-21T12:34:56.987654+00:00',
                          t3['time'].isoformat())
         self.assertEqual('2008-07-22T12:34:56.987654+00:00',
                          t3['changetime'].isoformat())
-        self.assertEqual(None, t3['due'])
+        self.assertIsNone(t3['due'])
 
     def test_time_fields_order(self):
         dt = datetime(2018, 4, 25, 12, 34, 56, 987654, utc)
@@ -1338,7 +1330,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""".format(**quoted))
         orig_args = arg_list_to_args(arg_list)
         mod = QueryModule(self.env)
         self.assertTrue(mod.match_request(req))
-        template, data, content_type = mod.process_request(req)
+        template, data = mod.process_request(req)
         self.assertEqual([{'type': ['defect', 'enhancement']},
                           {'type': ['task']}],
                          data['query'].constraints)
@@ -1360,14 +1352,15 @@ ORDER BY COALESCE(t.id,0)=0,t.id""".format(**quoted))
                           args={'id': id_range, 'col': 'blah', 'order': 'id'})
         mod = QueryModule(self.env)
         self.assertTrue(mod.match_request(req))
-        template, data, content_type = mod.process_request(req)
+        template, data = mod.process_request(req)
         tickets = data['tickets']
         self.assertEqual(tktids[0], tickets[0]['id'])
         self.assertEqual(True, tickets[0]['blah'])
         self.assertEqual(tktids[1], tickets[1]['id'])
         self.assertEqual(False, tickets[1]['blah'])
-
-        rendered = Chrome(self.env).render_template(req, template, data)
+        rendered = Chrome(self.env).render_template(req, template, data,
+                                                    {'fragment': False,
+                                                     'iterable': False})
         matches = list(re.finditer(r'<td class="blah">\s*([^<\s]*)\s*</td>',
                                    rendered))
         self.assertEqual('yes', matches[0].group(1))
@@ -1384,19 +1377,6 @@ ORDER BY COALESCE(t.id,0)=0,t.id""".format(**quoted))
                          '%d,0\r\n' % (tktids[0], tktids[1]),
                          csv)
 
-    def test_rss_feed_for_authenticated_users(self):
-        req = MockRequest(self.env, authname='admin', path_info='/query',
-                          args={'status': '!closed'})
-        self._process_request(req)
-        alternates = req.chrome['links']['alternate']
-        for link in alternates:
-            if link['type'] == 'application/rss+xml':
-                break
-        else:
-            self.fail('No application/rss+xml in %r' % alternates)
-        self.assertIn('/trac.cgi/login?referer=%2Ftrac.cgi%2Fquery%3F',
-                      link['href'])
-
 
 class QueryLinksTestCase(unittest.TestCase):
 
@@ -1411,30 +1391,26 @@ class QueryLinksTestCase(unittest.TestCase):
         self.env.reset_db()
 
     def _insert_ticket(self, **attrs):
-        attrs.setdefault('reporter', 'joe')
-        attrs.setdefault('summary', 'Summary')
-        attrs.setdefault('status', 'new')
-        ticket = Ticket(self.env)
-        for name, value in attrs.iteritems():
-            ticket[name] = value
-        ticket.insert()
-        return ticket
+        reporter = attrs.pop('reporter', 'joe')
+        summary = attrs.pop('summary', 'Summary')
+        status = attrs.pop('status', 'new')
+        return insert_ticket(self.env, reporter=reporter, summary=summary,
+                             status=status, **attrs)
 
     def _format_link(self, query, label):
-        return str(self.query_module._format_link(self.formatter, 'query',
-                                                  query, label))
+        return unicode(self.query_module._format_link(self.formatter, 'query',
+                                                      query, label))
 
     def test_empty_query(self):
-        self.assertEqual(self._format_link('', 'label'),
-                         '<em class="error">[Error: Query filter requires '
-                         'field and constraints separated by a "="]</em>')
+        self.assertEqual(u'<em class="error">[Error: Query filter requires '
+                         'field and constraints separated by a "="]</em>',
+                         self._format_link('', 'label'))
 
     def _process_request(self, query_string):
         self.req.arg_list = parse_arg_list(query_string)
         self.req.args = arg_list_to_args(self.req.arg_list)
-        self.assertEqual(True, self.query_module.match_request(self.req))
-        template, data, content_type = \
-                self.query_module.process_request(self.req)
+        self.assertTrue(self.query_module.match_request(self.req))
+        template, data = self.query_module.process_request(self.req)
         return data
 
     def test_duplicated_order_arguments(self):
@@ -1445,7 +1421,7 @@ class QueryLinksTestCase(unittest.TestCase):
     def test_duplicated_report_arguments(self):
         data = self._process_request('report=1&report=2')
         self.assertEqual([], data['tickets'])
-        self.assertEqual(1, data['query'].id)
+        self.assertEqual('1', data['query'].id)
 
     def test_duplicated_group_arguments(self):
         self._insert_ticket(status='new')
@@ -1453,12 +1429,12 @@ class QueryLinksTestCase(unittest.TestCase):
         data = self._process_request(
                 'group=status&group=status&order=priority')
         self.assertNotEqual([], data['tickets'])
-        self.assertEqual(set(('new', 'assigned')),
-                         set(t['status'] for t in data['tickets']))
+        self.assertEqual({'new', 'assigned'},
+                         {t['status'] for t in data['tickets']})
         self.assertEqual(2, len(data['tickets']))
         self.assertNotEqual([], data['groups'])
-        self.assertEqual(set(('new', 'assigned')),
-                         set(value for value, tickets in data['groups']))
+        self.assertEqual({'new', 'assigned'},
+                         {value for value, tickets in data['groups']})
         self.assertEqual(2, len(data['groups']))
 
     def test_max_only_argument(self):
@@ -1486,16 +1462,6 @@ class QueryLinksTestCase(unittest.TestCase):
         self.assertTrue(data['query'].groupdesc)
         self.assertFalse(data['query'].desc)
 
-    def test_non_numeric_report_argument(self):
-        data = self._process_request('id=0&report=42xx')
-        self.assertEqual([], data['tickets'])
-        self.assertEqual(None, data['query'].id)
-
-    def test_unicode_report_argument(self):
-        data = self._process_request(u'id=0&report=42éá')
-        self.assertEqual([], data['tickets'])
-        self.assertEqual(None, data['query'].id)
-
 
 class TicketQueryMacroTestCase(unittest.TestCase):
 
@@ -1585,12 +1551,12 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-progress">
-
-  <table xmlns="http://www.w3.org/1999/xhtml" class="progress">
+  <table class="progress">
     <tr>
       <td class="closed" style="width: 33%">
         <a href="/query?status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/3 closed"></a>
-      </td><td class="open" style="width: 67%">
+      </td>
+      <td class="open" style="width: 67%">
         <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="2/3 active"></a>
       </td>
     </tr>
@@ -1604,7 +1570,8 @@ QUERY_TEST_CASES = u"""
     </span>
     <span class="interval">
       - <a href="/query?status=closed&amp;group=resolution&amp;max=0&amp;order=time">closed: 1</a>
-    </span><span class="interval">
+    </span>
+    <span class="interval">
       - <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id">active: 2</a>
     </span>
   </p>
@@ -1616,12 +1583,12 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-progress">
-
-  <table xmlns="http://www.w3.org/1999/xhtml" class="progress">
+  <table class="progress">
     <tr>
       <td class="closed" style="display: none">
         <a href="/query?status=closed&amp;reporter=santa&amp;group=resolution&amp;max=0&amp;order=time" title="0/1 closed"></a>
-      </td><td class="open" style="width: 100%">
+      </td>
+      <td class="open" style="width: 100%">
         <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;max=0&amp;order=id" title="1/1 active"></a>
       </td>
     </tr>
@@ -1635,7 +1602,8 @@ QUERY_TEST_CASES = u"""
     </span>
     <span class="interval">
       - <a href="/query?status=closed&amp;reporter=santa&amp;group=resolution&amp;max=0&amp;order=time">closed: 0</a>
-    </span><span class="interval">
+    </span>
+    <span class="interval">
       - <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;max=0&amp;order=id">active: 1</a>
     </span>
   </p>
@@ -1647,12 +1615,12 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-progress">
-
-  <table xmlns="http://www.w3.org/1999/xhtml" class="progress">
+  <table class="progress">
     <tr>
       <td class="closed" style="width: 50%">
         <a href="/query?status=closed&amp;reporter=santa&amp;or&amp;owner=santa&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/2 closed"></a>
-      </td><td class="open" style="width: 50%">
+      </td>
+      <td class="open" style="width: 50%">
         <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;or&amp;owner=santa&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/2 active"></a>
       </td>
     </tr>
@@ -1666,7 +1634,8 @@ QUERY_TEST_CASES = u"""
     </span>
     <span class="interval">
       - <a href="/query?status=closed&amp;reporter=santa&amp;or&amp;owner=santa&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time">closed: 1</a>
-    </span><span class="interval">
+    </span>
+    <span class="interval">
       - <a href="/query?status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;or&amp;owner=santa&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id">active: 1</a>
     </span>
   </p>
@@ -1678,21 +1647,19 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table xmlns="http://www.w3.org/1999/xhtml" summary="Ticket completion status for each project">
+  <table summary="Ticket completion status for each project">
     <tr>
       <th scope="row">
         <i><a href="/query?project=&amp;max=0&amp;order=id">(none)</a></i>
-
-
       </th>
       <td>
 
-
   <table class="progress" style="width: 40%">
     <tr>
       <td class="closed" style="display: none">
         <a href="/query?project=&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="0/1 closed"></a>
-      </td><td class="open" style="width: 100%">
+      </td>
+      <td class="open" style="width: 100%">
         <a href="/query?project=&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/1 active"></a>
       </td>
     </tr>
@@ -1700,23 +1667,20 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">0 / 1</p>
 
-
-
       </td>
-    </tr><tr>
+    </tr>
+    <tr>
       <th scope="row">
-
-
         <a href="/query?project=xmas&amp;max=0&amp;order=id">xmas</a>
       </th>
       <td>
 
-
   <table class="progress" style="width: 80%">
     <tr>
       <td class="closed" style="width: 50%">
         <a href="/query?project=xmas&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/2 closed"></a>
-      </td><td class="open" style="width: 50%">
+      </td>
+      <td class="open" style="width: 50%">
         <a href="/query?project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/2 active"></a>
       </td>
     </tr>
@@ -1724,12 +1688,9 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">1 / 2</p>
 
-
-
       </td>
     </tr>
-  </table>
-</div><p>
+  </table></div><p>
 </p>
 ------------------------------
 ============================== TicketQuery(format=progress, group=project, groupdesc=1)
@@ -1737,21 +1698,19 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table xmlns="http://www.w3.org/1999/xhtml" summary="Ticket completion status for each project">
+  <table summary="Ticket completion status for each project">
     <tr>
       <th scope="row">
-
-
         <a href="/query?project=xmas&amp;max=0&amp;order=id">xmas</a>
       </th>
       <td>
 
-
   <table class="progress" style="width: 80%">
     <tr>
       <td class="closed" style="width: 50%">
         <a href="/query?project=xmas&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/2 closed"></a>
-      </td><td class="open" style="width: 50%">
+      </td>
+      <td class="open" style="width: 50%">
         <a href="/query?project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/2 active"></a>
       </td>
     </tr>
@@ -1759,23 +1718,20 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">1 / 2</p>
 
-
-
       </td>
-    </tr><tr>
+    </tr>
+    <tr>
       <th scope="row">
         <i><a href="/query?project=&amp;max=0&amp;order=id">(none)</a></i>
-
-
       </th>
       <td>
 
-
   <table class="progress" style="width: 40%">
     <tr>
       <td class="closed" style="display: none">
         <a href="/query?project=&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="0/1 closed"></a>
-      </td><td class="open" style="width: 100%">
+      </td>
+      <td class="open" style="width: 100%">
         <a href="/query?project=&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/1 active"></a>
       </td>
     </tr>
@@ -1783,12 +1739,9 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">0 / 1</p>
 
-
-
       </td>
     </tr>
-  </table>
-</div><p>
+  </table></div><p>
 </p>
 ------------------------------
 ============================== TicketQuery(reporter=santa, format=progress, group=project)
@@ -1796,21 +1749,19 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table xmlns="http://www.w3.org/1999/xhtml" summary="Ticket completion status for each project">
+  <table summary="Ticket completion status for each project">
     <tr>
       <th scope="row">
-
-
         <a href="/query?project=xmas&amp;reporter=santa&amp;max=0&amp;order=id">xmas</a>
       </th>
       <td>
 
-
   <table class="progress" style="width: 80%">
     <tr>
       <td class="closed" style="display: none">
         <a href="/query?project=xmas&amp;status=closed&amp;reporter=santa&amp;group=resolution&amp;max=0&amp;order=time" title="0/1 closed"></a>
-      </td><td class="open" style="width: 100%">
+      </td>
+      <td class="open" style="width: 100%">
         <a href="/query?project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;max=0&amp;order=id" title="1/1 active"></a>
       </td>
     </tr>
@@ -1818,12 +1769,9 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">0 / 1</p>
 
-
-
       </td>
     </tr>
-  </table>
-</div><p>
+  </table></div><p>
 </p>
 ------------------------------
 ============================== TicketQuery(reporter=santa&or&owner=santa, format=progress, group=project)
@@ -1831,21 +1779,19 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table xmlns="http://www.w3.org/1999/xhtml" summary="Ticket completion status for each project">
+  <table summary="Ticket completion status for each project">
     <tr>
       <th scope="row">
-
-
         <a href="/query?project=xmas&amp;reporter=santa&amp;or&amp;owner=santa&amp;project=xmas&amp;max=0&amp;order=id">xmas</a>
       </th>
       <td>
 
-
   <table class="progress" style="width: 80%">
     <tr>
       <td class="closed" style="width: 50%">
         <a href="/query?project=xmas&amp;status=closed&amp;reporter=santa&amp;or&amp;owner=santa&amp;project=xmas&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/2 closed"></a>
-      </td><td class="open" style="width: 50%">
+      </td>
+      <td class="open" style="width: 50%">
         <a href="/query?project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;reporter=santa&amp;or&amp;owner=santa&amp;project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/2 active"></a>
       </td>
     </tr>
@@ -1853,12 +1799,9 @@ QUERY_TEST_CASES = u"""
 
   <p class="percent">1 / 2</p>
 
-
-
       </td>
     </tr>
-  </table>
-</div><p>
+  </table></div><p>
 </p>
 ------------------------------
 """
@@ -1866,23 +1809,12 @@ QUERY_TEST_CASES = u"""
 
 def ticket_setup(tc):
     tc.env.config.set('ticket-custom', 'project', 'text')
-    ticket = Ticket(tc.env)
-    ticket.populate({'reporter': 'santa',
-                     'summary': 'This is the summary',
-                     'status': 'new',
-                     'project': 'xmas'})
-    ticket.insert()
-    ticket = Ticket(tc.env)
-    ticket.populate({'owner': 'elf',
-                     'summary': 'This is another summary',
-                     'status': 'assigned'})
-    ticket.insert()
-    ticket = Ticket(tc.env)
-    ticket.populate({'owner': 'santa',
-                     'summary': 'This is th third summary',
-                     'status': 'closed',
-                     'project': 'xmas'})
-    ticket.insert()
+    insert_ticket(tc.env, reporter='santa', summary='This is the summary',
+                  status='new', project='xmas')
+    insert_ticket(tc.env, owner='elf',
+                  summary='This is another summary', status='assigned')
+    insert_ticket(tc.env, owner='santa', status='closed',
+                  summary='This is th third summary', project='xmas')
 
     tc.env.config.set('milestone-groups', 'closed.status', 'closed')
     tc.env.config.set('milestone-groups', 'closed.query_args',
