@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,17 +16,22 @@ import unittest
 from datetime import datetime, timedelta
 
 import trac
-import trac.tests.compat
 from trac.perm import PermissionSystem
 from trac.resource import ResourceNotFound
-from trac.ticket.model import Ticket
 from trac.ticket.query import QueryModule
 from trac.ticket.report import Report, ReportModule
 from trac.test import EnvironmentStub, MockRequest
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import utc
+from trac.util.html import genshi
 from trac.web.api import HTTPBadRequest, RequestDone
 from trac.web.chrome import Chrome
 
+if genshi:
+    from genshi.input import XML
+else:
+    XML = None
+
 
 class ReportModuleTestCase(unittest.TestCase):
 
@@ -37,6 +42,15 @@ class ReportModuleTestCase(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db()
 
+    def _insert_report(self, title, query, description):
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute("""
+                INSERT INTO report (title,query,description)
+                VALUES (%s,%s,%s)
+                """, (title, query, description))
+            return db.get_last_id(cursor, 'report')
+
     def test_create_report(self):
         req = MockRequest(self.env, method='POST', args={
             'action': 'new',
@@ -194,23 +208,47 @@ class ReportModuleTestCase(unittest.TestCase):
                          " * Color each row based on priority.\n",
                          data['description'])
 
+    def test_render_empty_report(self):
+        """Empty report raises a TracError."""
+        rid = self._insert_report('Empty report', '', '')
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'view',
+            'id': rid})
+        data = self.report_module.process_request(req)[1]
+        self.assertEqual("Report failed: Report {%d} has no SQL query." % rid,
+                         data['message'])
+
+    def test_render_view_sort_order_preserved_on_update(self):
+        """Sort column and order are preserved on update."""
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'view',
+            'id': 1,
+            'sort': 'summary',
+            'asc': '1'
+        })
+
+        data = self.report_module.process_request(req)[1]
+
+        self.assertEqual(1, data['asc'])
+        self.assertEqual('summary', data['sort'])
+
     def test_sub_var_no_quotes(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
-            "$VAR", {'VAR': 'value'})
+            u"$VAR", {'VAR': 'value'})
         self.assertEqual("%s", sql)
         self.assertEqual(['value'], values)
         self.assertEqual([], missing_args)
 
     def test_sub_var_digits_underscore(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
-            "$_VAR, $VAR2, $2VAR", {'_VAR': 'value1', 'VAR2': 'value2'})
+            u"$_VAR, $VAR2, $2VAR", {'_VAR': 'value1', 'VAR2': 'value2'})
         self.assertEqual("%s, %s, $2VAR", sql)
         self.assertEqual(['value1', 'value2'], values)
         self.assertEqual([], missing_args)
 
     def test_sub_var_quotes(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
-            "'$VAR'", {'VAR': 'value'})
+            u"'$VAR'", {'VAR': 'value'})
         with self.env.db_query as db:
             concatenated = db.concat("''", '%s', "''")
         self.assertEqual(concatenated, sql)
@@ -219,7 +257,7 @@ class ReportModuleTestCase(unittest.TestCase):
 
     def test_sub_var_missing_args(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
-            "$VAR, $PARAM, $MISSING", {'VAR': 'value'})
+            u"$VAR, $PARAM, $MISSING", {'VAR': 'value'})
         self.assertEqual("%s, %s, %s", sql)
         self.assertEqual(['value', '', ''], values)
         self.assertEqual(['PARAM', 'MISSING'], missing_args)
@@ -237,24 +275,20 @@ class ReportModuleTestCase(unittest.TestCase):
 
     def test_saved_custom_query_redirect(self):
         query = u'query:?type=résumé'
-        with self.env.db_transaction as db:
-            cursor = db.cursor()
-            cursor.execute("INSERT INTO report (title,query,description) "
-                           "VALUES (%s,%s,%s)", ('redirect', query, ''))
-            id = db.get_last_id(cursor, 'report')
+        rid = self._insert_report('redirect', query, '')
         req = MockRequest(self.env)
 
         self.assertRaises(RequestDone,
-                          self.report_module._render_view, req, id)
+                          self.report_module._render_view, req, rid)
         self.assertEqual('http://example.org/trac.cgi/query?' +
-                         'type=r%C3%A9sum%C3%A9&report=' + str(id),
+                         'type=r%C3%A9sum%C3%A9&report=' + str(rid),
                          req.headers_sent['Location'])
 
     def test_quoted_id_with_var(self):
         req = MockRequest(self.env)
         name = """%s"`'%%%?"""
         with self.env.db_query as db:
-            sql = 'SELECT 1 AS %s, $USER AS user' % db.quote(name)
+            sql = u'SELECT 1 AS %s, $USER AS user' % db.quote(name)
             rv = self.report_module.execute_paginated_report(req, 1, sql,
                                                              {'USER': 'joe'})
         self.assertEqual(5, len(rv), repr(rv))
@@ -262,19 +296,7 @@ class ReportModuleTestCase(unittest.TestCase):
         self.assertEqual([name, 'user'], cols)
         self.assertEqual([(1, 'joe')], results)
         self.assertEqual([], missing_args)
-        self.assertEqual(None, limit_offset)
-
-    def test_with_no_sql(self):
-        with self.env.db_transaction as db:
-            cursor = db.cursor()
-            cursor.execute("""INSERT INTO report (title,query,description)
-                              VALUES ('#12709', '', '')""")
-            id_ = db.get_last_id(cursor, 'report')
-        req = MockRequest(self.env, path_info='/report/%d' % id_)
-        self.assertTrue(self.report_module.match_request(req))
-        data = self.report_module.process_request(req)[1]
-        self.assertEqual('Report failed: Report {%d} has no SQL query.' % id_,
-                         data['message'])
+        self.assertIsNone(limit_offset)
 
     def test_value_error_from_get_var_args(self):
         req = MockRequest(self.env, path_info='/report/1')
@@ -286,20 +308,14 @@ class ReportModuleTestCase(unittest.TestCase):
         self.assertEqual('Report failed: <error>', data['message'])
 
     def test_title_and_description_with_sub_vars(self):
-        with self.env.db_transaction as db:
-            cursor = db.cursor()
-            cursor.execute(
-                """INSERT INTO report (title,query,description)
-                   VALUES (%s,%s,%s)""",
-                ('Tickets on $M for $USER',
-                 'SELECT * FROM ticket WHERE milestone=$M AND owner=$USER',
-                 'Show tickets on $M for $USER'))
-            id_ = db.get_last_id(cursor, 'report')
+        with self.env.db_transaction:
+            id_ = self._insert_report(
+                'Tickets on $M for $USER',
+                'SELECT * FROM ticket WHERE milestone=$M AND owner=$USER',
+                'Show tickets on $M for $USER')
             for milestone in ('milestone1', 'milestone2'):
-                ticket = Ticket(self.env)
-                ticket.populate({'status': 'new', 'summary': 'Test 1',
-                                 'owner': 'joe', 'milestone': milestone})
-                ticket.insert()
+                insert_ticket(self.env, status='new', summary='Test 1',
+                              owner='joe', milestone=milestone)
         req = MockRequest(self.env, path_info='/report/%d' % id_,
                           authname='joe', args={'M': 'milestone2'})
         self.assertTrue(self.report_module.match_request(req))
@@ -311,6 +327,27 @@ class ReportModuleTestCase(unittest.TestCase):
         self.assertIsNone(data['message'])
         self.assertEqual(1, data['numrows'])
 
+    def test_title_and_description_with_sub_vars_in_sql(self):
+        with self.env.db_transaction:
+            id_ = self._insert_report(
+                'Tickets on $M for $USER',
+                '-- M=milestone1\r\n'
+                'SELECT * FROM ticket WHERE milestone=$M AND owner=$USER\r\n',
+                'Show tickets on $M for $USER')
+            for milestone in ('milestone1', 'milestone2'):
+                insert_ticket(self.env, status='new', summary='Test 1',
+                              owner='joe', milestone=milestone)
+        req = MockRequest(self.env, path_info='/report/%d' % id_,
+                          authname='joe')
+        self.assertTrue(self.report_module.match_request(req))
+        data = self.report_module.process_request(req)[1]
+        self.assertEqual('{%d} Tickets on milestone1 for joe' % id_,
+                         data['title'])
+        self.assertEqual('Show tickets on milestone1 for joe',
+                         data['description'])
+        self.assertIsNone(data['message'])
+        self.assertEqual(1, data['numrows'])
+
     def test_clear(self):
         req = MockRequest(self.env, method='GET', path_info='/report',
                           args={'action': 'clear'})
@@ -329,6 +366,62 @@ class ReportModuleTestCase(unittest.TestCase):
         self.assertNotIn('query_href', req.session)
         self.assertNotIn('query_tickets', req.session)
 
+    def test_valid_html_for_report(self):
+        req = MockRequest(self.env, method='POST', path_info='/report', args={
+            'action': 'new',
+            'title': '#13046',
+            'query': "SELECT '#13046' AS foo_, 42 AS bar, 'blah' AS _baz_",
+            'description': ''})
+        self.assertTrue(self.report_module.match_request(req))
+        self.assertRaises(RequestDone, self.report_module.process_request, req)
+
+        req = MockRequest(self.env, method='GET', path_info='/report/9')
+        self.assertTrue(self.report_module.match_request(req))
+        rv = self.report_module.process_request(req)
+        rendered = Chrome(self.env).render_template(req, rv[0], rv[1],
+                                                    {'fragment': False,
+                                                     'iterable': False})
+        self.assertRegexpMatches(rendered,
+                                 r'<tr[^>]*>\s*'
+                                 r'<td class="fullrow foo" colspan="100">'
+                                 r'\s*#13046\s*<hr />\s*</td>\s*</tr>')
+        self.assertRegexpMatches(rendered,
+                                 r'<tr[^>]*>\s*'
+                                 r'<td class="fullrow bar" colspan="100">'
+                                 r'\s*42\s*<hr />\s*</td>\s*</tr>')
+        self.assertRegexpMatches(rendered,
+                                 r'<tr[^>]*>\s*'
+                                 r'<td class="fullrow baz" colspan="100">'
+                                 r'\s*blah\s*<hr />\s*</td>\s*</tr>')
+        if genshi:
+            XML(rendered)  # validates as XML
+
+    def test_timestamp_columns(self):
+        req = MockRequest(self.env, method='POST', path_info='/report', args={
+            'action': 'new',
+            'title': '#13134',
+            'query': "SELECT %d AS time, %d AS created, %d AS datetime"
+                     % ((1 * 86400 + 42) * 1000000,
+                        (2 * 86400 + 43) * 1000000,
+                        (3 * 86400 + 44) * 1000000),
+            'description': ''})
+        self.assertTrue(self.report_module.match_request(req))
+        self.assertRaises(RequestDone, self.report_module.process_request, req)
+
+        req = MockRequest(self.env, method='GET', path_info='/report/9')
+        self.assertTrue(self.report_module.match_request(req))
+        rv = self.report_module.process_request(req)
+        rendered = Chrome(self.env).render_template(req, rv[0], rv[1],
+                                                    {'fragment': False,
+                                                     'iterable': False})
+        self.assertRegexpMatches(rendered,
+            r'<td class="date">\s*(12:00:42 AM|00:00:42)\s*</td>')
+        self.assertRegexpMatches(rendered,
+            r'<td class="date">\s*(Jan 3, 1970|01/03/70)\s*</td>')
+        self.assertRegexpMatches(rendered,
+            r'<td class="date">\s*(Jan 4, 1970, 12:00:44 AM|'
+            r'01/04/70 00:00:44)\s*</td>')
+
 
 class ExecuteReportTestCase(unittest.TestCase):
 
@@ -340,12 +433,7 @@ class ExecuteReportTestCase(unittest.TestCase):
         self.env.reset_db()
 
     def _insert_ticket(self, when=None, **kwargs):
-        ticket = Ticket(self.env)
-        for name, value in kwargs.iteritems():
-            ticket[name] = value
-        ticket['status'] = 'new'
-        ticket.insert(when=when)
-        return ticket
+        return insert_ticket(self.env, when=when, **kwargs)
 
     def _save_ticket(self, ticket, author=None, comment=None, when=None,
                      **kwargs):
@@ -412,8 +500,7 @@ class ExecuteReportTestCase(unittest.TestCase):
                           'new minor'],
                          [r[idx_summary] for r in results])
         idx_color = cols.index('__color__')
-        self.assertEqual(set(('2', '3', '4')),
-                         set(r[idx_color] for r in results))
+        self.assertEqual({'2', '3', '4'}, {r[idx_color] for r in results})
 
     REPORT_2_DATA = """\
         # status    version     priority
@@ -448,11 +535,11 @@ class ExecuteReportTestCase(unittest.TestCase):
                           'new None minor'],
                          [r[idx_summary] for r in results])
         idx_color = cols.index('__color__')
-        self.assertEqual(set(('2', '4')),
-                         set(r[idx_color] for r in results))
+        self.assertEqual({'2', '4'},
+                         {r[idx_color] for r in results})
         idx_group = cols.index('__group__')
-        self.assertEqual(set(('1.0', '2.0', None)),
-                         set(r[idx_group] for r in results))
+        self.assertEqual({'1.0', '2.0', None},
+                         {r[idx_group] for r in results})
 
     REPORT_3_DATA = """\
         # status    milestone   priority
@@ -487,12 +574,11 @@ class ExecuteReportTestCase(unittest.TestCase):
                           'new None minor'],
                          [r[idx_summary] for r in results])
         idx_color = cols.index('__color__')
-        self.assertEqual(set(('3', '4')),
-                         set(r[idx_color] for r in results))
+        self.assertEqual({'3', '4'}, {r[idx_color] for r in results})
         idx_group = cols.index('__group__')
-        self.assertEqual(set(('Milestone milestone1', 'Milestone milestone3',
-                              None)),
-                         set(r[idx_group] for r in results))
+        self.assertEqual(
+            {'Milestone milestone1', 'Milestone milestone3', None},
+            {r[idx_group] for r in results})
 
     REPORT_4_DATA = """\
         # status    owner   priority
@@ -533,11 +619,10 @@ class ExecuteReportTestCase(unittest.TestCase):
                           'accepted john trivial'],
                          [r[idx_summary] for r in results])
         idx_color = cols.index('__color__')
-        self.assertEqual(set(('1', '5')),
-                         set(r[idx_color] for r in results))
+        self.assertEqual({'1', '5'}, {r[idx_color] for r in results})
         idx_group = cols.index('__group__')
-        self.assertEqual(set(('jack', 'john', 'foo')),
-                         set(r[idx_group] for r in results))
+        self.assertEqual({'jack', 'john', 'foo'},
+                         {r[idx_group] for r in results})
         if full_description:
             self.assertNotIn('_description', cols)
             self.assertIn('_description_', cols)
@@ -596,14 +681,12 @@ class ExecuteReportTestCase(unittest.TestCase):
                          results[3][idx_style])  # closed and not owned
         self.assertEqual('font-weight: bold',
                          results[1][idx_style])  # not closed and owned
-        self.assertEqual(None,
-                         results[0][idx_style])  # not closed and not owned
+        self.assertIsNone(results[0][idx_style])  # not closed and not owned
         idx_color = cols.index('__color__')
-        self.assertEqual(set(('2', '5')),
-                         set(r[idx_color] for r in results))
+        self.assertEqual({'2', '5'}, {r[idx_color] for r in results})
         idx_group = cols.index('__group__')
-        self.assertEqual(set(('milestone2', 'milestone4', None)),
-                         set(r[idx_group] for r in results))
+        self.assertEqual({'milestone2', 'milestone4', None},
+                         {r[idx_group] for r in results})
 
     REPORT_7_DATA = """\
         # status    owner   reporter    priority
@@ -642,8 +725,8 @@ class ExecuteReportTestCase(unittest.TestCase):
                           'new foo john trivial'],
                          [r[idx_summary] for r in results])
         idx_group = cols.index('__group__')
-        self.assertEqual(set(('Accepted', 'Owned', 'Reported')),
-                         set(r[idx_group] for r in results))
+        self.assertEqual({'Accepted', 'Owned', 'Reported'},
+                         {r[idx_group] for r in results})
 
         self._save_ticket(tickets[-1], author='john', comment='commented')
         rv = self._execute_report(7, {'USER': 'john'})
@@ -651,8 +734,8 @@ class ExecuteReportTestCase(unittest.TestCase):
 
         self.assertEqual(7, len(results))
         self.assertEqual('new bar foo major', results[-1][idx_summary])
-        self.assertEqual(set(('Accepted', 'Owned', 'Reported', 'Commented')),
-                         set(r[idx_group] for r in results))
+        self.assertEqual({'Accepted', 'Owned', 'Reported', 'Commented'},
+                         {r[idx_group] for r in results})
 
         rv = self._execute_report(7, {'USER': 'blah <blah@example.org>'})
         cols, results, num_items, missing_args, limit_offset = rv
@@ -704,24 +787,7 @@ class ExecuteReportTestCase(unittest.TestCase):
                          [r[idx_summary] for r in results])
         idx_group = cols.index('__group__')
         self.assertEqual(['Active Tickets'],
-                         sorted(set(r[idx_group] for r in results)))
-
-    def test_execute_paginated_report_legacy_signature(self):
-        """`execute_paginated_report` returns the same results with and
-        without the deprecated `db` argument."""
-        id = 1
-        attrs = dict(reporter='joe', component='component1', version='1.0',
-                     milestone='milestone1', type='defect', owner='joe')
-        self._generate_tickets(('status', 'priority'), self.REPORT_1_DATA,
-                               attrs)
-        mod = self.report_module
-        req = MockRequest(self.env)
-        sql = Report(self.env, id).query
-
-        rv1 = mod.execute_paginated_report(req, id, sql, {})
-        with self.env.db_query as db:
-            rv2 = mod.execute_paginated_report(req, db, id, sql, {})
-        self.assertEqual(rv2, rv1)
+                         sorted({r[idx_group] for r in results}))
 
     def test_asc_argument_is_invalid(self):
         """Invalid value for `asc` argument is coerced to default."""
