@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,15 +14,16 @@
 import unittest
 
 from trac.core import ComponentManager
+from trac.perm import PermissionSystem
 from trac.resource import Resource, ResourceNotFound, render_resource_link
 from trac.test import EnvironmentStub, MockRequest
-from trac.ticket.model import Ticket
 from trac.ticket.roadmap import (
     DefaultTicketGroupStatsProvider, Milestone, MilestoneModule,
-    TicketGroupStats, get_tickets_for_all_milestones,
+    RoadmapModule, TicketGroupStats, get_tickets_for_all_milestones,
     get_tickets_for_milestone)
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import datetime_now, utc
-from trac.web.api import HTTPBadRequest
+from trac.web.api import HTTPBadRequest, RequestDone
 from trac.web.chrome import web_context
 from trac.web.tests.api import RequestHandlerPermissionsTestCaseBase
 
@@ -45,7 +46,8 @@ class TicketGroupStatsTestCase(unittest.TestCase):
         int = self.stats.intervals[0]
         self.assertEqual('intTitle', int['title'], 'title incorrect')
         self.assertEqual(3, int['count'], 'count incorrect')
-        self.assertEqual({'k1': 'v1'}, int['qry_args'], 'query args incorrect')
+        self.assertEqual({'k1': 'v1'}, int['qry_args'],
+                         'query args incorrect')
         self.assertEqual('css', int['css_class'], 'css class incorrect')
         self.assertEqual(100, int['percent'], 'percent incorrect')
         self.stats.add_interval('intTitle', 3, {'k1': 'v1'}, 'css', 0)
@@ -93,18 +95,12 @@ class DefaultTicketGroupStatsProviderTestCase(unittest.TestCase):
         self.milestone2.name = 'Test2'
         self.milestone2.insert()
 
-        tkt1 = Ticket(self.env)
-        tkt1.populate({'summary': 'Foo', 'milestone': 'Test', 'owner': 'foman',
-                        'status': 'new'})
-        tkt1.insert()
-        tkt2 = Ticket(self.env)
-        tkt2.populate({'summary': 'Bar', 'milestone': 'Test',
-                        'status': 'closed', 'owner': 'barman'})
-        tkt2.insert()
-        tkt3 = Ticket(self.env)
-        tkt3.populate({'summary': 'Sum', 'milestone': 'Test', 'owner': 'suman',
-                        'status': 'reopened'})
-        tkt3.insert()
+        tkt1 = insert_ticket(self.env, summary='Foo', milestone='Test',
+                             owner='foman', status='new')
+        tkt2 = insert_ticket(self.env, summary='Bar', milestone='Test',
+                             status='closed', owner='barman')
+        tkt3 = insert_ticket(self.env, summary='Sum', milestone='Test',
+                             owner='suman', status='reopened')
         self.tkt1 = tkt1
         self.tkt2 = tkt2
         self.tkt3 = tkt3
@@ -120,12 +116,14 @@ class DefaultTicketGroupStatsProviderTestCase(unittest.TestCase):
     def test_stats(self):
         self.assertEqual(self.stats.title, 'ticket status', 'title incorrect')
         self.assertEqual(self.stats.unit, 'tickets', 'unit incorrect')
-        self.assertEqual(2, len(self.stats.intervals), 'more than 2 intervals')
+        self.assertEqual(2, len(self.stats.intervals),
+                         'more than 2 intervals')
 
     def test_closed_interval(self):
         closed = self.stats.intervals[0]
         self.assertEqual('closed', closed['title'], 'closed title incorrect')
-        self.assertEqual('closed', closed['css_class'], 'closed class incorrect')
+        self.assertEqual('closed', closed['css_class'],
+                         'closed class incorrect')
         self.assertTrue(closed['overall_completion'],
                         'closed should contribute to overall completion')
         self.assertEqual({'status': ['closed'], 'group': ['resolution']},
@@ -170,11 +168,34 @@ est laborum."""
         self.env.reset_db()
 
     def test_invalid_post_request_raises_exception(self):
-        req = MockRequest(self.env, method='POST', action=None)
+        req = MockRequest(self.env, method='POST', action=None,
+                          args={'id': 'milestone1'})
 
         self.assertRaises(HTTPBadRequest,
                           MilestoneModule(self.env).process_request, req)
 
+    def test_milestone_redirects_to_roadmap(self):
+        """The path /milestone redirects to /roadmap."""
+        def test_milestone_redirect(authname=None):
+            req = MockRequest(self.env, method='GET', path_info='/milestone',
+                              authname=authname)
+            with self.assertRaises(RequestDone):
+                self.mmodule.process_request(req)
+            self.assertEqual('http://example.org/trac.cgi/roadmap',
+                             req.headers_sent['Location'])
+            self.assertEqual('302 Found', req._status)
+            return req
+
+        # Redirects for user with MILESTONE_VIEW
+        req = test_milestone_redirect()
+        self.assertIn('MILESTONE_VIEW', req.perm)
+
+        # Redirects for user without MILESTONE_VIEW
+        perm_sys = PermissionSystem(self.env)
+        perm_sys.revoke_permission('anonymous', 'MILESTONE_VIEW')
+        req = test_milestone_redirect('user1')
+        self.assertNotIn('MILESTONE_VIEW', req.perm)
+
     def test_get_search_filters(self):
         req = MockRequest(self.env)
         filters = self.mmodule.get_search_filters(req)
@@ -231,6 +252,7 @@ consequat. Duis a ..."""
 
     def test_get_search_results_matches_ignorecase(self):
         req = MockRequest(self.env)
+
         def search(terms):
             return list(self.mmodule.get_search_results(req, terms,
                                                         ['milestone']))
@@ -301,6 +323,13 @@ class RoadmapTestCase(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db()
 
+    def insert_milestone(self, name, due=None):
+        milestone = Milestone(self.env)
+        milestone.name = name
+        milestone.due = due
+        milestone.insert()
+        return milestone
+
     def test_get_tickets_for_all_milestones(self):
         tickets = get_tickets_for_all_milestones(self.env, field='owner')
         milestone1 = [{'id': 9, 'status': 'new', 'owner': 'blah'},
@@ -341,6 +370,36 @@ class RoadmapTestCase(unittest.TestCase):
                                                    field='project'))
         self.assertEqual(['milestone1', 'milestone2'], sorted(tickets))
 
+    def test_export_ical_from_roadmap(self):
+        self.insert_milestone('milestone1', datetime_now(utc))
+        self.insert_milestone('milestone2')
+        rm = RoadmapModule(self.env)
+        req = MockRequest(self.env, path_info='/roadmap',
+                          args={'format': 'ics'})
+
+        self.assertTrue(rm.match_request(req))
+        with self.assertRaises(RequestDone):
+            rm.process_request(req)
+
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertRegexpMatches(req.response_sent.getvalue(), """\
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Edgewall Software//NONSGML Trac [\.\w]+//EN
+METHOD:PUBLISH
+X-WR-CALNAME:My Project - Roadmap
+X-WR-CALDESC:My example project
+X-WR-TIMEZONE:UTC
+BEGIN:VEVENT
+UID:</trac.cgi/milestone/milestone1@example.org/trac.cgi>
+DTSTAMP:\w+
+DTSTART;VALUE=DATE:\w+
+SUMMARY:Milestone milestone1
+URL:http://example.org/trac.cgi/milestone/milestone1
+END:VEVENT
+END:VCALENDAR
+""".replace('\n', '\r\n'))
+
 
 class ResourceTestCase(unittest.TestCase):
     """Test cases for milestone resources."""
