@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -68,8 +68,8 @@ class TestRssFormat(FunctionalTwillTestCaseSetup):
         page = WikiPage(self._testenv.get_trac_environment(), pagename)
         self._tester.go_to_timeline()
         tc.follow("RSS Feed")
-        tc.find(r"""<\?xml version="1.0"\?>
-<rss xmlns:dc="http://purl.org/dc/elements/1.1/" version="2.0">
+        tc.find(r"""<\?xml version="1.0"\?>[\n]+
+<rss version="2.0" xmlns:dc="http://purl.org/dc/elements/1.1/">
   <channel>
     <title>Functional Tests</title>
     <link>http://127.0.0.1:\d+/timeline</link>
@@ -83,15 +83,14 @@ class TestRssFormat(FunctionalTwillTestCaseSetup):
     </image>
     <item>
       <title>%(pagename)s created</title>
-\s+
-        <dc:creator>admin</dc:creator>
 
+      <dc:creator>admin</dc:creator>
       <pubDate>%(http_date)s</pubDate>
       <link>http://127.0.0.1:\d+/wiki/%(pagename)s\?version=1</link>
       <guid isPermaLink="false">http://127.0.0.1:\d+/wiki/%(pagename)s\?version=1/\d+</guid>
       <description>[^<]+</description>
       <category>wiki</category>
-    </item><item>
+    </item>
 """ % {'pagename': pagename, 'http_date': http_date(page.time)}, 'ms')
 
 
