@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -10,18 +10,21 @@
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
-
+import textwrap
 import unittest
 from datetime import datetime, timedelta
+from xml.dom import minidom
 
+from trac.core import Component, ComponentMeta, implements
 from trac.perm import PermissionError, PermissionSystem
-from trac.test import EnvironmentStub, MockRequest, locale_en
+from trac.test import EnvironmentStub, Mock, MockRequest, locale_en
+from trac.timeline.api import ITimelineEventProvider
 from trac.timeline.web_ui import TimelineModule
 from trac.util.datefmt import (
     datetime_now, format_date, format_datetime, format_time,
     get_date_format_hint, pretty_timedelta, utc,
 )
-from trac.util.html import plaintext
+from trac.util.html import plaintext, tag
 from trac.web.chrome import Chrome
 from trac.web.tests.api import RequestHandlerPermissionsTestCaseBase
 
@@ -37,14 +40,14 @@ class PrettyDateinfoTestCase(unittest.TestCase):
         self.env.reset_db()
 
     def _format_chrome(self, d, format, dateonly):
-        data = Chrome(self.env).populate_data(self.req, {})
+        data = Chrome(self.env).populate_data(self.req)
         return plaintext(data['pretty_dateinfo'](d, format=format,
                                                  dateonly=dateonly))
 
     def _format_timeline(self, d, format, dateonly):
-        data = Chrome(self.env).populate_data(self.req, {})
-        TimelineModule(self.env) \
-            .post_process_request(self.req, 'timeline.html', data, None)
+        data = Chrome(self.env).populate_data(self.req)
+        TimelineModule(self.env).post_process_request(
+            self.req, 'timeline.html', data, None)
         return plaintext(data['pretty_dateinfo'](d, format=format,
                                                  dateonly=dateonly))
 
@@ -91,18 +94,18 @@ class PrettyDateinfoTestCase(unittest.TestCase):
 
 class TimelinePermissionsTestCase(RequestHandlerPermissionsTestCaseBase):
 
-    authz_policy = """\
-[timeline:*]
-user1 = TIMELINE_VIEW
-user2 =
-    """
+    authz_policy = textwrap.dedent("""\
+        [timeline:*]
+        user1 = TIMELINE_VIEW
+        user2 =
+        """)
 
     def setUp(self):
         super(TimelinePermissionsTestCase, self).setUp(TimelineModule)
 
     def test_get_navigation_items_with_timeline_view(self):
         req = MockRequest(self.env, authname='user1', path_info='/timeline')
-        self.assertEqual('timeline', self.get_navigation_items(req).next()[1])
+        self.assertEqual('timeline', next(self.get_navigation_items(req))[1])
 
     def test_get_navigation_items_without_timeline_view(self):
         req = MockRequest(self.env, authname='user2', path_info='/timeline')
@@ -204,26 +207,115 @@ class TimelineModuleTestCase(unittest.TestCase):
 
         self.assertEqual(90, data['daysback'])
 
-    def test_render_rss_with_unicode_unknown_author(self):
-        self.env.config.set('trac', 'show_email_addresses', 'enabled')
-        self.env.insert_users([('blah', 'Blah user', 'blah@example.org')])
+
+class TimelineEventProviderTestCase(unittest.TestCase):
+
+    timeline_event_providers = None
+
+    @classmethod
+    def setUpClass(cls):
+
+        class TimelineEventProvider(Component):
+            implements(ITimelineEventProvider)
+
+            def __init__(self):
+                self._events = None
+
+            def get_timeline_filters(self, req):
+                yield ('test', 'Test')
+
+            def get_timeline_events(self, req, start, stop, filters):
+                return iter(self._events or ())
+
+            def render_timeline_event(self, context, field, event):
+                return event[3].render(context, field, event)
+
+        cls.timeline_event_providers = {
+            'normal': TimelineEventProvider,
+        }
+
+    @classmethod
+    def tearDownClass(cls):
+        for c in (cls.timeline_event_providers or {}).itervalues():
+            ComponentMeta.deregister(c)
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.env.config.set('trac', 'use_chunked_encoding', False)
+
+    def tearDown(self):
+        self.env.reset_db()
+
+    def test_rss(self):
+        def render(context, field, event):
+            if event[0] == 'test&1':
+                if field == 'url':
+                    return 'http://example.org/path?foo=bar&baz=1'
+                if field == 'summary':
+                    return 'summary 1: <b>&</b>'
+                if field == 'description':
+                    return tag(tag.h1('Title 1st'), tag.p('body & < >'))
+            if event[0] == 'test&2':
+                if field == 'url':
+                    return 'http://example.org/path?baz=2&foo=bar'
+                if field == 'summary':
+                    return tag('summary 2: ', tag.b('&'))
+                if field == 'description':
+                    return tag(tag.h1('Title 2nd'), tag.p('body & < >'))
+
+        provider = self.timeline_event_providers['normal'](self.env)
+        provider._events = [
+            ('test&1', datetime(2018, 4, 27, 12, 34, 56, 123456, utc),
+             'jo&hn', Mock(render=render)),
+            ('test&2', datetime(2018, 3, 19, 23, 56, 12, 987654, utc),
+             'Joe <joe@example.org>', Mock(render=render)),
+        ]
         req = MockRequest(self.env, path_info='/timeline',
                           args={'format': 'rss'})
-        rv = TimelineModule(self.env).process_request(req)
+        rv = self._process_request(req)
         self.assertEqual('timeline.rss', rv[0])
-        author = u'jöé'
-        event = ('mock', datetime_now(utc), author, None)
-        rv[1]['events'] = [
-            {'kind': event[0], 'date': event[1], 'author': event[2],
-             'data': event[3], 'event': event, 'provider': None,
-             'dateuid': '42', 'render': lambda field, context: 'mock'},
-        ]
-        output = Chrome(self.env).render_template(req, rv[0], rv[1], rv[2])
-        if not output.startswith('<?xml version="1.0"?>'):
-            raise AssertionError('Missing <?xml?> in %r' % output)
-        dc_creator = '<dc:creator>%s</dc:creator>' % author.encode('utf-8')
-        if dc_creator not in output:
-            raise AssertionError('Missing %r in %r' % (dc_creator, output))
+        self.assertEqual({'content_type': 'application/rss+xml'}, rv[2])
+        output = self._render_template(req, *rv)
+
+        self.assertIn('<title>summary 1: &lt;b&gt;&amp;&lt;/b&gt;</title>',
+                      output)
+        self.assertIn('<dc:creator>jo&amp;hn</dc:creator>', output)
+        self.assertIn('<pubDate>Fri, 27 Apr 2018 12:34:56 GMT</pubDate>',
+                      output)
+        self.assertIn('<link>http://example.org/path?foo=bar&amp;baz=1'
+                      '</link>', output)
+        self.assertIn('<guid isPermaLink="false">http://example.org/path?'
+                      'foo=bar&amp;baz=1/1524832496123456</guid>', output)
+        self.assertIn('<description>&lt;h1&gt;Title 1st&lt;/h1&gt;'
+                      '&lt;p&gt;body &amp;amp; &amp;lt; &amp;gt;&lt;/p&gt;'
+                      '</description>', output)
+        self.assertIn('<category>test&amp;1</category>', output)
+
+        self.assertIn('<title>summary 2: &amp;</title>', output)
+        self.assertIn('<author>Joe &lt;joe@example.org&gt;</author>', output)
+        self.assertIn('<pubDate>Mon, 19 Mar 2018 23:56:12 GMT</pubDate>',
+                      output)
+        self.assertIn('<link>http://example.org/path?baz=2&amp;foo=bar'
+                      '</link>', output)
+        self.assertIn('<guid isPermaLink="false">http://example.org/path?'
+                      'baz=2&amp;foo=bar/1521503772987654</guid>', output)
+        self.assertIn('<description>&lt;h1&gt;Title 2nd&lt;/h1&gt;'
+                      '&lt;p&gt;body &amp;amp; &amp;lt; &amp;gt;&lt;/p&gt;'
+                      '</description>', output)
+        self.assertIn('<category>test&amp;2</category>', output)
+
+        self.assertEqual('<?xml version="1.0"?>', output[:21])
+        minidom.parseString(output)  # verify valid xml
+
+    def _process_request(self, req):
+        mod = TimelineModule(self.env)
+        req = MockRequest(self.env, path_info='/timeline',
+                          args={'format': 'rss'})
+        self.assertTrue(mod.match_request(req))
+        return mod.process_request(req)
+
+    def _render_template(self, *args, **kwargs):
+        return Chrome(self.env).render_template(*args, **kwargs)
 
 
 def test_suite():
@@ -231,6 +323,7 @@ def test_suite():
     suite.addTest(unittest.makeSuite(PrettyDateinfoTestCase))
     suite.addTest(unittest.makeSuite(TimelinePermissionsTestCase))
     suite.addTest(unittest.makeSuite(TimelineModuleTestCase))
+    suite.addTest(unittest.makeSuite(TimelineEventProviderTestCase))
     return suite
 
 if __name__ == '__main__':
