@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,7 +11,7 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/.
 
-from trac.upgrades import backup_config_file
+from trac.util import backup_config_file
 
 
 def do_upgrade(env, version, cursor):
