@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,7 +12,7 @@
 # history and logs, available at https://trac.edgewall.org/.
 
 from trac.ticket.default_workflow import get_workflow_config
-from trac.upgrades import backup_config_file
+from trac.util import backup_config_file
 
 new_actions = {
     'create': {
@@ -33,10 +33,10 @@ def do_upgrade(env, version, cursor):
     """
     save = False
     all_actions = get_workflow_config(env.config)
-    all_states = list(set(
-        [state for action in all_actions.itervalues()
-               for state in action['oldstates']] +
-        [action['newstate'] for action in all_actions.itervalues()]))
+    all_states = list(
+        {state for action in all_actions.itervalues()
+               for state in action['oldstates']} |
+        {action['newstate'] for action in all_actions.itervalues()})
 
     for action, attributes in new_actions.items():
         if action == 'create_and_assign' and 'assigned' not in all_states:
