@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,7 +13,7 @@
 
 import unittest
 
-from trac.upgrades.tests import db31, db32, db39, db41
+from trac.upgrades.tests import db31, db32, db39, db41, db42, db44, db45
 
 
 def test_suite():
@@ -22,6 +22,9 @@ def test_suite():
     suite.addTest(db32.test_suite())
     suite.addTest(db39.test_suite())
     suite.addTest(db41.test_suite())
+    suite.addTest(db42.test_suite())
+    suite.addTest(db44.test_suite())
+    suite.addTest(db45.test_suite())
     return suite
 
 
