@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,25 +12,26 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import doctest
+import importlib
 import os.path
 import pkg_resources
 import random
 import re
 import sys
-import tempfile
+import textwrap
 import unittest
 
 import trac
 from trac import util
-from trac.test import rmtree
-from trac.util.tests import concurrency, datefmt, presentation, text, \
-                            translation, html
+from trac.test import mkdtemp, rmtree
+from trac.util.tests import (concurrency, datefmt, presentation, text,
+                             translation, html)
 
 
 class AtomicFileTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.dir = tempfile.mkdtemp()
+        self.dir = mkdtemp()
         self.path = os.path.join(self.dir, 'trac-tempfile')
 
     def tearDown(self):
@@ -80,7 +81,7 @@ class AtomicFileTestCase(unittest.TestCase):
     # to remove the dependency on the locale. So the test is disabled until
     # we require Python 3.
     def _test_unicode_path(self):
-        self.path = os.path.join(tempfile.gettempdir(), u'träc-témpfilè')
+        self.path = os.path.join(self.dir, u'träc-témpfilè')
         with util.AtomicFile(self.path) as f:
             f.write('test content')
         self.assertTrue(f.closed)
@@ -110,6 +111,25 @@ class PathTestCase(unittest.TestCase):
         self.assertFalse(util.is_path_below('../sub/repos',
                                             os.path.join(os.getcwd())))
 
+    def test_native_path(self):
+        self.assertIsNone(util.native_path(None))
+        if os.name == 'posix':
+            self.assertEqual('/D/Trac/x', util.native_path('D:\\Trac\\x'))
+            self.assertEqual('/D/Trac/x', util.native_path('/D/Trac/x'))
+            self.assertEqual('/D/', util.native_path('D:\\'))
+            self.assertEqual('/Trac/x', util.native_path('\\Trac\\x'))
+            self.assertEqual('Trac/x', util.native_path('Trac\\x'))
+            self.assertEqual('Trac/x', util.native_path('Trac/x'))
+        elif os.name == 'nt':
+            self.assertEqual('D:\\Trac\\x', util.native_path('/D/Trac/x'))
+            self.assertEqual('D:\\Trac\\x', util.native_path('D:/Trac/x'))
+            self.assertEqual('D:\\Trac\\x', util.native_path('D:\\Trac\\x'))
+            self.assertEqual('D:\\', util.native_path('/D/'))
+            self.assertEqual('D:', util.native_path('/D'))
+            self.assertEqual('C:\\', util.native_path('/'))
+            self.assertEqual('C:\\Trac\\x', util.native_path('/Trac/x'))
+            self.assertEqual('Trac\\x', util.native_path('Trac/x'))
+            self.assertEqual('Trac\\x', util.native_path('Trac\\x'))
 
 class RandomTestCase(unittest.TestCase):
 
@@ -125,7 +145,7 @@ class RandomTestCase(unittest.TestCase):
             self.assertEqual(i, len(util.urandom(i)))
         # For a large enough sample, each value should appear at least once
         entropy = util.urandom(65536)
-        values = set(ord(c) for c in entropy)
+        values = {ord(c) for c in entropy}
         self.assertEqual(256, len(values))
 
     def test_hex_entropy(self):
@@ -184,7 +204,7 @@ class SafeReprTestCase(unittest.TestCase):
 class SetuptoolsUtilsTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.dir = tempfile.mkdtemp()
+        self.dir = mkdtemp()
         sys.path.append(self.dir)
 
     def tearDown(self):
@@ -232,19 +252,17 @@ class SetuptoolsUtilsTestCase(unittest.TestCase):
             self.assertNotEqual({}, pkginfo)
             self.assertEqual(pkginfo, util.get_pkginfo(babel.core))
 
-    def test_get_pkginfo_mysqldb(self):
-        # MySQLdb's package name is "MySQL-Python"
+    def test_get_pkginfo_pymysql(self):
         try:
-            import MySQLdb
-            import MySQLdb.cursors
-            dist = pkg_resources.get_distribution('MySQL-Python')
+            import pymysql
+            dist = pkg_resources.get_distribution('pymysql')
             dist.get_metadata('top_level.txt')
         except:
             pass
         else:
-            pkginfo = util.get_pkginfo(MySQLdb)
+            pkginfo = util.get_pkginfo(pymysql)
             self.assertNotEqual({}, pkginfo)
-            self.assertEqual(pkginfo, util.get_pkginfo(MySQLdb.cursors))
+            self.assertEqual(pkginfo, util.get_pkginfo(pymysql.cursors))
 
     def test_get_pkginfo_psycopg2(self):
         # python-psycopg2 deb package doesn't provide SOURCES.txt and
@@ -269,6 +287,8 @@ class SetuptoolsUtilsTestCase(unittest.TestCase):
                     'Version: 0.1\n'
                     'Author: Joe\n'
                     'Author-email: joe@example.org\n'
+                    'Maintainer: Jim\n'
+                    'Maintainer-email: jim@example.org\n'
                     'Home-page: http://example.org/\n'
                     'Summary: summary.\n'
                     'Description: description.\n'
@@ -280,17 +300,74 @@ class SetuptoolsUtilsTestCase(unittest.TestCase):
             with open(os.path.join(self.dir, modname, name), 'w') as f:
                 f.write('# -*- coding: utf-8 -*-\n')
 
-        mod = __import__(modname, {}, {}, ['bar', 'foo'])
+        mod = importlib.import_module(modname)
+        mod.bar = importlib.import_module(modname + '.bar')
+        mod.foo = importlib.import_module(modname + '.foo')
         pkginfo = util.get_pkginfo(mod)
         self.assertEqual('0.1', pkginfo['version'])
         self.assertEqual('Joe', pkginfo['author'])
         self.assertEqual('joe@example.org', pkginfo['author_email'])
+        self.assertEqual('Jim', pkginfo['maintainer'])
+        self.assertEqual('jim@example.org', pkginfo['maintainer_email'])
         self.assertEqual('http://example.org/', pkginfo['home_page'])
         self.assertEqual('summary.', pkginfo['summary'])
         self.assertEqual('description.', pkginfo['description'])
         self.assertEqual(pkginfo, util.get_pkginfo(mod.bar))
         self.assertEqual(pkginfo, util.get_pkginfo(mod.foo))
 
+    def _write_module(self, version, url):
+        modname = 'TestModule_' + util.hex_entropy(16)
+        modpath = os.path.join(self.dir, modname + '.py')
+        with open(modpath, 'w') as f:
+            f.write(textwrap.dedent("""\
+                # -*- coding: utf-8 -*-
+                from trac.core import Component
+
+                version = '%s'
+                author = 'Joe'
+                author_email = 'joe@example.org'
+                maintainer = 'Jim'
+                maintainer_email = 'jim@example.org'
+                home_page = '%s'
+                license = 'BSD 3-Clause'
+                summary = 'summary.'
+                trac = 'http://my.trac.com'
+
+                class TestModule(Component):
+                    pass
+                """) % (version, url))
+        return modname
+
+    def test_get_module_metadata(self):
+        version = '0.1'
+        home_page = 'http://example.org'
+        modname = self._write_module(version, home_page)
+
+        mod = importlib.import_module(modname)
+        info = util.get_module_metadata(mod)
+
+        self.assertEqual(version, info['version'])
+        self.assertEqual('Joe', info['author'])
+        self.assertEqual('joe@example.org', info['author_email'])
+        self.assertEqual('Jim', info['maintainer'])
+        self.assertEqual('jim@example.org', info['maintainer_email'])
+        self.assertEqual(home_page, info['home_page'])
+        self.assertEqual('summary.', info['summary'])
+        self.assertEqual('BSD 3-Clause', info['license'])
+        self.assertEqual('http://my.trac.com', info['trac'])
+
+    def test_get_module_metadata_keyword_expansion(self):
+        version = '10'
+        url = 'http://example.org'
+        modname = self._write_module('$Rev: %s $' % version,
+                                     '$URL: %s $' % url)
+
+        mod = importlib.import_module(modname)
+        info = util.get_module_metadata(mod)
+
+        self.assertEqual('r%s' % version, info['version'])
+        self.assertEqual(url, info['home_page'])
+
 
 class LazyClass(object):
     @util.lazy
@@ -320,7 +397,7 @@ class LazyTestCase(unittest.TestCase):
 class FileTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.dir = tempfile.mkdtemp()
+        self.dir = mkdtemp()
         self.filename = os.path.join(self.dir, 'trac-tempfile')
         self.data = 'Lorem\ripsum\ndolor\r\nsit\namet,\rconsectetur\r\n'
 
@@ -344,6 +421,25 @@ class FileTestCase(unittest.TestCase):
         self.assertTrue(os.path.isfile(self.filename))
         self.assertEqual(0, os.path.getsize(self.filename))
 
+class UtilitiesTestCase(unittest.TestCase):
+
+    def test_as_int(self):
+        self.assertEqual(1, util.as_int('1'))
+        self.assertEqual(1, util.as_int('1', None))
+        self.assertIsNone(util.as_int('A', None))
+        self.assertEqual(2, util.as_int('A', 2))
+        self.assertEqual(2, util.as_int('1', None, min=2))
+        self.assertEqual(0, util.as_int('1', None, max=0))
+
+    def test_as_float(self):
+        self.assertEqual(1.1, util.as_float('1.1'))
+        self.assertEqual(1.1, util.as_float('1.1', None))
+        self.assertEqual(1, util.as_float('1', None))
+        self.assertIsNone(util.as_float('A', None))
+        self.assertEqual(2.2, util.as_float('A', 2.2))
+        self.assertEqual(2.2, util.as_float('1.1', None, min=2.2))
+        self.assertEqual(0.1, util.as_float('1.1', None, max=0.1))
+
 
 def test_suite():
     suite = unittest.TestSuite()
@@ -355,6 +451,7 @@ def test_suite():
     suite.addTest(unittest.makeSuite(SetuptoolsUtilsTestCase))
     suite.addTest(unittest.makeSuite(LazyTestCase))
     suite.addTest(unittest.makeSuite(FileTestCase))
+    suite.addTest(unittest.makeSuite(UtilitiesTestCase))
     suite.addTest(concurrency.test_suite())
     suite.addTest(datefmt.test_suite())
     suite.addTest(presentation.test_suite())
@@ -362,6 +459,7 @@ def test_suite():
     suite.addTest(text.test_suite())
     suite.addTest(translation.test_suite())
     suite.addTest(html.test_suite())
+    suite.addTest(doctest.DocTestSuite(util.html))
     return suite
 
 if __name__ == '__main__':
