@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # Copyright (C) 2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -12,13 +12,29 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
 
+import datetime
 import doctest
 import unittest
 
 from trac.core import TracError
-from trac.util import presentation
+from trac.test import Mock
+from trac.util import datefmt, presentation
 
 
+class FiltersTestCase(unittest.TestCase):
+    def test_htmlattr(self):
+        self.assertEqual(
+            (' autocomplete="on" checked="checked" class="my list"'
+             ' id="list-42"'
+             ' style="border-radius: 3px; background: #f7f7f7"'),
+            presentation.htmlattr_filter(
+                Mock(autoescape=False),
+                {'class': {'my': 1, 'list': True, 'empty': False},
+                 'missing': None, 'checked': 1, 'selected': False,
+                 'autocomplete': True, 'id': 'list-%d' % 42,
+                 'style': {'border-radius': '3px',
+                           'background': '#f7f7f7'}}))
+
 class ToJsonTestCase(unittest.TestCase):
 
     def test_simple_types(self):
@@ -28,6 +44,12 @@ class ToJsonTestCase(unittest.TestCase):
         self.assertEqual('false', presentation.to_json(False))
         self.assertEqual('null', presentation.to_json(None))
         self.assertEqual('"String"', presentation.to_json('String'))
+        self.assertEqual('1551895815012345',
+                         presentation.to_json(datetime.datetime(
+                             2019, 3, 6, 18, 10, 15, 12345, datefmt.utc)))
+        self.assertEqual('1551895815012345',
+                         presentation.to_json(datetime.datetime(
+                             2019, 3, 6, 18, 10, 15, 12345)))
         self.assertEqual(r'"a \" quote"', presentation.to_json('a " quote'))
         self.assertEqual('''"a ' single quote"''',
                          presentation.to_json("a ' single quote"))
@@ -52,7 +74,7 @@ class PaginatorTestCase(unittest.TestCase):
 
     def test_paginate(self):
         """List of objects is paginated."""
-        items = range(0, 20)
+        items = xrange(20)
         paginator = presentation.Paginator(items, 1)
 
         self.assertEqual(1, paginator.page)
@@ -63,7 +85,7 @@ class PaginatorTestCase(unittest.TestCase):
 
     def test_page_out_of_range_raises_exception(self):
         """Out of range value for page raises a `TracError`."""
-        items = range(0, 20)
+        items = list(xrange(20))
 
         self.assertRaises(TracError, presentation.Paginator, items, 2)
 
@@ -71,6 +93,7 @@ class PaginatorTestCase(unittest.TestCase):
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(presentation))
+    suite.addTest(unittest.makeSuite(FiltersTestCase))
     suite.addTest(unittest.makeSuite(ToJsonTestCase))
     suite.addTest(unittest.makeSuite(PaginatorTestCase))
     return suite
