@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,20 +11,20 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os
+import re
 import socket
 import textwrap
 import unittest
-from StringIO import StringIO
 
-import trac.tests.compat
 from trac.util.text import (
-    _get_default_ambiwidth, empty, expandtabs, fix_eol, javascript_quote,
-    levenshtein_distance, normalize_whitespace, print_table,
-    quote_query_string, shorten_line, strip_line_ws, stripws, sub_vars,
-    text_width, to_js_string, to_unicode, to_utf8, unicode_from_base64,
-    unicode_quote, unicode_quote_plus, unicode_to_base64, unicode_unquote,
-    unicode_urlencode, wrap)
+    _get_default_ambiwidth, empty, exception_to_unicode, expandtabs, fix_eol,
+    javascript_quote, jinja2template, levenshtein_distance,
+    normalize_whitespace, print_table, quote_query_string, shorten_line,
+    strip_line_ws, stripws, sub_vars, text_width, to_js_string, to_unicode,
+    to_utf8, unicode_from_base64, unicode_quote, unicode_quote_plus,
+    unicode_to_base64, unicode_unquote, unicode_urlencode, wrap)
 
 
 class ToUnicodeTestCase(unittest.TestCase):
@@ -46,25 +46,36 @@ class ToUnicodeTestCase(unittest.TestCase):
     def test_from_exception_using_unicode_args(self):
         u = u'\uB144'
         try:
-            raise ValueError, '%s is not a number.' % u
+            raise ValueError('%s is not a number.' % u)
         except ValueError as e:
             self.assertEqual(u'\uB144 is not a number.', to_unicode(e))
 
     def test_from_exception_using_str_args(self):
         u = u'Das Ger\xe4t oder die Ressource ist belegt'
         try:
-            raise ValueError, u.encode('utf-8')
+            raise ValueError(u.encode('utf-8'))
         except ValueError as e:
             self.assertEqual(u, to_unicode(e))
 
     def test_from_windows_error(self):
         try:
-            os.stat('non/existent/file.txt')
+            os.stat('non\\existent\\file.txt')
         except OSError as e:
             uc = to_unicode(e)
             self.assertIsInstance(uc, unicode, uc)
             self.assertTrue(uc.startswith('[Error '), uc)
             self.assertIn(e.strerror.decode('mbcs'), uc)
+            self.assertTrue(uc.endswith(u": 'non\\existent\\file.txt'"), uc)
+
+    def test_from_windows_error_with_unicode_path(self):
+        try:
+            os.stat(u'nön\\existént\\file.txt')
+        except OSError as e:
+            uc = to_unicode(e)
+            self.assertIsInstance(uc, unicode, uc)
+            self.assertTrue(uc.startswith('[Error '), uc)
+            self.assertIn(e.strerror.decode('mbcs'), uc)
+            self.assertTrue(uc.endswith(u": 'nön\\existént\\file.txt'"), uc)
 
     def test_from_socket_error(self):
         for res in socket.getaddrinfo('127.0.0.1', 65536, 0,
@@ -81,6 +92,7 @@ class ToUnicodeTestCase(unittest.TestCase):
 
     if os.name != 'nt':
         del test_from_windows_error
+        del test_from_windows_error_with_unicode_path
         del test_from_socket_error
 
 
@@ -318,7 +330,7 @@ class PrintTableTestCase(unittest.TestCase):
         self._validate_print_table(expected, data, headers=headers, sep=' | ')
 
     def _validate_print_table(self, expected, data, **kwargs):
-        out = StringIO()
+        out = io.BytesIO()
         kwargs['out'] = out
         print_table(data, **kwargs)
         self.assertEqual(expected.encode('utf-8'),
@@ -414,6 +426,17 @@ class StripwsTestCase(unittest.TestCase):
                                  leading=False, trailing=False))
 
 
+class Jinja2TemplateTestCase(unittest.TestCase):
+    def test_html_template(self):
+        self.assertEqual("<h1>Hell&amp;O</h1>",
+                         jinja2template("<h1>${hell}O</h1>").render(
+                             {'hell': 'Hell&'}))
+    def test_text_template(self):
+        self.assertEqual("<h1>Hell&O</h1>",
+                         jinja2template("<h1>${hell}O</h1>", text=True).render(
+                             {'hell': 'Hell&'}))
+
+
 class LevenshteinDistanceTestCase(unittest.TestCase):
     def test_distance(self):
         self.assertEqual(5, levenshtein_distance('kitten', 'sitting'))
@@ -513,6 +536,28 @@ class DefaultAmbiwidthTestCase(unittest.TestCase):
         self.assertEqual(1, _get_default_ambiwidth())
 
 
+class ExceptionToUnicodeTestCase(unittest.TestCase):
+
+    def test_without_traceback(self):
+        try:
+            raise ValueError('test')
+        except ValueError as e:
+            self.assertEqual('ValueError: test', exception_to_unicode(e))
+
+    def test_with_traceback(self):
+        try:
+            raise ValueError('test')
+        except ValueError as e:
+            result = exception_to_unicode(e, traceback=True)
+            result = re.sub(r'\n  File "[^"]+", line [0-9]+, ',
+                            '\n  File "<file>", line <line>, ', result)
+            self.assertEqual("""
+Traceback (most recent call last):
+  File "<file>", line <line>, in test_with_traceback
+    raise ValueError('test')
+ValueError: test""", result)
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ToUnicodeTestCase))
@@ -529,11 +574,13 @@ def test_suite():
     suite.addTest(unittest.makeSuite(FixEolTestCase))
     suite.addTest(unittest.makeSuite(UnicodeBase64TestCase))
     suite.addTest(unittest.makeSuite(StripwsTestCase))
+    suite.addTest(unittest.makeSuite(Jinja2TemplateTestCase))
     suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase))
     suite.addTest(unittest.makeSuite(SubVarsTestCase))
     suite.addTest(unittest.makeSuite(ShortenLineTestCase))
     if os.name != 'nt':
         suite.addTest(unittest.makeSuite(DefaultAmbiwidthTestCase))
+    suite.addTest(unittest.makeSuite(ExceptionToUnicodeTestCase))
     return suite
 
 
