@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,13 +13,16 @@
 
 import unittest
 
-from trac.versioncontrol.tests import admin, cache, diff, svn_authz, api
+from trac.versioncontrol.tests import admin, cache, console, diff, \
+                                      svn_authz, api
 from trac.versioncontrol.tests.functional import functionalSuite
 
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(admin.test_suite())
     suite.addTest(cache.test_suite())
+    suite.addTest(console.test_suite())
     suite.addTest(diff.test_suite())
     suite.addTest(svn_authz.test_suite())
     suite.addTest(api.test_suite())
