@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -36,55 +36,55 @@ class DiffTestCase(unittest.TestCase):
 
     def test_insert_blank_line(self):
         opcodes = get_opcodes(['A', 'B'], ['A', 'B', ''], ignore_blank_lines=0)
-        self.assertEqual(('equal', 0, 2, 0, 2), opcodes.next())
-        self.assertEqual(('insert', 2, 2, 2, 3), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 2), next(opcodes))
+        self.assertEqual(('insert', 2, 2, 2, 3), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B'], ['A', 'B', ''], ignore_blank_lines=1)
-        self.assertEqual(('equal', 0, 2, 0, 3), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 3), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A'], ['A', 'B', ''], ignore_blank_lines=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('insert', 1, 1, 1, 3), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('insert', 1, 1, 1, 3), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A'], ['A', 'B', ''], ignore_blank_lines=1)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('insert', 1, 1, 1, 3), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('insert', 1, 1, 1, 3), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_delete_blank_line(self):
         opcodes = get_opcodes(['A', 'B', ''], ['A', 'B'], ignore_blank_lines=0)
-        self.assertEqual(('equal', 0, 2, 0, 2), opcodes.next())
-        self.assertEqual(('delete', 2, 3, 2, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 2), next(opcodes))
+        self.assertEqual(('delete', 2, 3, 2, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B', ''], ['A', 'B'], ignore_blank_lines=1)
-        self.assertEqual(('equal', 0, 3, 0, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 3, 0, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B', ''], ['A'], ignore_blank_lines=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('delete', 1, 3, 1, 1), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('delete', 1, 3, 1, 1), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B', ''], ['A'], ignore_blank_lines=1)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('delete', 1, 3, 1, 1), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('delete', 1, 3, 1, 1), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_space_changes(self):
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B  b'],
                               ignore_space_changes=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('replace', 1, 2, 1, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('replace', 1, 2, 1, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B  b'],
                               ignore_space_changes=1)
-        self.assertEqual(('equal', 0, 2, 0, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_space_changes_2(self):
         left = textwrap.dedent("""\
@@ -109,47 +109,47 @@ class DiffTestCase(unittest.TestCase):
             """)
         right = right.splitlines()
         opcodes = get_opcodes(left, right, ignore_space_changes=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('replace', 1, 6, 1, 5), opcodes.next())
-        self.assertEqual(('equal', 6, 8, 5, 7), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('replace', 1, 6, 1, 5), next(opcodes))
+        self.assertEqual(('equal', 6, 8, 5, 7), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(left, right, ignore_space_changes=1)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('delete', 1, 2, 1, 1), opcodes.next())
-        self.assertEqual(('equal', 2, 4, 1, 3), opcodes.next())
-        self.assertEqual(('replace', 4, 5, 3, 4), opcodes.next())
-        self.assertEqual(('equal', 5, 8, 4, 7), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('delete', 1, 2, 1, 1), next(opcodes))
+        self.assertEqual(('equal', 2, 4, 1, 3), next(opcodes))
+        self.assertEqual(('replace', 4, 5, 3, 4), next(opcodes))
+        self.assertEqual(('equal', 5, 8, 4, 7), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_case_changes(self):
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B B'], ignore_case=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('replace', 1, 2, 1, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('replace', 1, 2, 1, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B B'], ignore_case=1)
-        self.assertEqual(('equal', 0, 2, 0, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_space_and_case_changes(self):
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B  B'],
                               ignore_case=0, ignore_space_changes=0)
-        self.assertEqual(('equal', 0, 1, 0, 1), opcodes.next())
-        self.assertEqual(('replace', 1, 2, 1, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 1, 0, 1), next(opcodes))
+        self.assertEqual(('replace', 1, 2, 1, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
         opcodes = get_opcodes(['A', 'B b'], ['A', 'B  B'],
                               ignore_case=1, ignore_space_changes=1)
-        self.assertEqual(('equal', 0, 2, 0, 2), opcodes.next())
-        self.assertRaises(StopIteration, opcodes.next)
+        self.assertEqual(('equal', 0, 2, 0, 2), next(opcodes))
+        self.assertRaises(StopIteration, next, opcodes)
 
     def test_grouped_opcodes_context1(self):
         groups = diff.get_filtered_hunks(
             ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H'],
             ['A', 'B', 'C', 'd', 'e', 'f', 'G', 'H'], context=1)
-        group = groups.next()
-        self.assertRaises(StopIteration, groups.next)
+        group = next(groups)
+        self.assertRaises(StopIteration, next, groups)
         self.assertEqual(('equal', 2, 3, 2, 3), group[0])
         self.assertEqual(('replace', 3, 6, 3, 6), group[1])
         self.assertEqual(('equal', 6, 7, 6, 7), group[2])
@@ -158,11 +158,11 @@ class DiffTestCase(unittest.TestCase):
         old = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
         new = ['X', 'B', 'C', 'd', 'e', 'f', 'G', 'Y']
         groups = diff.get_filtered_hunks(old, new, context=1, ignore_case=1)
-        group = groups.next()
+        group = next(groups)
         self.assertEqual([('replace', 0, 1, 0, 1), ('equal', 1, 2, 1, 2)],
                          group)
-        group = groups.next()
-        self.assertRaises(StopIteration, groups.next)
+        group = next(groups)
+        self.assertRaises(StopIteration, next, groups)
         self.assertEqual([('equal', 6, 7, 6, 7), ('replace', 7, 8, 7, 8)],
                          group)
 
@@ -170,8 +170,8 @@ class DiffTestCase(unittest.TestCase):
         old = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
         new = ['X', 'B', 'C', 'd', 'e', 'f', 'G', 'Y']
         groups = diff.get_filtered_hunks(old, new, context=None)
-        group = groups.next()
-        self.assertRaises(StopIteration, groups.next)
+        group = next(groups)
+        self.assertRaises(StopIteration, next, groups)
         self.assertEqual([
                 ('replace', 0, 1, 0, 1),
                 ('equal', 1, 3, 1, 3),
@@ -181,8 +181,8 @@ class DiffTestCase(unittest.TestCase):
                 ], group)
 
         groups = diff.get_filtered_hunks(old, new, context=None, ignore_case=1)
-        group = groups.next()
-        self.assertRaises(StopIteration, groups.next)
+        group = next(groups)
+        self.assertRaises(StopIteration, next, groups)
         self.assertEqual([
                 ('replace', 0, 1, 0, 1),
                 ('equal', 1, 7, 1, 7),
@@ -198,8 +198,8 @@ class DiffTestCase(unittest.TestCase):
                                          ['A', 'B', 'C', 'D', 'E', 'F', 'G'],
                                          context=3)
         self.assertEqual([('insert', 0, 0, 0, 1), ('equal', 0, 3, 1, 4)],
-                         groups.next())
-        self.assertRaises(StopIteration, groups.next)
+                         next(groups))
+        self.assertRaises(StopIteration, next, groups)
 
     def test_unified_diff_no_context(self):
         diff_lines = list(diff.unified_diff(['a'], ['b']))
