@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,23 +12,28 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import os.path
-import tempfile
 import textwrap
 import unittest
 
 from trac.config import ConfigurationError
 from trac.resource import Resource
-from trac.test import EnvironmentStub, Mock
+from trac.test import EnvironmentStub, Mock, mkdtemp, rmtree
 from trac.util import create_file
 from trac.versioncontrol.api import RepositoryManager
-from trac.versioncontrol.svn_authz import AuthzSourcePolicy, ParseError, \
-                                          parse
+from trac.versioncontrol.svn_authz import AuthzSourcePolicy, parse
 
 
 class AuthzParserTestCase(unittest.TestCase):
 
+    def setUp(self):
+        self.tmpdir = mkdtemp()
+        self.authz_file = os.path.join(self.tmpdir, 'trac-authz')
+
+    def tearDown(self):
+        rmtree(self.tmpdir)
+
     def test_parse_file(self):
-        authz = parse(textwrap.dedent("""\
+        create_file(self.authz_file, textwrap.dedent("""\
             [groups]
             developers = foo, bar
             users = @developers, &baz
@@ -59,27 +64,34 @@ class AuthzParserTestCase(unittest.TestCase):
             Foo = rw
             BAZ = r
 
+            ; Unicode module names
+            [module:/c/résumé]
+            bar = rw
+            Foo = rw
+            BAZ = r
+
             ; Unused module, not parsed
             [unused:/some/path]
             foo = r
-            """), set(['', 'module']))
+            """))
+        authz = parse(self.authz_file, {'', 'module'})
         self.assertEqual({
             '': {
-                '/': {
-                    '*': True,
+                u'/': {
+                    u'*': True,
                 },
-                '/trunk': {
-                    'foo': True,
-                    'bar': True,
+                u'/trunk': {
+                    u'foo': True,
+                    u'bar': True,
                     u'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
                 },
-                '/branches': {
-                    'bar': True,
+                u'/branches': {
+                    u'bar': True,
                 },
             },
-            'module': {
-                '/trunk': {
-                    'foo': True,
+            u'module': {
+                u'/trunk': {
+                    u'foo': True,
                     u'CN=Hàröld Hacker,OU=Enginéers,DC=red-bean,DC=com': True,
                 },
                 u'/c/résumé': {
@@ -90,25 +102,13 @@ class AuthzParserTestCase(unittest.TestCase):
             },
         }, authz)
 
-    def test_parse_errors(self):
-        self.assertRaises(ParseError, parse, textwrap.dedent("""\
-            user = r
-
-            [module:/trunk]
-            user = r
-            """), set(['', 'module']))
-        self.assertRaises(ParseError, parse, textwrap.dedent("""\
-            [module:/trunk]
-            user
-            """), set(['', 'module']))
-
 
 class AuthzSourcePolicyTestCase(unittest.TestCase):
 
     def setUp(self):
-        tmpdir = tempfile.mkdtemp(prefix='trac-')
-        self.authz = os.path.join(tmpdir, 'trac-authz')
-        create_file(self.authz, textwrap.dedent("""\
+        tmpdir = mkdtemp()
+        self.authz_file = os.path.join(tmpdir, 'trac-authz')
+        create_file(self.authz_file, textwrap.dedent("""\
             [groups]
             group1 = user
             group2 = @group1
@@ -254,16 +254,15 @@ class AuthzSourcePolicyTestCase(unittest.TestCase):
         self.env = EnvironmentStub(enable=[AuthzSourcePolicy], path=tmpdir)
         self.env.config.set('trac', 'permission_policies',
                             'AuthzSourcePolicy, DefaultPermissionPolicy')
-        self.env.config.set('svn', 'authz_file', self.authz)
+        self.env.config.set('svn', 'authz_file', self.authz_file)
 
         # Monkey-subclass RepositoryManager to serve mock repositories
         rm = RepositoryManager(self.env)
 
         class TestRepositoryManager(rm.__class__):
             def get_real_repositories(self):
-                return set([Mock(reponame='module'),
-                            Mock(reponame='other'),
-                            Mock(reponame='scoped')])
+                return {Mock(reponame='module'), Mock(reponame='other'),
+                        Mock(reponame='scoped')}
 
             def get_repository(self, reponame):
                 if reponame == 'scoped':
@@ -314,14 +313,14 @@ class AuthzSourcePolicyTestCase(unittest.TestCase):
     def test_get_authz_file_removed_raises(self):
         """ConfigurationError exception is raised if file is removed."""
         policy = AuthzSourcePolicy(self.env)
-        os.remove(self.authz)
+        os.remove(self.authz_file)
         self.assertRaises(ConfigurationError, policy.check_permission,
                           'BROWSER_VIEW', 'user', None, None)
 
     def test_parse_error_raises(self):
         """ConfigurationError exception is raised when exception occurs
         parsing the `[svn authz_file`."""
-        create_file(self.authz, textwrap.dedent("""\
+        create_file(self.authz_file, textwrap.dedent("""\
             [/somepath
             joe = r
             """))
@@ -364,7 +363,7 @@ class AuthzSourcePolicyTestCase(unittest.TestCase):
         self.assertRevPerm(True, 'joe')
         # Granted if at least one fine permission is granted
         policy._mtime = 0
-        create_file(self.authz, textwrap.dedent("""\
+        create_file(self.authz_file, textwrap.dedent("""\
             [/somepath]
             joe = r
             denied =
