@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -26,7 +26,7 @@ from trac.versioncontrol.web_ui.log import LogModule
 from trac.web.api import parse_arg_list
 from trac.web.chrome import web_context
 from trac.web.tests.api import RequestHandlerPermissionsTestCaseBase
-from trac.wiki.formatter import format_to_html
+from trac.wiki.formatter import format_to_oneliner
 
 
 mock_repotype = 'mock:' + __name__
@@ -48,13 +48,7 @@ class TestLogModulePermissionPolicy(Component):
     implements(IPermissionPolicy)
 
     def check_permission(self, action, username, resource, perm):
-        if action == 'LOG_VIEW':
-            if resource.realm != 'source':
-                raise AssertionError('Invalid realm %r' % resource.realm)
-            if resource.id == 'deny' or resource.id.endswith('/deny'):
-                return False
-            if resource.id == 'allow' or resource.id.endswith('/allow'):
-                return True
+        if action == 'LOG_VIEW' and resource.realm == 'repository':
             return username != 'anonymous'
 
 
@@ -184,7 +178,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_without_rev(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         self.assertEqual('revisionlog.html', template)
         items = data['items']
         self.assertEqual(5, len(items))
@@ -200,7 +194,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '49'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -214,7 +208,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev_and_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '49', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -234,7 +228,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev_on_start(self):
         req = MockRequest(self.env, path_info='/log/mock/file-old',
                           args={'rev': '10'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(4, len(items))
         self.assertEqual([10, 7, 4, 1],
@@ -250,7 +244,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_with_rev_and_limit_on_start(self):
         req = MockRequest(self.env, path_info='/log/mock/file-old',
                           args={'rev': '10', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(4, len(items))
         self.assertEqual([10, 7, 4, 1],
@@ -278,7 +272,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_1(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '70,79-82,94-100'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(9, len(items))
         self.assertEqual([100, 97, 94, 91, 82, 79, 76, 70, 67],
@@ -296,7 +290,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_2(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-49'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -313,7 +307,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_3(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-46,55-61'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -331,7 +325,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_4(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '40-46,55-61'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -349,7 +343,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_1_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '70,79-82,94-100', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(6, len(items))
         self.assertEqual([100, 97, 94, 91, 82, 79],
@@ -372,7 +366,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             req = MockRequest(self.env, path_info='/log/mock/file',
                               args={'revs': '70,79-82,94-100',
                                     'limit': str(limit)})
-            template, data, ctype = self.process_request(req)
+            template, data = self.process_request(req)
             links = req.chrome['links']
             if 'next' in links:
                 link = links['next'][0]['href']
@@ -391,12 +385,12 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
                           'mode': 'stop_on_copy'}, next_link_args(4))
         self.assertEqual({'limit': '5', 'rev': '76', 'revs': '70',
                           'mode': 'stop_on_copy'}, next_link_args(5))
-        self.assertEqual(None, next_link_args(6))
+        self.assertIsNone(next_link_args(6))
 
     def test_revranges_2_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-49', 'limit': '4'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([49, 46, 43, 40, 37],
@@ -417,7 +411,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_3_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '22-46,55-61', 'limit': '7'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -435,7 +429,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_revranges_4_with_limit(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'revs': '40-46,55-61', 'limit': '7'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(8, len(items))
         self.assertEqual([61, 58, 55, 52, 46, 43, 40, 37],
@@ -471,7 +465,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'rev': '43', 'limit': '4',
                                 'mode': 'follow_copy'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(5, len(items))
         self.assertEqual([43, 40, 37, 34, 31],
@@ -491,7 +485,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_path_history(self):
         req = MockRequest(self.env, path_info='/log/mock/file',
                           args={'mode': 'path_history'})
-        template, data, ctype = self.process_request(req)
+        template, data = self.process_request(req)
         items = data['items']
         self.assertEqual(3, len(items))
         self.assertEqual(['delete', 'move', 'add'],
@@ -501,22 +495,19 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
     def test_log_link_checking_repository_resource(self):
         self.env.config.set('trac', 'permission_policies',
             'TestLogModulePermissionPolicy, DefaultPermissionPolicy')
-
-        self.assertIn(' title="No permission to view change log"',
-                      self._format_to_html('anonymous', 'log:mock@42-43'))
-        self.assertIn(' href="/trac.cgi/log/mock/allow?revs=42-43"',
-                      self._format_to_html('anonymous',
-                                           'log:mock/allow@42-43'))
-        self.assertIn(' href="/trac.cgi/log/mock/?revs=42-43"',
-                      self._format_to_html('blah', 'log:mock@42-43'))
-        self.assertIn(' title="No permission to view change log"',
-                      self._format_to_html('blah', 'log:mock/deny'))
-
-    def _format_to_html(self, authname, wiki):
         resource = Resource('wiki', 'WikiStart')
-        req = MockRequest(self.env, authname=authname)
-        return unicode(format_to_html(self.env, web_context(req, resource),
-                                      wiki))
+
+        req = MockRequest(self.env, authname='anonymous')
+        rendered = unicode(format_to_oneliner(self.env,
+                                              web_context(req, resource),
+                                              'log:mock@42-43'))
+        self.assertIn(' title="No permission to view change log"', rendered)
+
+        req = MockRequest(self.env, authname='blah')
+        rendered = unicode(format_to_oneliner(self.env,
+                                              web_context(req, resource),
+                                              'log:mock@42-43'))
+        self.assertIn(' href="/trac.cgi/log/mock/?revs=42-43"', rendered)
 
 
 def test_suite():
