@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Matthew Good <trac@matt-good.net>
 # All rights reserved.
@@ -19,7 +19,6 @@
 from __future__ import print_function
 
 import cgi
-import dircache
 import fnmatch
 from functools import partial
 import gc
@@ -33,9 +32,7 @@ import sys
 import traceback
 from urlparse import urlparse
 
-from genshi.builder import tag
-from genshi.output import DocType
-from genshi.template import TemplateLoader
+from jinja2 import FileSystemLoader
 
 from trac import __version__ as TRAC_VERSION
 from trac.config import BoolOption, ChoiceOption, ConfigSection, \
@@ -49,23 +46,25 @@ from trac.resource import ResourceNotFound
 from trac.util import arity, get_frame_info, get_last_traceback, hex_entropy, \
                       lazy, read_file, safe_repr, translation, \
                       warn_setuptools_issue
-from trac.util.concurrency import threading
+from trac.util.concurrency import get_thread_id
 from trac.util.datefmt import format_datetime, localtz, timezone, user_time
-from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
-                           to_utf8, unicode_quote
+from trac.util.html import tag, valid_html_bytes
+from trac.util.text import (exception_to_unicode, jinja2env, shorten_line,
+                            to_unicode, to_utf8, unicode_quote)
 from trac.util.translation import _, get_negotiated_locale, has_babel, \
                                   safefmt, tag_
 from trac.web.api import HTTPBadRequest, HTTPException, HTTPForbidden, \
-                         HTTPInternalError, HTTPNotFound, IAuthenticator, \
+                         HTTPInternalServerError, HTTPNotFound, IAuthenticator, \
                          IRequestFilter, IRequestHandler, Request, \
                          RequestDone, TracNotImplementedError, \
                          is_valid_default_handler
-from trac.web.chrome import Chrome, ITemplateProvider, add_notice, add_warning
+from trac.web.chrome import Chrome, ITemplateProvider, add_notice, \
+                            add_stylesheet, add_warning
 from trac.web.href import Href
 from trac.web.session import SessionDict, Session
 
 #: This URL is used for semi-automatic bug reports (see
-#: `send_internal_error`).  Please modify it to point to your own
+#: `send_internal_error`). Please modify it to point to your own
 #: Trac instance if you distribute a patched version of Trac.
 default_tracker = 'https://trac.edgewall.org'
 
@@ -114,9 +113,9 @@ class RequestWithSession(Request):
 class RequestDispatcher(Component):
     """Web request dispatcher.
 
-    This component dispatches incoming requests to registered
-    handlers.  Besides, it also takes care of user authentication and
-    request pre- and post-processing.
+    This component dispatches incoming requests to registered handlers.
+    It also takes care of user authentication and request pre- and
+    post-processing.
     """
     required = True
 
@@ -136,14 +135,17 @@ class RequestDispatcher(Component):
 
         Options include `TimelineModule`, `RoadmapModule`,
         `BrowserModule`, `QueryModule`, `ReportModule`, `TicketModule`
-        and `WikiModule`.""")
+        and `WikiModule`.
+
+        The [/prefs/userinterface session preference] for default handler
+        take precedence, when set.
+        """)
 
     default_timezone = Option('trac', 'default_timezone', '',
         """The default timezone to use""")
 
     default_language = Option('trac', 'default_language', '',
-        """The preferred language to use if no user preference has
-        been set. (''since 0.12.1'')
+        """The preferred language to use if no user preference has been set.
         """)
 
     default_date_format = ChoiceOption('trac', 'default_date_format',
@@ -247,25 +249,27 @@ class RequestDispatcher(Component):
             # Process the request and render the template
             resp = chosen_handler.process_request(req)
             if resp:
-                if len(resp) == 2: # old Clearsilver template and HDF data
-                    self.log.error("Clearsilver template are no longer "
-                                   "supported (%s)", resp[0])
-                    raise TracError(
-                        _("Clearsilver templates are no longer supported, "
-                          "please contact your Trac administrator."))
-                # Genshi
-                template, data, content_type, method = \
-                    self._post_process_request(req, *resp)
+                resp = self._post_process_request(req, *resp)
+                template, data, metadata, method = resp
                 if 'hdfdump' in req.args:
                     req.perm.require('TRAC_ADMIN')
                     # debugging helper - no need to render first
                     out = io.BytesIO()
-                    pprint(data, out)
+                    pprint({'template': template,
+                            'metadata': metadata,
+                            'data': data}, out)
                     req.send(out.getvalue(), 'text/plain')
-                self.log.debug("Rendering response from handler")
-                output = chrome.render_template(
-                        req, template, data, content_type, method=method,
-                        iterable=chrome.use_chunked_encoding)
+                self.log.debug("Rendering response with template %s", template)
+                iterable = chrome.use_chunked_encoding
+                if isinstance(metadata, dict):
+                    iterable = metadata.setdefault('iterable', iterable)
+                    content_type = metadata.get('content_type')
+                else:
+                    content_type = metadata
+                output = chrome.render_template(req, template, data, metadata,
+                                                iterable=iterable,
+                                                method=method)
+                # TODO (1.5.1) remove iterable and method parameters
                 req.send(output, content_type or 'text/html')
             else:
                 self.log.debug("Empty or no response from handler. "
@@ -297,9 +301,9 @@ class RequestDispatcher(Component):
                 self.log.warning("%s caught from %s:%d in %s: %s",
                                  e.__class__.__name__, tb[0], tb[1], tb[2],
                                  to_unicode(e) or "(no message)")
-                raise HTTPInternalError(TracNotImplementedError(e))
+                raise HTTPInternalServerError(TracNotImplementedError(e))
             if isinstance(e, TracError):
-                raise HTTPInternalError(e)
+                raise HTTPInternalServerError(e)
             raise err[0], err[1], err[2]
 
     # ITemplateProvider methods
@@ -331,8 +335,8 @@ class RequestDispatcher(Component):
 
     @lazy
     def _request_handlers(self):
-        return dict((handler.__class__.__name__, handler)
-                    for handler in self.handlers)
+        return {handler.__class__.__name__: handler
+                for handler in self.handlers}
 
     def _get_valid_default_handler(self, req):
         # Use default_handler from the Session if it is a valid value.
@@ -401,7 +405,7 @@ class RequestDispatcher(Component):
         """Used to protect against CSRF.
 
         The 'form_token' is strong shared secret stored in a user
-        cookie.  By requiring that every POST form to contain this
+        cookie. By requiring that every POST form to contain this
         value we're able to protect against CSRF attacks. Since this
         value is only known by the user and not by an attacker.
 
@@ -411,12 +415,12 @@ class RequestDispatcher(Component):
         if 'trac_form_token' in req.incookie:
             return req.incookie['trac_form_token'].value
         else:
-            req.outcookie['trac_form_token'] = hex_entropy(24)
+            req.outcookie['trac_form_token'] = form_token = hex_entropy(24)
             req.outcookie['trac_form_token']['path'] = req.base_path or '/'
             if self.env.secure_cookies:
                 req.outcookie['trac_form_token']['secure'] = True
             req.outcookie['trac_form_token']['httponly'] = True
-            return req.outcookie['trac_form_token'].value
+            return form_token
 
     def _get_use_xsendfile(self, req):
         return self.use_xsendfile
@@ -427,8 +431,11 @@ class RequestDispatcher(Component):
         if Request.is_valid_header(header):
             return to_utf8(header)
         else:
-            self.log.warn("[trac] xsendfile_header is invalid: '%s'",
-                          header)
+            if not self._warn_xsendfile_header:
+                self._warn_xsendfile_header = True
+                self.log.warning("[trac] xsendfile_header is invalid: '%s'",
+                                 header)
+            return None
 
     def _get_xsendfile_header(self, req):
         return self._xsendfile_header
@@ -457,10 +464,21 @@ class RequestDispatcher(Component):
 
     def _post_process_request(self, req, *args):
         resp = args
-        # `method` is optional in IRequestHandler's response. If not
-        # specified, the default value is appended to response.
-        if len(resp) == 3:
+        # `metadata` and the backward compatibility `method` are
+        # optional in IRequestHandler's response. If not specified,
+        # the default value is appended to response.
+        metadata = {}
+        method = None
+        if len(resp) == 2:
+            resp += (metadata, None)
+        elif len(resp) == 3:
+            metadata = resp[2]
             resp += (None,)
+        elif len(resp) == 4:
+            metadata = resp[2]
+            method = resp[3]
+        if method and isinstance(metadata, dict):
+            metadata['method'] = method
         nbargs = len(resp)
         for f in reversed(self.filters):
             # As the arity of `post_process_request` has changed since
@@ -477,7 +495,7 @@ class RequestDispatcher(Component):
                 resp = f.post_process_request(req, *resp[:-1])
                 resp += (method,)
             elif nbargs == 0:
-                f.post_process_request(req, *(None,)*extra_arg_count)
+                f.post_process_request(req, *(None,) * extra_arg_count)
         return resp
 
 
@@ -496,9 +514,6 @@ def dispatch_request(environ, start_response):
         _warn_setuptools = True
         warn_setuptools_issue(out=environ.get('wsgi.errors'))
 
-    if sys.flags.optimize != 0:
-        raise EnvironmentError("Python with optimizations is not supported.")
-
     # SCRIPT_URL is an Apache var containing the URL before URL rewriting
     # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
     script_url = environ.get('SCRIPT_URL')
@@ -526,7 +541,6 @@ def dispatch_request(environ, start_response):
     environ.setdefault('trac.base_url',
                        os.getenv('TRAC_BASE_URL'))
 
-
     locale.setlocale(locale.LC_ALL, environ['trac.locale'])
 
     # Determine the environment
@@ -555,6 +569,9 @@ def dispatch_request(environ, start_response):
             script_name = environ.get('SCRIPT_NAME', '')
             try:
                 script_name = unicode(script_name, 'utf-8')
+            except UnicodeDecodeError:
+                errmsg = 'Invalid URL encoding (was %r)' % script_name
+            else:
                 # (as Href expects unicode parameters)
                 environ['SCRIPT_NAME'] = Href(script_name)(env_name)
                 environ['PATH_INFO'] = '/' + '/'.join(path_info)
@@ -566,8 +583,6 @@ def dispatch_request(environ, start_response):
 
                 if not env_path or not os.path.isdir(env_path):
                     errmsg = 'Environment not found'
-            except UnicodeDecodeError:
-                errmsg = 'Invalid URL encoding (was %r)' % script_name
 
             if errmsg:
                 start_response('404 Not Found',
@@ -586,6 +601,9 @@ def dispatch_request(environ, start_response):
     env = env_error = None
     try:
         env = open_environment(env_path, use_cache=not run_once)
+    except Exception as e:
+        env_error = e
+    else:
         if env.base_url_for_redirect:
             environ['trac.base_url'] = env.base_url
 
@@ -595,7 +613,7 @@ def dispatch_request(environ, start_response):
             if mod_wsgi_version:
                 mod_wsgi_version = (
                         "%s (WSGIProcessGroup %s WSGIApplicationGroup %s)" %
-                        ('.'.join([str(x) for x in mod_wsgi_version]),
+                        ('.'.join(str(x) for x in mod_wsgi_version),
                          environ.get('mod_wsgi.process_group'),
                          environ.get('mod_wsgi.application_group') or
                          '%{GLOBAL}'))
@@ -605,42 +623,16 @@ def dispatch_request(environ, start_response):
             env.webfrontend = environ.get('trac.web.frontend')
             if env.webfrontend:
                 env.webfrontend_version = environ['trac.web.version']
-    except Exception as e:
-        env_error = e
 
     req = RequestWithSession(environ, start_response)
-    translation.make_activable(lambda: req.locale, env.path if env else None)
-    try:
-        return _dispatch_request(req, env, env_error)
-    finally:
-        translation.deactivate()
-        if env and not run_once:
-            env.shutdown(threading._get_ident())
-            # Now it's a good time to do some clean-ups
-            #
-            # Note: enable the '##' lines as soon as there's a suspicion
-            #       of memory leak due to uncollectable objects (typically
-            #       objects with a __del__ method caught in a cycle)
-            #
-            ##gc.set_debug(gc.DEBUG_UNCOLLECTABLE)
-            unreachable = gc.collect()
-            ##env.log.debug("%d unreachable objects found.", unreachable)
-            ##uncollectable = len(gc.garbage)
-            ##if uncollectable:
-            ##    del gc.garbage[:]
-            ##    env.log.warn("%d uncollectable objects found.", uncollectable)
-
-
-def _dispatch_request(req, env, env_error):
-    resp = []
-
     # fixup env.abs_href if `[trac] base_url` was not specified
     if env and not env.abs_href.base:
         env.abs_href = req.abs_href
-
+    translation.make_activable(lambda: req.locale, env.path if env else None)
+    resp = []
     try:
-        if not env and env_error:
-            raise HTTPInternalError(env_error)
+        if env_error:
+            raise HTTPInternalServerError(env_error)
         dispatcher = RequestDispatcher(env)
         dispatcher.set_default_callbacks(req)
         try:
@@ -655,7 +647,57 @@ def _dispatch_request(req, env, env_error):
             send_internal_error(env, req, sys.exc_info())
     else:
         resp = resp or req._response or []
-    return resp
+    finally:
+        translation.deactivate()
+        if env and not run_once:
+            env.shutdown(get_thread_id())
+            # Now it's a good time to do some clean-ups
+            #
+            # Note: enable the '##' lines as soon as there's a suspicion
+            #       of memory leak due to uncollectable objects (typically
+            #       objects with a __del__ method caught in a cycle)
+            #
+            ##gc.set_debug(gc.DEBUG_UNCOLLECTABLE)
+            unreachable = gc.collect()
+            ##env.log.debug("%d unreachable objects found.", unreachable)
+            ##uncollectable = len(gc.garbage)
+            ##if uncollectable:
+            ##    del gc.garbage[:]
+            ##    env.log.warning("%d uncollectable objects found.",
+            ##                    uncollectable)
+        return resp
+
+
+def _send_error(req, exc_info, template='error.html', content_type='text/html',
+                status=500, env=None, data={}):
+    if env:
+        add_stylesheet(req, 'common/css/code.css')
+        metadata = {'content_type': 'text/html', 'iterable': False}
+        try:
+            content = Chrome(env).render_template(req, template,
+                                                  data, metadata)
+        except Exception:
+            # second chance rendering, in "safe" mode
+            data['trac_error_rendering'] = True
+            try:
+                content = Chrome(env).render_template(req, template,
+                                                      data, metadata)
+            except Exception:
+                content = get_last_traceback()
+                content_type = 'text/plain'
+    else:
+        content_type = 'text/plain'
+        content = '%s\n\n%s: %s' % (data.get('title'),
+                                    data.get('type'),
+                                    data.get('message'))
+
+    if isinstance(content, unicode):
+        content = content.encode('utf-8')
+
+    try:
+        req.send_error(exc_info, content, content_type, status)
+    except RequestDone:
+        pass
 
 
 def _send_user_error(req, env, e):
@@ -666,15 +708,12 @@ def _send_user_error(req, env, e):
                         req, req.environ.get('HTTP_REFERER'))
     data = {'title': e.title, 'type': 'TracError', 'message': e.message,
             'frames': [], 'traceback': None}
-    if e.code == 403 and req.authname == 'anonymous':
+    if e.code == 403 and not req.is_authenticated:
         # TRANSLATOR: ... not logged in, you may want to 'do so' now (link)
         do_so = tag.a(_("do so"), href=req.href.login())
         add_notice(req, tag_("You are currently not logged in. You may want "
                              "to %(do_so)s now.", do_so=do_so))
-    try:
-        req.send_error(sys.exc_info(), status=e.code, env=env, data=data)
-    except RequestDone:
-        pass
+    _send_error(req, sys.exc_info(), status=e.code, env=env, data=data)
 
 
 def send_internal_error(env, req, exc_info):
@@ -687,11 +726,10 @@ def send_internal_error(env, req, exc_info):
 
     frames, plugins, faulty_plugins, interface_custom = [], [], [], []
     th = 'http://trac-hacks.org'
-    has_admin = False
     try:
         has_admin = 'TRAC_ADMIN' in req.perm
     except Exception:
-        pass
+        has_admin = False
 
     tracker = default_tracker
     tracker_args = {}
@@ -718,13 +756,14 @@ def send_internal_error(env, req, exc_info):
                     plugin_name = info.get('home_page', '').rstrip('/') \
                                                            .split('/')[-1]
                     tracker_args = {'component': plugin_name}
-        interface_custom = Chrome(env).get_interface_customization_files()
+            interface_custom = Chrome(env).get_interface_customization_files()
 
     def get_description(_):
         if env and has_admin:
             sys_info = "".join("|| '''`%s`''' || `%s` ||\n"
-                               % (k, v.replace('\n', '` [[br]] `'))
-                               for k, v in env.get_systeminfo())
+                               % (k, (v.replace('\n', '` [[br]] `') if v
+                                      else _('N/A')))
+                               for k, v in env.system_info)
             sys_info += "|| '''`jQuery`''' || `#JQUERY#` ||\n" \
                         "|| '''`jQuery UI`''' || `#JQUERYUI#` ||\n" \
                         "|| '''`jQuery Timepicker`''' || `#JQUERYTP#` ||\n"
@@ -741,7 +780,7 @@ def send_internal_error(env, req, exc_info):
             interface_files = _("''Interface customization information not "
                                  "available''\n")
         return _("""\
-==== How to Reproduce ====
+==== How to Reproduce
 
 While doing a %(method)s operation on `%(path_info)s`, \
 Trac issued an internal error.
@@ -755,13 +794,13 @@ Request parameters:
 
 User agent: `#USER_AGENT#`
 
-==== System Information ====
+==== System Information
 %(sys_info)s
-==== Enabled Plugins ====
+==== Enabled Plugins
 %(enabled_plugins)s
-==== Interface Customization ====
+==== Interface Customization
 %(interface_customization)s
-==== Python Traceback ====
+==== Python Traceback
 {{{
 %(traceback)s}}}""",
             method=req.method, path_info=req.path_info,
@@ -786,11 +825,9 @@ User agent: `#USER_AGENT#`
             'tracker': tracker, 'tracker_args': tracker_args,
             'description': description, 'description_en': description_en}
 
-    Chrome(env).add_jquery_ui(req)
-    try:
-        req.send_error(exc_info, status=500, env=env, data=data)
-    except RequestDone:
-        pass
+    if env:
+        Chrome(env).add_jquery_ui(req)
+    _send_error(req, sys.exc_info(), status=500, env=env, data=data)
 
 
 def send_project_index(environ, start_response, parent_dir=None,
@@ -812,38 +849,34 @@ def send_project_index(environ, start_response, parent_dir=None,
         for pair in req.environ['trac.template_vars'].split(','):
             key, val = pair.split('=')
             data[key] = val
-    try:
-        href = Href(req.base_path)
-        projects = []
-        for env_name, env_path in get_environments(environ).items():
-            try:
-                env = open_environment(env_path,
-                                       use_cache=not environ['wsgi.run_once'])
-                proj = {
-                    'env': env,
-                    'name': env.project_name,
-                    'description': env.project_description,
-                    'href': href(env_name)
-                }
-            except Exception as e:
-                proj = {'name': env_name, 'description': to_unicode(e)}
-            projects.append(proj)
-        projects.sort(lambda x, y: cmp(x['name'].lower(), y['name'].lower()))
 
-        data['projects'] = projects
-
-        loader = TemplateLoader(loadpaths, variable_lookup='lenient',
-                                default_encoding='utf-8')
-        tmpl = loader.load(template)
-        stream = tmpl.generate(**data)
-        if template.endswith('.xml'):
-            output = stream.render('xml')
-            req.send(output, 'text/xml')
+    href = Href(req.base_path)
+    projects = []
+    for env_name, env_path in get_environments(environ).items():
+        try:
+            env = open_environment(env_path,
+                                   use_cache=not environ['wsgi.run_once'])
+        except Exception as e:
+            proj = {'name': env_name, 'description': to_unicode(e)}
         else:
-            output = stream.render('xhtml', doctype=DocType.XHTML_STRICT,
-                                   encoding='utf-8')
-            req.send(output, 'text/html')
+            proj = {
+                'env': env,
+                'name': env.project_name,
+                'description': env.project_description,
+                'href': href(env_name)
+            }
+        projects.append(proj)
+    projects.sort(key=lambda proj: proj['name'].lower())
 
+    data['projects'] = projects
+
+    jenv = jinja2env(loader=FileSystemLoader(loadpaths))
+    jenv.globals.update(translation.functions)
+    tmpl = jenv.get_template(template)
+    output = valid_html_bytes(tmpl.render(**data).encode('utf-8'))
+    content_type = 'text/xml' if template.endswith('.xml') else 'text/html'
+    try:
+        req.send(output, content_type)
     except RequestDone:
         pass
 
@@ -870,14 +903,12 @@ def get_environments(environ, warn=False):
     env_parent_dir = environ.get('trac.env_parent_dir')
     if env_parent_dir:
         env_parent_dir = os.path.normpath(env_parent_dir)
-        paths = dircache.listdir(env_parent_dir)[:]
-        dircache.annotate(env_parent_dir, paths)
-
         # Filter paths that match the .tracignore patterns
         ignore_patterns = get_tracignore_patterns(env_parent_dir)
-        paths = [path[:-1] for path in paths if path[-1] == '/'
-                 and not any(fnmatch.fnmatch(path[:-1], pattern)
-                             for pattern in ignore_patterns)]
+        paths = [name for name in os.listdir(env_parent_dir)
+                      if os.path.isdir(os.path.join(env_parent_dir, name)) and
+                      not any(fnmatch.fnmatch(name, pattern)
+                              for pattern in ignore_patterns)]
         env_paths.extend(os.path.join(env_parent_dir, project)
                          for project in paths)
     envs = {}
