@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,12 +15,11 @@
 import os
 import unittest
 
-from genshi.builder import tag
-
 from trac.mimeview.rst import has_docutils
 from trac.tests.contentgen import random_sentence, random_unique_camel
 from trac.tests.functional import FunctionalTwillTestCaseSetup, tc
 from trac.util import create_file, get_pkginfo
+from trac.util.html import tag
 
 
 class TestWiki(FunctionalTwillTestCaseSetup):
@@ -42,26 +41,45 @@ class TestWikiEdit(FunctionalTwillTestCaseSetup):
 class TestWikiDelete(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Delete a wiki page."""
+        # Delete page with single version.
         name = self._tester.create_wiki_page()
         self._tester.go_to_wiki(name)
         tc.formvalue('delete', 'action', 'delete')
-        tc.submit()
+        tc.submit('delete_page')
+        tc.find("Are you sure you want to completely delete this page?")
         tc.notfind("The following attachments will also be deleted:")
         tc.submit('delete', 'delete-confirm')
         tc.find("The page %s has been deleted." % name)
         tc.url(self._tester.url)
 
+        # Delete page with attachment.
         name = self._tester.create_wiki_page()
         filename = self._tester.attach_file_to_wiki(name)
         self._tester.go_to_wiki(name)
         tc.formvalue('delete', 'action', 'delete')
-        tc.submit()
+        tc.submit('delete_page')
+        tc.find("Are you sure you want to completely delete this page?")
         tc.find("The following attachments will also be deleted:")
         tc.find(filename)
         tc.submit('delete', 'delete-confirm')
         tc.find("The page %s has been deleted." % name)
         tc.url(self._tester.url)
 
+        # Delete page with multiple versions.
+        name = self._tester.create_wiki_page(content="Initial content.")
+        self._tester.edit_wiki_page(name, content="Revised content.")
+        self._tester.go_to_wiki(name)
+        tc.formvalue('delete', 'action', 'delete')
+        tc.submit('delete_page')
+        tc.find("Are you sure you want to completely delete this page?")
+        tc.find(r'Removing all\s+<a href="/wiki/%s\?action=history&amp;'
+                r'version=2">2 versions</a>\s+of the page' % name)
+        tc.notfind("The following attachments will also be deleted:")
+        tc.submit('delete', 'delete-confirm')
+        tc.find("The page %s has been deleted." % name)
+        tc.url(self._tester.url)
+
+
 class TestWikiAddAttachment(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Add attachment to a wiki page. Test that the attachment
@@ -75,15 +93,15 @@ class TestWikiAddAttachment(FunctionalTwillTestCaseSetup):
 
         self._tester.go_to_wiki(name)
         tc.find("Attach another file")
-        tc.find('Attachments <span class="trac-count">\(1\)</span>')
+        tc.find('Attachments[ \n]+<span class="trac-count">\(1\)</span>')
         tc.find(filename)
         tc.find('Download all attachments as:\s+<a rel="nofollow" '
                 'href="/zip-attachment/wiki/%s/">.zip</a>' % name)
 
 
 _plugin_py = """\
-from genshi.builder import tag
 from trac.core import Component, implements
+from trac.util.html import tag
 from trac.util.translation import tag_
 from trac.wiki.api import IWikiPageManipulator
 
@@ -134,37 +152,75 @@ class TestWikiHistory(FunctionalTwillTestCaseSetup):
     def runTest(self):
         pagename = self._tester.create_wiki_page()
         self._tester.edit_wiki_page(pagename)
+        url = self._tester.url
         tc.follow(r"\bHistory\b")
-        tc.url(self._tester.url + r'/wiki/%s\?action=history' % pagename)
-        version_link = '<td class="version">[ \t\n]*' \
-                       '<a href="/wiki/%(pagename)s\?version=%%(version)s" ' \
-                       'title="View this version">%%(version)s[ \t\n]*</a>' \
-                       % {'pagename': pagename}
+        tc.url(url + r'/wiki/%s\?action=history' % pagename)
+        version_link = ('<td class="version">[ \n]*'
+                        '<a href="/wiki/%(pagename)s\?version=%%(version)s"'
+                        '[ \n]*title="View this version">%%(version)s[ \n]*</a>'
+                        % {'pagename': pagename})
         tc.find(version_link % {'version': 1})
         tc.find(version_link % {'version': 2})
+        tc.find(r'<th class="comment">Comment</th>')
         tc.formvalue('history', 'old_version', '1')
         tc.formvalue('history', 'version', '2')
         tc.submit()
         tc.url(r'%s/wiki/%s\?action=diff&version=2&old_version=1'
-               % (self._tester.url, pagename))
+               % (url, pagename))
         tc.find(r'<a href="/wiki/%s\?version=1">Version 1</a>' % pagename)
         tc.find(r'<a href="/wiki/%s\?version=2">Version 2</a>' % pagename)
         tc.find(r'<a href="/wiki/%(name)s">%(name)s</a>' % {'name': pagename})
 
 
+class TestWikiEditComment(FunctionalTwillTestCaseSetup):
+    """Edit wiki page comment from diff and history."""
+    def runTest(self):
+        initial_comment = "Initial comment"
+        pagename = self._tester.create_wiki_page(comment=initial_comment)
+        url = self._tester.url
+        tc.follow(r"\bHistory\b")
+        history_url = url + r'/wiki/%s\?action=history' % pagename
+        tc.url(history_url)
+
+        # Comment edit from history page
+        tc.follow(r"\bEdit\b")
+        tc.url(url + r'/wiki/%s\?action=edit_comment&version=1' % pagename)
+        tc.find("Old comment:[ \t\n]+%s" % initial_comment)
+        first_comment_edit = "First comment edit"
+        tc.formvalue('edit-comment-form', 'new_comment', first_comment_edit)
+        tc.submit()
+        tc.url(history_url)
+        tc.find(r'<td class="comment">[ \t\n]+%s' % first_comment_edit)
+
+        # Comment edit from diff page
+        tc.formvalue('history', 'version', '1')
+        tc.submit()
+        diff_url = url + r'/wiki/%s\?action=diff&version=1' % pagename
+        tc.url(diff_url)
+        tc.find(r'<p>[ \t\n]+%s[ \t\n]+</p>' % first_comment_edit)
+        tc.follow(r"\bEdit\b")
+        tc.url(url + r'/wiki/%s\?action=edit_comment&version=1&redirect_to=diff'
+               % pagename)
+        second_comment_edit = "Second comment edit"
+        tc.formvalue('edit-comment-form', 'new_comment', second_comment_edit)
+        tc.submit()
+        tc.url(diff_url)
+        tc.find(r'<p>[ \t\n]+%s[ \t\n]+</p>' % second_comment_edit)
+
+
 class TestWikiReadonlyAttribute(FunctionalTwillTestCaseSetup):
     """Test the wiki readonly attribute, which is enforce when
-    ReadonlyWikiPolicy is in the list of active permission policies."""
+    DefaultWikiPolicy is in the list of active permission policies."""
     def runTest(self):
         self._tester.logout()
         self._tester.login('user')
         page_name = self._tester.create_wiki_page()
         permission_policies = \
             self._testenv.get_config('trac', 'permission_policies')
-        readonly_checkbox = '<input type="checkbox" name="readonly" ' + \
-                            'id="readonly" />'
-        attach_button = '<input type="submit" id="attachfilebutton" ' + \
-                        'value="Attach.+file" />'
+        readonly_checkbox = (
+            '<input type="checkbox" name="readonly" id="readonly"/>')
+        attach_button = (
+            '<input type="submit" id="attachfilebutton" value="Attach.+file"/>')
         try:
             # User without WIKI_ADMIN can't set a page read-only
             tc.formvalue('modifypage', 'action', 'edit')
@@ -191,11 +247,11 @@ class TestWikiReadonlyAttribute(FunctionalTwillTestCaseSetup):
             tc.go(self._tester.url + '/attachment/wiki/' + page_name)
             tc.notfind(attach_button)
 
-            # Read-only checkbox is not present when ReadonlyWikiPolicy
+            # Read-only checkbox is not present when DefaultWikiPolicy
             # is not in the list of active permission policies
             pp_list = [p.strip() for p in permission_policies.split(',')]
-            pp_list.remove('ReadonlyWikiPolicy')
-            self._testenv.set_config('trac', 'permission_policies',
+            pp_list.remove('DefaultWikiPolicy')
+            self._testenv._tracadmin('trac', 'permission_policies',
                                      ', '.join(pp_list))
             self._testenv.grant_perm('user', 'WIKI_ADMIN')
             self._tester.go_to_wiki(page_name)
@@ -279,7 +335,7 @@ class TestWikiRename(FunctionalTwillTestCaseSetup):
         # this time, the original page is gone
         tc.go(page_url)
         tc.url(page_url)
-        tc.find("The page %s does not exist" % tag.strong(pagename))
+        tc.find("The page[ \n]+%s[ \n]+does not exist" % tag.strong(pagename))
 
 
 class RegressionTestTicket4812(FunctionalTwillTestCaseSetup):
@@ -325,7 +381,7 @@ class ReStructuredTextCodeBlockTest(FunctionalTwillTestCaseSetup):
         self._tester.go_to_wiki(pagename)
         tc.notfind("code-block")
         tc.find('print')
-        tc.find('"123"')
+        tc.find('&quot;123&quot;')
 
 
 class RegressionTestTicket8976(FunctionalTwillTestCaseSetup):
@@ -380,11 +436,11 @@ class RegressionTestTicket10274(FunctionalTwillTestCaseSetup):
     def runTest(self):
         """Test for regression of https://trac.edgewall.org/ticket/10274"""
         self._tester.go_to_wiki('WikiStart/..')
-        tc.find("Invalid Wiki page name 'WikiStart/..'")
+        tc.find("Invalid Wiki page name &#39;WikiStart/..&#39;")
         self._tester.go_to_wiki('../WikiStart')
-        tc.find("Invalid Wiki page name '../WikiStart'")
+        tc.find("Invalid Wiki page name &#39;../WikiStart&#39;")
         self._tester.go_to_wiki('WikiStart/./SubPage')
-        tc.find("Invalid Wiki page name 'WikiStart/./SubPage'")
+        tc.find("Invalid Wiki page name &#39;WikiStart/./SubPage&#39;")
 
 
 class RegressionTestTicket10850(FunctionalTwillTestCaseSetup):
@@ -435,7 +491,7 @@ class RegressionTestTicket10957(FunctionalTwillTestCaseSetup):
             tc.go(self._tester.url + '/wiki/%s?action=edit' % page_name)
             tc.find("Error: Forbidden")
             tc.find("WIKI_CREATE privileges are required to perform this "
-                    "operation on %s. You don't have the required permissions."
+                    "operation on %s. You don&#39;t have the required permissions."
                     % page_name)
 
             # Check that page can be created when user has WIKI_CREATE
@@ -450,7 +506,7 @@ class RegressionTestTicket10957(FunctionalTwillTestCaseSetup):
             tc.go(self._tester.url + '/wiki/%s?action=edit' % page_name)
             tc.find("Error: Forbidden")
             tc.find("WIKI_MODIFY privileges are required to perform this "
-                    "operation on %s. You don't have the required permissions."
+                    "operation on %s. You don&#39;t have the required permissions."
                     % page_name)
 
             # Check that page can be edited when user has WIKI_MODIFY
@@ -476,7 +532,7 @@ class RegressionTestTicket10957(FunctionalTwillTestCaseSetup):
             tc.notfind("Revert to this version")
             tc.go(self._tester.url + '/wiki/%s?action=edit&version=1' % page_name)
             tc.find("WIKI_MODIFY privileges are required to perform this "
-                    "operation on %s. You don't have the required permissions."
+                    "operation on %s. You don&#39;t have the required permissions."
                     % page_name)
 
         finally:
@@ -496,19 +552,6 @@ class RegressionTestTicket11302(FunctionalTwillTestCaseSetup):
                 r' href="/wiki/%s\?version=1#point1">@1</a>' % pagename)
 
 
-class RegressionTestTicket11518(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        """Test for regression of https://trac.edgewall.org/ticket/11518
-        ResourceNotFound should be raised when version is invalid.
-        """
-        tc.go(self._tester.url + '/wiki/WikiStart?version=1abc')
-        tc.find(r"<h1>Error: Bad Request</h1>")
-        tc.find(r"Invalid value for request argument <em>version</em>.")
-        tc.go(self._tester.url + '/wiki/WikiStart?version=')
-        tc.find(r"<h1>Error: Bad Request</h1>")
-        tc.find(r"Invalid value for request argument <em>version</em>.")
-
-
 def functionalSuite(suite=None):
     if not suite:
         import trac.tests.functional
@@ -519,6 +562,7 @@ def functionalSuite(suite=None):
     suite.addTest(TestWikiAddAttachment())
     suite.addTest(TestWikiPageManipulator())
     suite.addTest(TestWikiHistory())
+    suite.addTest(TestWikiEditComment())
     suite.addTest(TestWikiReadonlyAttribute())
     suite.addTest(TestWikiRename())
     suite.addTest(RegressionTestTicket4812())
@@ -527,7 +571,6 @@ def functionalSuite(suite=None):
     suite.addTest(RegressionTestTicket10850())
     suite.addTest(RegressionTestTicket10957())
     suite.addTest(RegressionTestTicket11302())
-    suite.addTest(RegressionTestTicket11518())
     if has_docutils:
         import docutils
         if get_pkginfo(docutils):
