@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2020 Edgewall Software
+# Copyright (C) 2009-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -38,8 +38,6 @@
 import re
 import textwrap
 
-from genshi.builder import tag
-
 from trac.config import BoolOption, Option
 from trac.core import Component, implements
 from trac.notification.api import NotificationSystem
@@ -48,6 +46,7 @@ from trac.resource import Resource
 from trac.ticket import Ticket
 from trac.ticket.notification import TicketChangeEvent
 from trac.util.datefmt import datetime_now, utc
+from trac.util.html import tag
 from trac.util.text import exception_to_unicode
 from trac.util.translation import _, cleandoc_
 from trac.versioncontrol import IRepositoryChangeListener, RepositoryManager
@@ -210,8 +209,7 @@ class CommitTicketUpdater(Component):
             drev += '/' + repos.reponame
         return textwrap.dedent("""\
             In [changeset:"%s" %s]:
-            {{{
-            #!CommitTicketReference repository="%s" revision="%s"
+            {{{#!CommitTicketReference repository="%s" revision="%s"
             %s
             }}}""") % (revstring, drev, repos.reponame, rev,
                        changeset.message.strip())
@@ -221,9 +219,9 @@ class CommitTicketUpdater(Component):
         authname = self._authname(changeset)
         perm = PermissionCache(self.env, authname)
         for tkt_id, cmds in tickets.iteritems():
+            self.log.debug("Updating ticket #%d", tkt_id)
+            save = False
             try:
-                self.log.debug("Updating ticket #%d", tkt_id)
-                save = False
                 with self.env.db_transaction:
                     ticket = Ticket(self.env, tkt_id)
                     ticket_perm = perm(ticket.resource)
@@ -273,7 +271,7 @@ class CommitTicketUpdater(Component):
 
     def cmd_close(self, ticket, changeset, perm):
         authname = self._authname(changeset)
-        if self.check_perms and not 'TICKET_MODIFY' in perm:
+        if self.check_perms and 'TICKET_MODIFY' not in perm:
             self.log.info("%s doesn't have TICKET_MODIFY permission for #%d",
                           authname, ticket.id)
             return False
@@ -283,7 +281,7 @@ class CommitTicketUpdater(Component):
             ticket['owner'] = authname
 
     def cmd_refs(self, ticket, changeset, perm):
-        if self.check_perms and not 'TICKET_APPEND' in perm:
+        if self.check_perms and 'TICKET_APPEND' not in perm:
             self.log.info("%s doesn't have TICKET_APPEND permission for #%d",
                           self._authname(changeset), ticket.id)
             return False
@@ -312,18 +310,19 @@ class CommitTicketReferenceMacro(WikiMacroBase):
         repos = RepositoryManager(self.env).get_repository(reponame)
         try:
             changeset = repos.get_changeset(rev)
-            message = changeset.message
-            rev = changeset.rev
-            resource = repos.resource
         except Exception:
             message = content
             resource = Resource('repository', reponame)
+        else:
+            message = changeset.message
+            rev = changeset.rev
+            resource = repos.resource
         if formatter.context.resource.realm == 'ticket':
             ticket_re = CommitTicketUpdater.ticket_re
             if not any(int(tkt_id) == int(formatter.context.resource.id)
                        for tkt_id in ticket_re.findall(message)):
-                return tag.p(_("(The changeset message doesn't reference this "
-                               "ticket)"), class_='hint')
+                return tag.p(_("(The changeset message doesn't reference "
+                               "this ticket)"), class_='hint')
         if ChangesetModule(self.env).wiki_format_messages:
             return tag.div(format_to_html(self.env,
                 formatter.context.child('changeset', rev, parent=resource),
