@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2020 Edgewall Software
+# Copyright (C) 2013-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,7 +16,7 @@ from datetime import datetime
 
 from trac.test import EnvironmentStub, Mock
 from trac.tests.contentgen import random_sentence
-from trac.ticket.model import Ticket
+from trac.ticket.test import insert_ticket
 from trac.util.datefmt import utc
 from trac.versioncontrol.api import Repository, RepositoryManager
 from trac.wiki.tests import formatter
@@ -38,11 +38,9 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
 
     def _make_tickets(self, num):
         self.tickets = []
-        for i in xrange(0, num):
-            ticket = Ticket(self.env)
-            ticket['reporter'] = 'someone'
-            ticket['summary'] = random_sentence()
-            ticket.insert()
+        for i in xrange(num):
+            ticket = insert_ticket(self.env, reporter='someone',
+                                   summary=random_sentence())
             self.tickets.append(ticket)
 
     def test_changeset_added(self):
@@ -54,8 +52,7 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
         changes = self.tickets[0].get_change(cnum=1)
         self.assertEqual(textwrap.dedent("""\
             In [changeset:"1/repos1" 1/repos1]:
-            {{{
-            #!CommitTicketReference repository="repos1" revision="1"
+            {{{#!CommitTicketReference repository="repos1" revision="1"
             This is the first comment. Refs #1.
             }}}"""), changes['fields']['comment']['new'])
 
@@ -70,8 +67,7 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
         changes = self.tickets[0].get_change(cnum=1)
         self.assertEqual(textwrap.dedent("""\
             In [changeset:"1/repos1" 1/repos1]:
-            {{{
-            #!CommitTicketReference repository="repos1" revision="1"
+            {{{#!CommitTicketReference repository="repos1" revision="1"
             This is a multiline comment.
 
             It is multiline.
@@ -94,15 +90,13 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
         changes = self.tickets[0].get_change(cnum=1)
         self.assertEqual(textwrap.dedent("""\
             In [changeset:"1/repos1" 1/repos1]:
-            {{{
-            #!CommitTicketReference repository="repos1" revision="1"
+            {{{#!CommitTicketReference repository="repos1" revision="1"
             This is the first comment. Refs #1.
             }}}"""), changes['fields']['comment']['new'])
         changes = self.tickets[1].get_change(cnum=1)
         self.assertEqual(textwrap.dedent("""\
             In [changeset:"1/repos1" 1/repos1]:
-            {{{
-            #!CommitTicketReference repository="repos1" revision="1"
+            {{{#!CommitTicketReference repository="repos1" revision="1"
             This is the first comment after an edit. Refs #1, #2.
             }}}"""), changes['fields']['comment']['new'])
 
@@ -132,10 +126,7 @@ class CommitTicketUpdaterTestCase(unittest.TestCase):
 def macro_setup(tc):
     tc.env = EnvironmentStub(enable=('trac.*',
                                      'tracopt.ticket.commit_updater.*',))
-    ticket = Ticket(tc.env)
-    ticket['summary'] = 'the summary'
-    ticket['status'] = 'new'
-    ticket.insert()
+    insert_ticket(tc.env, summary='the summary', status='new')
     def _get_repository(reponame):
         return Mock(get_changeset=_get_changeset, resource=None)
     def _get_changeset(rev=None):
