@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2020 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,8 +16,7 @@ import tempfile
 import unittest
 from datetime import datetime
 
-import trac.tests.compat
-from trac.test import EnvironmentStub, rmtree
+from trac.test import EnvironmentStub, mkdtemp, rmtree
 from trac.util import create_file
 from trac.versioncontrol.api import Changeset, DbRepositoryProvider, \
                                     RepositoryManager
@@ -152,7 +151,7 @@ class NormalTestCase(unittest.TestCase, GitCommandMixin):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.repos_path = tempfile.mkdtemp(prefix='trac-gitrepos-')
+        self.repos_path = mkdtemp()
         # create git repository and master branch
         self._git('init')
         self._git('config', 'core.quotepath', 'true')  # ticket:11198
@@ -213,7 +212,7 @@ class NormalTestCase(unittest.TestCase, GitCommandMixin):
         # regression test for #11215
         path = os.path.join(self.repos_path, '.git')
         DbRepositoryProvider(self.env).add_repository('gitrepos', path, 'git')
-        repos = self.env.get_repository('gitrepos')
+        repos = RepositoryManager(self.env).get_repository('gitrepos')
         parent_rev = repos.youngest_rev
 
         create_file(os.path.join(self.repos_path, 'ticket11215.txt'))
@@ -236,7 +235,7 @@ class NormalTestCase(unittest.TestCase, GitCommandMixin):
         # regression test for #11328
         path = os.path.join(self.repos_path, '.git')
         DbRepositoryProvider(self.env).add_repository('gitrepos', path, 'git')
-        repos = self.env.get_repository('gitrepos')
+        repos = RepositoryManager(self.env).get_repository('gitrepos')
         parent_rev = repos.youngest_rev
 
         self._git_commit('-m', 'ticket:11328', '--allow-empty',
@@ -268,10 +267,10 @@ class NormalTestCase(unittest.TestCase, GitCommandMixin):
         self.assertEqual(['b1', 'master'],
                          sorted(b[0] for b in storage.get_branches()))
         self._git('branch', '-D', 'b1')
-        self.assertEqual(True, storage.sync())
+        self.assertTrue(storage.sync())
         self.assertEqual(['master'],
                          sorted(b[0] for b in storage.get_branches()))
-        self.assertEqual(False, storage.sync())
+        self.assertFalse(storage.sync())
 
     def test_turn_off_persistent_cache(self):
         # persistent_cache is enabled
@@ -291,7 +290,7 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.repos_path = tempfile.mkdtemp(prefix='trac-gitrepos-')
+        self.repos_path = mkdtemp()
         # create git repository and master branch
         self._git('init')
         self._git('config', 'core.quotepath', 'true')  # ticket:11198
@@ -313,7 +312,7 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 
     def test_unicode_verifyrev(self):
         storage = self._storage()
-        self.assertNotEqual(None, storage.verifyrev(u'master'))
+        self.assertIsNotNone(storage.verifyrev(u'master'))
         self.assertIsNone(storage.verifyrev(u'tété'))
 
     def test_unicode_filename(self):
@@ -358,11 +357,11 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
         self.assertEqual([u'tɐg-t10980', 'v0.42.1'], tags)
 
         rev = storage.verifyrev(u'tɐg-t10980')
-        self.assertNotEqual(None, rev)
+        self.assertIsNotNone(rev)
         self.assertEqual([u'tɐg-t10980'], storage.get_tags(rev))
 
         rev = storage.verifyrev('v0.42.1')
-        self.assertNotEqual(None, rev)
+        self.assertIsNotNone(rev)
         self.assertEqual(['v0.42.1'], storage.get_tags(rev))
 
     def test_ls_tree(self):
@@ -431,7 +430,7 @@ class SizedDictTestCase(unittest.TestCase):
 #        import timeit
 #
 #        g = Storage(path_to_repo, logging) # Need a git repository path here
-#        revs = g.get_commits().keys()
+#        revs = list(g.get_commits())
 #
 #        def shortrev_test():
 #            for i in revs:
@@ -518,13 +517,13 @@ class SizedDictTestCase(unittest.TestCase):
 #        print("--------------")
 #
 #        p = g.head()
-#        for i in range(-5, 5):
+#        for i in xrange(-5, 5):
 #            print(i, g.history_relative_rev(p, i))
 #
 #        # check for loops
 #        def check4loops(head):
 #            print("check4loops", head)
-#            seen = set([head])
+#            seen = {head}
 #            for _sha in g.children_recursive(head):
 #                if _sha in seen:
 #                    print("dupe detected :-/", _sha, len(seen))
@@ -534,9 +533,9 @@ class SizedDictTestCase(unittest.TestCase):
 #        print(len(check4loops(g.parents(g.head())[0])))
 #
 #        #p = g.head()
-#        #revs = [ g.history_relative_rev(p, i) for i in range(0,10) ]
+#        #revs = [g.history_relative_rev(p, i) for i in xrange(10)]
 #        print_data_usage()
-#        revs = g.get_commits().keys()
+#        revs = list(g.get_commits())
 #        print_data_usage()
 #
 #        #print(len(check4loops(g.oldest_rev())))
