@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2020 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,19 +11,17 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os
 import sys
-import tempfile
 import unittest
-from cStringIO import StringIO
 from datetime import datetime, timedelta
-from subprocess import Popen, PIPE
+from subprocess import PIPE
 
-import trac.tests.compat
 from trac.core import TracError
-from trac.test import EnvironmentStub, MockRequest, locate, rmtree
+from trac.test import EnvironmentStub, MockRequest, locate, mkdtemp, rmtree
 from trac.util import create_file
-from trac.util.compat import close_fds
+from trac.util.compat import Popen, close_fds
 from trac.util.datefmt import to_timestamp, utc
 from trac.util.text import to_utf8
 from trac.versioncontrol.api import Changeset, DbRepositoryProvider, \
@@ -59,9 +57,8 @@ class GitCommandMixin(object):
         return Popen(args, close_fds=close_fds, **kwargs)
 
     def _git(self, *args, **kwargs):
-        proc = self._spawn_git(*args, **kwargs)
-        stdout, stderr = proc.communicate()
-        self._close_proc_pipes(proc)
+        with self._spawn_git(*args, **kwargs) as proc:
+            stdout, stderr = proc.communicate()
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, args %r, kwargs %r, stdout %r, '
                          'stderr %r' %
@@ -71,9 +68,8 @@ class GitCommandMixin(object):
     def _git_fast_import(self, data, **kwargs):
         if isinstance(data, unicode):
             data = data.encode('utf-8')
-        proc = self._spawn_git('fast-import', stdin=PIPE, **kwargs)
-        stdout, stderr = proc.communicate(input=data)
-        self._close_proc_pipes(proc)
+        with self._spawn_git('fast-import', stdin=PIPE, **kwargs) as proc:
+            stdout, stderr = proc.communicate(input=data)
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, stdout %r, stderr %r' %
                          (proc.returncode, stdout, stderr))
@@ -95,17 +91,12 @@ class GitCommandMixin(object):
         env['GIT_COMMITTER_DATE'] = dt
         env['GIT_AUTHOR_DATE'] = dt
 
-    def _close_proc_pipes(self, proc):
-        for f in (proc.stdin, proc.stdout, proc.stderr):
-            if f:
-                f.close()
-
 
 class BaseTestCase(unittest.TestCase, GitCommandMixin):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.tmpdir = tempfile.mkdtemp(prefix='trac-tempdir-')
+        self.tmpdir = mkdtemp()
         self.repos_path = os.path.join(self.tmpdir, 'gitrepos')
         os.mkdir(self.repos_path)
         if self.git_bin:
@@ -270,7 +261,7 @@ class GitNormalTestCase(BaseTestCase):
         self._add_repository()
         repos = self._repomgr.get_repository('gitrepos')
         rev = repos.youngest_rev
-        self.assertNotEqual(None, rev)
+        self.assertIsNotNone(rev)
         self.assertEqual(40, len(rev))
 
         self.assertEqual(rev, repos.get_node('/').rev)
@@ -315,14 +306,14 @@ class GitNormalTestCase(BaseTestCase):
         else:
             repos.sync()
         youngest_rev = repos.youngest_rev
-        self.assertEqual(None, youngest_rev)
-        self.assertEqual(None, repos.oldest_rev)
-        self.assertEqual(None, repos.normalize_rev(''))
-        self.assertEqual(None, repos.normalize_rev(None))
-        self.assertEqual(None, repos.display_rev(''))
-        self.assertEqual(None, repos.display_rev(None))
-        self.assertEqual(None, repos.short_rev(''))
-        self.assertEqual(None, repos.short_rev(None))
+        self.assertIsNone(youngest_rev)
+        self.assertIsNone(repos.oldest_rev)
+        self.assertIsNone(repos.normalize_rev(''))
+        self.assertIsNone(repos.normalize_rev(None))
+        self.assertIsNone(repos.display_rev(''))
+        self.assertIsNone(repos.display_rev(None))
+        self.assertIsNone(repos.short_rev(''))
+        self.assertIsNone(repos.short_rev(None))
 
         node = repos.get_node('/', youngest_rev)
         self.assertEqual([], list(node.get_entries()))
@@ -334,7 +325,7 @@ class GitNormalTestCase(BaseTestCase):
         self.assertTrue(browser_mod.match_request(req))
         rv = browser_mod.process_request(req)
         self.assertEqual('browser.html', rv[0])
-        self.assertEqual(None, rv[1]['rev'])
+        self.assertIsNone(rv[1]['rev'])
 
         req = MockRequest(self.env, path_info='/log/gitrepos')
         log_mod = LogModule(self.env)
@@ -374,12 +365,10 @@ class GitRepositoryTestCase(BaseTestCase):
     def test_invalid_path_raises(self):
         def try_init(reponame):
             params = {'name': reponame}
-            try:
+            with self.assertRaises(InvalidRepository) as cm:
                 GitRepository(self.env, '/the/invalid/path', params,
                               self.env.log)
-                self.fail('InvalidRepository not raised')
-            except InvalidRepository as e:
-                return e
+            return cm.exception
 
         e = try_init('')
         self.assertEqual('"(default)" is not readable or not a Git '
@@ -810,17 +799,16 @@ from :17
         FILE = Node.FILE
 
         cset = repos.get_changeset(rev1)
-        self.assertEqual(set([('0100644',     FILE, ADD, None, None),
-                              ('0100644.txt', FILE, ADD, None, None),
-                              (':100644',     FILE, ADD, None, None),
-                              (':100644.txt', FILE, ADD, None, None),
-                              ('a100644',     FILE, ADD, None, None),
-                              ('a100644.txt', FILE, ADD, None, None)
-                             ]),
+        self.assertEqual({('0100644',     FILE, ADD, None, None),
+                          ('0100644.txt', FILE, ADD, None, None),
+                          (':100644',     FILE, ADD, None, None),
+                          (':100644.txt', FILE, ADD, None, None),
+                          ('a100644',     FILE, ADD, None, None),
+                          ('a100644.txt', FILE, ADD, None, None)},
                          set(cset.get_changes()))
 
         cset = repos.get_changeset(rev2)
-        self.assertEqual(set([(':100666', FILE, MOVE, ':100644', rev1)]),
+        self.assertEqual({(':100666', FILE, MOVE, ':100644', rev1)},
                          set(cset.get_changes()))
 
     _data_colon_character_in_filename = """\
@@ -890,8 +878,8 @@ from :4
                           ('sub',         DIRECTORY, ADD, None, None)],
                          sorted(cset1.get_changes()))
         node1 = repos.get_node('sub', rev1)
-        self.assertEqual(None, node1.get_content())
-        self.assertEqual(None, node1.get_content_length())
+        self.assertIsNone(node1.get_content())
+        self.assertIsNone(node1.get_content_length())
         self.assertEqual(DIRECTORY, node1.kind)
         self.assertEqual({'mode': '160000', 'commit': submodule_rev1},
                          node1.get_properties())
@@ -1077,7 +1065,7 @@ from :2
         self.assertEqual(202, rows[0][0])
 
     def _generate_data_many_merges(self, n, timestamp=1400000000):
-        init = """\
+        init = b"""\
 blob
 mark :1
 data 0
@@ -1101,7 +1089,7 @@ from :2
 M 100644 :1 master.txt
 
 """
-        merge = """\
+        merge = b"""\
 commit refs/heads/dev
 mark :%(dev)d
 author Joe <joe@example.com> %(timestamp)d +0000
@@ -1122,7 +1110,7 @@ merge :%(dev)d
 M 100644 :1 dev%(dev)08d.txt
 
 """
-        data = StringIO()
+        data = io.BytesIO()
         data.write(init % {'timestamp': timestamp})
         for idx in xrange(n):
             data.write(merge % {'timestamp': timestamp,
@@ -1177,11 +1165,11 @@ data 13
 ref-%(ref)08d
 from :2
 """
-        data = StringIO()
-        data.write(root_commit % {'timestamp': timestamp})
+        data = io.BytesIO()
+        data.write(to_utf8(root_commit % {'timestamp': timestamp}))
         for idx in xrange(n):
-            data.write(ref_commit % {'timestamp': timestamp + idx,
-                                     'mark': idx + 3, 'ref': idx})
+            data.write(to_utf8(ref_commit % {'timestamp': timestamp + idx,
+                                             'mark': idx + 3, 'ref': idx}))
         return data.getvalue()
 
 
@@ -1189,7 +1177,7 @@ class GitwebProjectsRepositoryProviderTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        self.projects_base = os.path.realpath(tempfile.mkdtemp())
+        self.projects_base = mkdtemp()
         self.projects_list = os.path.join(self.projects_base, 'projects_list')
         with open(self.projects_list, 'w') as f:
             f.write("""
@@ -1253,7 +1241,7 @@ class GitConnectorTestCase(BaseTestCase):
 
     def _git_version_from_system_info(self):
         git_version = None
-        for name, version in self.env.get_systeminfo():
+        for name, version in self.env.system_info:
             if name == 'GIT':
                 git_version = version
         return git_version
